

package cimeroEditor;

import it.eng.cimero.plugin.preferences.PreferenceConstants;
import it.eng.serviceregistry.IServiceRegistry;
import it.eng.serviceregistry.JAXRServiceRegistryImpl;
import it.eng.spagosoa.metadb.api.IMetadbApi;
import it.eng.spagosoa.metadb.impl.HibernateMetadbImpl;
import it.eng.spagosoa.metadb.model.ServiceAssembly;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.xml.registry.infomodel.ClassificationScheme;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * The main plugin class to be used in the desktop.
 * This classes is create by Eclipse and using
 * to manage cycle life of this plugin
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class CimeroEditorPlugin extends AbstractUIPlugin implements IPropertyChangeListener {


	private IServiceRegistry aServiceRegistry = null;
	/** The Cimero Plugin id. */
	public static String pluginId = "com.bull.cimero.pluginEditor";

	/** The shared instance. */
	private static CimeroEditorPlugin plugin;


	private IMetadbApi metadb = null;
	
	/** The constructor. */
	public CimeroEditorPlugin() {
		plugin = this;
		
	}

	/**
	 * This method is called upon plug-in activation.
	 * @param context  : the context
	 * @throws Exception : if error
	 */
	public final void start(final BundleContext context) throws Exception {
		super.start(context);
		
		getPluginPreferences().addPropertyChangeListener(this);
		
		IPath path = getStateLocation();
		System.out.println(" Configuration file for Spagic Studio are located to ["+path.toOSString()+"]");
		
		String cfgDirPath = path.toOSString() + File.separator + "cfg"+ File.separator;
		File cfgDirectory = new File(cfgDirPath);
		if (!cfgDirectory.exists())
			cfgDirectory.mkdirs();
		
		File customComponentsFile = new File(cfgDirPath + "CustomComponents.xml");
		File dataSourceFile = new File(cfgDirPath + "DataSources.xml");
		File hl7MapFile = new File(cfgDirPath + "hl7map.xml");
		
		if (!customComponentsFile.exists()) 
			copyInCfgDirectory(customComponentsFile, "/ressources/CustomComponents.xml");
		if (!dataSourceFile.exists()) 
			copyInCfgDirectory(dataSourceFile, "/ressources/DataSources.xml");
		if (!hl7MapFile.exists()) 
			copyInCfgDirectory(hl7MapFile, "/ressources/hl7map.xml");
	}
	
	
	
	public URL getResourceURL(String path) throws IOException {
		if (pluginId == null || path == null)
    		throw new IllegalArgumentException();
		Bundle bundle = Platform.getBundle(pluginId);
		URL url = FileLocator.find(bundle, new Path(path), null);
		return FileLocator.resolve(url);
	}
	        
	public void copyInCfgDirectory(File destination, String bundleRelatedPath){
		BufferedReader sourceReader = null;
		BufferedWriter bw = null;
		try{
		 sourceReader = new BufferedReader (new InputStreamReader(getResourceURL(bundleRelatedPath).openStream()));
		 bw = new BufferedWriter(new FileWriter(destination));
		 String line = null;
		 while ( (line = sourceReader.readLine()) != null){
			 bw.write(line);
			 bw.write("\n");
			 bw.flush();
		 }
		}catch (Exception e) {
			e.printStackTrace();
		}finally{
			try{
				sourceReader.close();
				bw.close();
			}catch (Exception e) {
				e.printStackTrace();
			}
			
		}
	}


	/**
	 * This method is called when the plug-in is stopped.
	 * @param context : the context
	 * @throws Exception : if error
	 */
	public final void stop(final BundleContext context)
					throws Exception {
		super.stop(context);
		plugin = null;
	}

	/**
	 * Returns the shared instance.
	 * @return CimeroEditorPlugin : the plugin
	 */
	public static CimeroEditorPlugin getDefault() {
		if (plugin == null) {
			plugin = new CimeroEditorPlugin();
		}
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path.
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(final String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(pluginId, path);
	}


	/**
	 * Returns the Cimero plugin id.
	 * @return pluginId
	 */
	public static final String getPluginId() {
		return pluginId;
	}

	/**
	 * return the full path of jar file.
	 * @param path : the relative path of the file
	 * @return the fullpath.
	 * @deprecated Platform deprecated use FileLocator
	 * with eclipse 3.2 ; not 3.1
	 
	public static String getRessourcePath(final String path) {
        if (pluginId == null || path == null) {
            throw new IllegalArgumentException();
        }
        URL url = null;
        //we use Platform whereas it's depreciated in eclipse 3.2
        //but FileLocator is not allowed into eclipse 3.1
        Bundle bundle = Platform.getBundle(pluginId);
        url = FileLocator.find(bundle, new Path(path), null);
        //url = Platform.find(bundle, new Path(path), null);
        try {
        	URL fullPath = FileLocator.resolve(url);
			return fullPath.toString();
		} catch (IOException e) {
			CimeroLog.errorLog("Error in retrieving path"+path , e);
			return null;
		}
    }
    */
	
	public static String getRessourcePath(final String pathResource) {
		
		if (pathResource.startsWith("/ressources") || pathResource.startsWith("/resources")){
			IPath path = CimeroEditorPlugin.getDefault().getStateLocation();
			System.out.println(" Configuration file for Spagic Studio are located to ["+path.toOSString()+"]");
			
			String cfgDirPath = path.toOSString() + File.separator + "cfg"+ File.separator;
        	String resourceName = pathResource.substring(pathResource.lastIndexOf("/") + 1);
        	return "file:"+ cfgDirPath + resourceName;
        }else{
        	// Il Vecchio codice
        	if (pluginId == null || pathResource == null) {
        		throw new IllegalArgumentException();
        	}
        	URL url = null;
        	//we use Platform whereas it's depreciated in eclipse 3.2
        	//but FileLocator is not allowed into eclipse 3.1
        	Bundle bundle = Platform.getBundle(pluginId);
        	url = FileLocator.find(bundle, new Path(pathResource), null);
        	//url = Platform.find(bundle, new Path(path), null);
        	try {
        		URL fullPath = FileLocator.resolve(url);
        		return fullPath.toString();
        	} catch (IOException e) {
        		CimeroLog.errorLog("Error in retrieving path"+pathResource , e);
        		return null;
        	}
        }
    }
	

	private  SessionFactory getSessionFactory(){
		Configuration cfg = new Configuration().configure("/hibernate.cfg.xml");
		org.eclipse.core.runtime.Preferences pref = getPluginPreferences();
		
		String dbUrl = pref.getString(PreferenceConstants.P_AUDIT_DB_URL);
		String dbDriver = pref.getString(PreferenceConstants.P_AUDIT_DB_DRIVER);
		String dbUser = pref.getString(PreferenceConstants.P_AUDIT_DB_USER);
		String dbPassword = pref.getString(PreferenceConstants.P_AUDIT_DB_PASSWORD);
		String dbDialect = pref.getString(PreferenceConstants.P_AUDIT_DB_HIB_DIALECT);
		String dbShowSql = pref.getString(PreferenceConstants.P_AUDIT_DB_SHOW_SQL);
		
        cfg.setProperty("hibernate.connection.driver_class", dbDriver);
        cfg.setProperty("hibernate.connection.url",dbUrl);
        cfg.setProperty("hibernate.connection.username",dbUser);
        cfg.setProperty("hibernate.connection.password",dbPassword);
        
        cfg.setProperty("hibernate.dialect",dbDialect);
        cfg.setProperty("hibernate.show_sql",dbShowSql);
        
		return cfg.buildSessionFactory();
	}
	
	public void initMetadbApi(){
		this.metadb = new HibernateMetadbImpl(getSessionFactory());
	}
	
	public void initServiceRegistry(){
		org.eclipse.core.runtime.Preferences pref = getPluginPreferences();
		Properties properties = new Properties();
		properties.setProperty("javax.xml.registry.queryManagerURL", pref.getString(PreferenceConstants.P_SERVICE_REGISTRY_INQUIRY_URL));
		properties.setProperty("javax.xml.registry.lifeCycleManagerURL", pref.getString(PreferenceConstants.P_SERVICE_REGISTRY_PUBLISH_URL));
		properties.setProperty("registry.username", pref.getString(PreferenceConstants.P_SERVICE_REGISTRY_USER));
		properties.setProperty("registry.password", pref.getString(PreferenceConstants.P_SERVICE_REGISTRY_PASSWORD));
		properties.setProperty("metadb.db.driver", pref.getString(PreferenceConstants.P_AUDIT_DB_DRIVER));
		properties.setProperty("metadb.db.user",pref.getString(PreferenceConstants.P_AUDIT_DB_USER) );
		properties.setProperty("metadb.db.password",pref.getString(PreferenceConstants.P_AUDIT_DB_PASSWORD));
		properties.setProperty("metadb.db.url", pref.getString(PreferenceConstants.P_AUDIT_DB_URL));
		try{
			aServiceRegistry = new JAXRServiceRegistryImpl(properties);
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	

	public IMetadbApi getMetadbApi(){
		if (this.metadb == null)
			initMetadbApi();
		return metadb;
	}
	
	public IServiceRegistry getServiceRegistry(){
		if (this.aServiceRegistry == null){
			initServiceRegistry();
		}
		return aServiceRegistry;
	}



	

	public List<String> getTaxonomies() {
		List<String> l = new ArrayList<String>();
		IServiceRegistry sevRegistry = null;
		try{
			sevRegistry = getServiceRegistry();
			sevRegistry.initRegistry();
			List<ClassificationScheme> lTaxs= getServiceRegistry().getAllClassificationScheme();
			for (ClassificationScheme tax : lTaxs){
				l.add(tax.getName().getValue());
			}
		}catch(Exception e){
			e.printStackTrace();
			CimeroLog.errorLog(
					"Error getting the list of taxonomies"
					, e);
		}finally{
			sevRegistry.finalizeRegistry();
		}
		return l;
		
	}
	
	public List<String> getTaxonomyValues(String taxonomyName){
		
		List<String> l = new ArrayList<String>();
		IServiceRegistry sevRegistry = null;
		try{
			sevRegistry = getServiceRegistry();
			sevRegistry.initRegistry();
			l = sevRegistry.getClassificationSchemeValues(taxonomyName);
		}catch(Exception e){
			e.printStackTrace();
			CimeroLog.errorLog(
					"Error getting values for taxonomy named["+taxonomyName+"]"
					, e);
		}finally{
			sevRegistry.finalizeRegistry();
		}
		return l;
			
		
	
	}
	
	public List<String> getOrganizations() {
		
		List<String> l = new ArrayList<String>();
		IServiceRegistry sevRegistry = null;
		try{
			sevRegistry = getServiceRegistry();
			sevRegistry.initRegistry();
			List<Organization> lOrgs=sevRegistry.getAllOrganizations();
			for (Organization org : lOrgs){
				l.add(org.getName().getValue());
			}
		}catch(Exception e){
			e.printStackTrace();
			CimeroLog.errorLog(
					"Error getting the list of organizations"
					, e);
		}finally{
			sevRegistry.finalizeRegistry();
		}
		return l;
	
	}
	
	public List<String> getClassificationForService(ServiceAssembly sa) {
		List<String> l = new ArrayList<String>();
		IServiceRegistry sevRegistry = null;
		try{
			sevRegistry = getServiceRegistry();
			sevRegistry.initRegistry();
			Organization org = sevRegistry.getOrganizationByName(sa.getUddiOrg());
		
			Service service = sevRegistry.getServiceByOrganizationAndServiceName(org, sa.getUddiName());
		
			return sevRegistry.getAllClassificationsForService(org, service);
		}catch(Exception e){
			e.printStackTrace();
			CimeroLog.errorLog(
				"Error getting the list of organizations"
				, e);
		}finally{
			sevRegistry.finalizeRegistry();
		}
		return l;
	}
	
	
	
	public void removeServiceFromServiceRegistry(ServiceAssembly sa) {
		
		IServiceRegistry sevRegistry = null;
		IMetadbApi metadbApi = getMetadbApi();
		try{
			metadbApi.openTx();
			sevRegistry = getServiceRegistry();
			sevRegistry.initRegistry();
			Organization org = sevRegistry.getOrganizationByName(sa.getUddiOrg());
			Service service = sevRegistry.getServiceByOrganizationAndServiceName(org, sa.getUddiName());
			sa.setUddiName(null);
			sa.setUddiOrg(null);
			sevRegistry.deleteService(org, service);
			metadbApi.commitAndClose();
		}catch(Throwable e){
			metadbApi.rollBackAndClose();
			
		}finally{
			sevRegistry.finalizeRegistry();
		}
	
	}

	

	public void propertyChange(org.eclipse.core.runtime.Preferences.PropertyChangeEvent event) {
		
		initMetadbApi();
		initServiceRegistry();
		
	}

}
