package com.bull.cimero.pluginEditor.editors;

import it.eng.cimero.plugin.dialogs.ListRulesForEndpointDialog;
import it.eng.spagosoa.metadb.api.IMetadbApi;
import it.eng.spagosoa.metadb.model.Endpoint;
import it.eng.spagosoa.metadb.model.ServiceAssembly;
import it.eng.utils.ProcessModel;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.dnd.TemplateTransferDragSourceListener;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.ZoomInAction;
import org.eclipse.gef.ui.actions.ZoomOutAction;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.gef.ui.parts.GraphicalEditorWithPalette;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.part.PluginTransfer;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.figure.GeneriqueCimeroFigure;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.palette.PaletteRootCimero;
import com.bull.cimero.pluginEditor.editors.parts.DiagramPart;
import com.bull.cimero.pluginEditor.editors.parts.GeneriqueCimeroPart;
import com.bull.cimero.pluginEditor.editors.parts.PartFactory;
import com.bull.cimero.pluginEditor.editors.partsOutline.TreePartFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.jarGenerator.JarGenerator;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardHttpClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJBIClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.ServiceAssemblyClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.ServiceUnitClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.StandardJBIJARClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.xmlGenerator.XmlGenerator;
import com.bull.cimero.pluginEditor.utils.listener.ResourceTracker;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.zoom.DelegatingZoomManager;
import com.bull.cimero.pluginEditor.validator.Validator;

/**
 * This class is the main classe of the Cimero Editor.
 * It's used to load the select file and draw his representation
 * with the Cimero pattern.
 * This class generate all the GEF components and the Palette.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class DiagramEditor extends GraphicalEditorWithPalette {

	
	
	/** delete code key. */
	private static final int DELETEKEY = 127;
	/** the paletteroot. */
	private PaletteRoot paletteRoot;
	/** the key listener. */
	private KeyHandler sharedKeyHandler;
	/** boolean. */
	private boolean savePreviouslyNeeded;
	/** the diagram. */
	private Diagram diagram = new Diagram();
	/** the delegating ZoomManager. */
	private DelegatingZoomManager delegatingZoomManager;
	/** boolean to know if errors are printed. */
	private boolean bShowErrors;
	/**The IFile listener.*/
	private ResourceTracker resourceListener = null;

	/** corresponding iFile. */
	private IFile iFile;
	
	private IFile wsdlFile;

	/**
	 * The drop target used to provide the drop support
	 * on the DiagramEditor.
	 */
	private DropTarget target = null;

	/**
	 * default constructor.
	 */
	public DiagramEditor() {
		// an EditDomain is a "session" of editing which contains things
		// like the CommandStack
		setEditDomain(new DefaultEditDomain(this));
		paletteRoot = null;
		//create the Ifile listener
		resourceListener = new ResourceTracker(this);
		//send an log to make the start
		//CimeroLog.infoLog("Starting Cimero Editor");
		setIFile(null);
	}


	/**
	 * Adds an action to this editor's <code>ActionRegistry</code>.
	 * (This is a helper method.)
	 * @param action the action to add.
	 */
	protected final void addAction(final IAction action) {
		getActionRegistry().registerAction(action);
	}



	/**
	 * method used to add a drop target listener on the drop target.
	 * @param dropTargetListener the DropTargetListener to add
	 */
	final void addDropTargetListener(
			final DropTargetListener dropTargetListener) {
//		use the package visibility
		if (getGraphicalViewer() != null) {
			// until the graphical Viewer isn't configured
			//we can't add a listner on it.
			if (target == null) {

				//
				// Add drag/drop support
				//
				int ops = DND.DROP_COPY | DND.DROP_MOVE;
				Transfer[] transfers = new Transfer[] {
						PluginTransfer.getInstance() };
				try {
					target = new DropTarget(getGraphicalViewer().getControl()
							, ops);
					target.setTransfer(transfers);
				} catch (Exception e) {
					return;
				}
			}
			if (target != null) {
				target.addDropListener(dropTargetListener);
			}
		}
	}

	/**
	 * close the editor.
	 * @param save : force or not the closing.
	 */
	public final void closeEditor(final boolean save) {
		getSite().getPage().closeEditor(DiagramEditor.this, save);
	}

	/** commandStackChanged.
	 * @param event : the event
	 */
	@Override
	public final void commandStackChanged(final EventObject event) {
		if (isDirty()) {
			if (!this.savePreviouslyNeeded) {
				this.savePreviouslyNeeded = true;
				firePropertyChange(IEditorPart.PROP_DIRTY);
			}
		} else {
			savePreviouslyNeeded = false;
			firePropertyChange(IEditorPart.PROP_DIRTY);
		}
		super.commandStackChanged(event);
	}

	/**
	 * method to configure the graphical viewer with the current diagram.
	 */
	@Override
	protected final void configureGraphicalViewer() {
		super.configureGraphicalViewer();
		getGraphicalViewer().setRootEditPart(
				new ScalableFreeformRootEditPart());
		//set the factory to use for creating
		//EditParts for elements in the model
		getGraphicalViewer().setEditPartFactory(new PartFactory(diagram, this));

		// Now the graphical viewer is configured
		// we get the listners on it from the registry
		List<DropTargetListener> dtlList =
			CimeroRegistry.getInstance().getDropTargetListenerList();
		for (Iterator<DropTargetListener> i = dtlList.iterator();
		i.hasNext();) {
			addDropTargetListener(i.next());
		}
	}


	/**
	 * method used to create actions.
	 */
	@Override
	protected final void createActions() {
		super.createActions();
		IAction zoomIn = new ZoomInAction(getDelegatingZoomManager());
		IAction zoomOut = new ZoomOutAction(getDelegatingZoomManager());
		addAction(zoomIn);
		addAction(zoomOut);
		getSite().getKeyBindingService().registerAction(zoomIn);
		getSite().getKeyBindingService().registerAction(zoomOut);
	}

	/**
	 * method called when the diagramEditor is closed.
	 * @see org.eclipse.gef.ui.parts.GraphicalEditor#dispose()
	 */
	@Override
	public final void dispose() {
		super.dispose();
		//we remove this diagramEditor from the registry
		CimeroRegistry.getInstance().remove(this);
	}

	/**
	 * method to save the current diagram.
	 * @param monitor : the monitor
	 */
	@Override
	public final void doSave(final IProgressMonitor monitor) {
		try {
			IFile file = ((IFileEditorInput) getEditorInput()).getFile();
			IProject project = file.getProject();
			System.out.println(" Saved file ["+file.getName()+"] in Project ["+project.getName()+"]");
			file.setContents(this.diagram.getAsStream(), true, false, monitor);
			getCommandStack().markSaveLocation();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * method to save the current diagram.
	 */
	@Override
	public final void doSaveAs() {
		SaveAsDialog dialog = null;
		dialog = new SaveAsDialog(getSite().getWorkbenchWindow().getShell());
		dialog.setOriginalFile(((IFileEditorInput) getEditorInput()).getFile());
		dialog.open();
		IPath path = dialog.getResult();

		if (path == null) {
			return;
		}

		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		final IFile file = workspace.getRoot().getFile(path);

		WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
			public void execute(final IProgressMonitor monitor)
			throws CoreException {
				try {
					file.create(diagram.getAsStream(), true, monitor);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		};

		try {
			new ProgressMonitorDialog(getSite().getWorkbenchWindow()
					.getShell()).run(false, true, op);
			setInput(new FileEditorInput((IFile) file));
			getCommandStack().markSaveLocation();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * method used to generate jar package associate to the model.
	 * @return boolean : true if no error
	 * @throws IOException : if user doesn't want to replace existing file
	 */
	public final boolean generateJar() throws IOException {
		IFile file = ((IFileEditorInput) getEditorInput()).getFile();
		JarGenerator generator = new JarGenerator(
				getDiagram(), file);
		//before generating, we valid the model
		if (!validate()) {
			MessageBox msg = new MessageBox(new Shell(), SWT.YES | SWT.NO);
			msg.setMessage("This file is not valid ! "
					+ "Are you sure you want to continue ? ");
			msg.setText("Confirmation");
			if (msg.open() != SWT.YES) { //YES BUTTON
				return false;
			}
			//we force the generation
			generator.setForce(true);
		}
		boolean res;
		try {
			res = generator.generateConfiguration();
		} catch (IOException ioe) {
			//file already exists and user doesn't want to replace it
			throw new IOException();
		} catch (Exception e) {
			CimeroLog.errorLog("", e);
			res = false;
		}
		return res;
	}
	
	// We must verify if the service assembly is already in database
	public ProcessModel getProcessModel(){
		IFile file = ((IFileEditorInput) getEditorInput()).getFile();
		JarGenerator generator = new JarGenerator(
				getDiagram(), file);
		
		IMetadbApi metadb = CimeroEditorPlugin.getDefault().getMetadbApi();
		ServiceAssemblyClass smModel = (ServiceAssemblyClass) generator.generateModel();
		try{
			metadb.open();
			
			ProcessModel currentProcessModel = generator.inferProcessModelFromSpagicStudio(smModel);
			currentProcessModel.calculateStatus(metadb);;
			return currentProcessModel;
		}catch(Throwable t){
			t.printStackTrace();
			return null;
		}finally{
			metadb.close();
		}
		
	}
	
	/**
	 * method used to generate jar package associate to the model.
	 * @return boolean : true if no error
	 * @throws IOException : if user doesn't want to replace existing file
	 */
	public final boolean storeProcessModel(ProcessModel aProcessModel) throws IOException {
		IFile file = ((IFileEditorInput) getEditorInput()).getFile();
		JarGenerator generator = new JarGenerator(
				getDiagram(), file);
		
		boolean res;
		try {
			res = generator.storeProcessModel(aProcessModel);
		} catch (IOException ioe) {
			//file already exists and user doesn't want to replace it
			throw new IOException();
		} catch (Exception e) {
			CimeroLog.errorLog("", e);
			res = false;
		}
		return res;
	}
	
	/**
	 * method used to generate jar package associate to the model.
	 * @return boolean : true if no error
	 * @throws IOException : if user doesn't want to replace existing file
	 */
	public final ServiceAssemblyClass getServiceAssemblyClassModel() throws IOException {
		IFile file = ((IFileEditorInput) getEditorInput()).getFile();
		JarGenerator generator = new JarGenerator(
				getDiagram(), file);
		return (ServiceAssemblyClass)generator.generateModel();
	}
	
	public final ServiceAssembly isSARegisteredInDatabase() throws IOException {
		IFile file = ((IFileEditorInput) getEditorInput()).getFile();
		JarGenerator generator = new JarGenerator(
				getDiagram(), file);
		ServiceAssembly saRes = null;
		try {
			saRes = generator.isSARegisteredInDatabase();
		} catch (IOException ioe) {
			//file already exists and user doesn't want to replace it
			throw new IOException();
		} catch (Exception e) {
			CimeroLog.errorLog("", e);
		}
		return saRes;
	}
	
	

	public void configureRulesForEndpoint(){
		IMetadbApi metadb = CimeroEditorPlugin.getDefault().getMetadbApi();
		String currentEnpointName = getDiagram().getCurrentSelectedNode();
		
		IFile file = ((IFileEditorInput) getEditorInput()).getFile();
		JarGenerator generator = new JarGenerator(
				getDiagram(), file);
		
			ServiceAssemblyClass smModel = (ServiceAssemblyClass) generator.generateModel();
			if (smModel != null){
				metadb.open();
				ServiceAssembly sa = metadb.getServiceAssemblyByName(smModel.getServiceAssemblyName());
				
				if (sa == null){
					MessageDialog.openError(getGraphicalViewer().getControl().getShell(), "Cimero Editor", "This service assembly is not published in db cannot configure rules");
					metadb.close();
					return;
				}else{
					Endpoint ep = metadb.getEndpointByName(sa, currentEnpointName);
					metadb.close();
					ListRulesForEndpointDialog lrulesDialog = new ListRulesForEndpointDialog(getGraphicalViewer().getControl().getShell(), ep, CimeroEditorPlugin.getDefault().getMetadbApi());
					lrulesDialog.open();
					return;
				}	
					
			}
	
		
		
	}
	/**
	 * method used to generate xml file associate to the model.
	 * @return boolean : true if no error
	 * @throws IOException : if user doesn't want to replace existing file
	 */
	public final boolean generateXml() throws IOException {
		IFile file = ((IFileEditorInput) getEditorInput()).getFile();
		XmlGenerator generator = new XmlGenerator(
				getDiagram(), file);
		//before generating, we valid the model
		if (!validate()) {
			MessageBox msg = new MessageBox(new Shell(), SWT.YES | SWT.NO);
			msg.setMessage("This file is not valid ! "
					+ "Are you sure you want to continue ? ");
			msg.setText("Confirmation");
			if (msg.open() != SWT.YES) { //YES BUTTON
				return false;
			}
			//we force the generation
			generator.setForce(true);
		}
		boolean res;
		try {
			res = generator.generateConfiguration();
		} catch (IOException ioe) {
			//file already exists and user doesn't want to replace it
			throw new IOException();
		} catch (Exception e) {
			//error occured
			res = false;
		}
		return res;
	}

	/**
	 * method used to get an adapter.
	 * @param type : the type of the wanted object
	 * @return Object wanted
	 */
	@Override
	public final Object getAdapter(final Class type) {
		if (type == IContentOutlinePage.class) {
			return new OutlinePage();
		} else {
			if (type == ZoomManager.class) {
				return getDelegatingZoomManager();
			}
		}
		return super.getAdapter(type);
	}

	/**
	 * Cette fonction permet de specifier les fonctions
	 * a appeler lors de la saisie d'une touche par l'utilisateur.
	 * @return KeyHandler
	 */
	protected final KeyHandler getCommonKeyHandler() {
		if (sharedKeyHandler == null) {
			sharedKeyHandler = new KeyHandler();
			sharedKeyHandler.put(
					KeyStroke.getPressed(SWT.DEL, DELETEKEY, 0),
					getActionRegistry().getAction(ActionFactory.DELETE));
		}
		return sharedKeyHandler;
	}

	/**
	 * Returns the <code>DelegatingZoomManager</code> for this editor.
	 * @return the <code>DelegatingZoomManager</code>
	 */
	public final DelegatingZoomManager getDelegatingZoomManager() {
		//create if not create
		if ((null == delegatingZoomManager)) {
			delegatingZoomManager = new DelegatingZoomManager();
		}
		//set the zoom manager when the graphicalviewer is not null
		if (null != getGraphicalViewer()) {
			delegatingZoomManager.setCurrentZoomManager(
					getZoomManager(getGraphicalViewer()));
		}
		return delegatingZoomManager;
	}

	/**
	 * get the current diagram.
	 * @return Diagram : the diagram wanted
	 */
	public final Diagram getDiagram() {
		return this.diagram;
	}

	
	/**
	 * get the filename.
	 * @return Returns the fileName.
	 */
	public final String getFileName() {
		if (getIFile() != null) {
			return getIFile().getName();
		} else {
			return "";
		}
	}

	/**
	 * @param model : the component using to retreive predecesor.
	* @return predecesor component of the component in parameter.
	*
	*/
	private List<GeneriqueCimeroModel> getFirstPredecessor(
			final GeneriqueCimeroModel model) {
		List < GeneriqueCimeroModel > lst =
			new ArrayList<GeneriqueCimeroModel>();
		if (model != null) {
			for (int i = 0; i < model.getIncomingConnections().size(); i++) {
				GeneriqueCimeroModel eltLst =
					((Connection) model.getIncomingConnections()
							.get(i)).getSource();
				lst.add(eltLst);
			}
		}
		return lst;
	}

	/**
	 * get the ifile.
	 * @return Returns the iFile.
	 */
	public final IFile getIFile() {
		return iFile;
	}

	/**
	 * method to get the palette root.
	 * @return PaletteRoot
	 */
	@Override
	public final PaletteRoot getPaletteRoot() {
		if (this.paletteRoot == null) {
			this.paletteRoot = PaletteRootCimero.createPalette();
		}
		return this.paletteRoot;
	}

	/**
	 * method used to get the parts.
	 * @return List : list of the parts
	 */
	public final List getParts() {
		return getGraphicalViewer().getRootEditPart().getChildren();
	}

	/**
	 * Returns the zoom manager of the specified viewer.
	 * @param viewer the viewer to get the zoom manager from
	 * @return the zoom manager
	 */
	private ZoomManager getZoomManager(final GraphicalViewer viewer) {
		// get zoom manager from root edit part
		RootEditPart rootEditPart = viewer.getRootEditPart();
		ZoomManager zoomManager = null;
		if (rootEditPart instanceof ScalableFreeformRootEditPart) {
			zoomManager =
				((ScalableFreeformRootEditPart) rootEditPart).getZoomManager();
		} else {
			if (rootEditPart instanceof ScalableRootEditPart) {
				zoomManager =
					((ScalableRootEditPart) rootEditPart).getZoomManager();
			}
		}
		return zoomManager;
	}

	/**
	 * method called when editor is init (open).
	 * @see org.eclipse.gef.ui.parts.GraphicalEditor
	 * #init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 * @param site : the IEditorSite
	 * @param input : the input IFile.
	 * @throws PartInitException exception if part doesn't exist.
	 */
	@Override
	public final void init(final IEditorSite site, final IEditorInput input)
	throws PartInitException {
		super.init(site, input);
		//we call function to show needed views
		showNeededViews();
	}

	/**
	 * init of the graphical viewer.
	 */
	@Override
	public final void initializeGraphicalViewer() {
		// this uses the PartFactory set in configureGraphicalViewer
		// to create an EditPart for the diagram and sets it as the
		// content for the viewer
		getGraphicalViewer().setContents(this.diagram);
	}

	/**
	 * init of the palette viewer.
	 */
	@Override
	protected final void initializePaletteViewer() {
		super.initializePaletteViewer();
		getPaletteViewer().addDragSourceListener(
				new TemplateTransferDragSourceListener(getPaletteViewer()));
	}

	/**
	 * @return Returns the bShowErrors.
	 */
	public final boolean isBShowErrors() {
		return bShowErrors;
	}

	/**
	 * method to test the dirty.
	 * @return boolean
	 */
	@Override
	public final boolean isDirty() {
		// rely on the command stack to determine dirty flag
		return getCommandStack().isDirty();
	}

	/**
	 * method to test if saving is allowed.
	 * @return boolean
	 */
	@Override
	public final boolean isSaveAsAllowed() {
		//allow Save As
		return true;
	}

	/**
	 * @see org.eclipse.ui.part.EditorPart#isSaveOnCloseNeeded()
	 * @return boolean : save needed
	 */
	@Override
	public final boolean isSaveOnCloseNeeded() {
		return super.isSaveOnCloseNeeded();
	}

	/**
	 * This method organise automaticaly the component
	 * drawinin the grpahical editor.
	 *
	 */
	public final void organiseDrawingComponent() {
		Iterator iterator = this.getParts().iterator();
		while (iterator.hasNext()) {
			DiagramPart myDiagPart = ((DiagramPart) iterator.next());

			//copy de la liste de generiqeuCimeroPart
			List <GeneriqueCimeroPart> lstPart =
				new ArrayList<GeneriqueCimeroPart>(myDiagPart.getChildren());
			int boucle = 0;
			int posX = 0;
			int posY = 0;

			//
			// set the X position component.
			//
			//on boucle tant que les lments ont pas t tous organis.
			while (lstPart.size() != 0) {
				//table with all generique cimero part
				Object [] tabPart = lstPart.toArray();
				List < GeneriqueCimeroPart > partToOrg =
					new ArrayList<GeneriqueCimeroPart>();
				//organise the element of this diagram part.
				for (int i = 0; i < tabPart.length; i++) {
					GeneriqueCimeroPart myCompo =
						(GeneriqueCimeroPart) tabPart[i];
					List < GeneriqueCimeroModel > lstTmp =
						((GeneriqueCimeroModel) myCompo.getModel())
						.researchInput((GeneriqueCimeroModel) myCompo.getModel());
					if (lstTmp.size() <= boucle) {
						//ajout du composant a organiser
						partToOrg.add(myCompo);
						//retire le composant de la liste gnrale
						lstPart.remove(myCompo);
					}
				}
				boucle++;
				if (partToOrg.size() != 0) {
					//we organise this list.
					Iterator < GeneriqueCimeroPart > iteratorLst =
						partToOrg.iterator();

					posX += 50;
					posY = 50;
					int maxWidth = 0;
					while (iteratorLst.hasNext()) {
						GeneriqueCimeroPart myPart = iteratorLst.next();
						GeneriqueCimeroModel myModel =
							(GeneriqueCimeroModel) myPart.getModel();
						myModel.setLocation(new Point(posX , posY));
						posY += 2 * ((GeneriqueCimeroFigure) myPart
								.getFigure()).getFigureDimension().height;
						if (maxWidth < ((GeneriqueCimeroFigure)
							   myPart.getFigure()).getFigureDimension().width) {
							maxWidth = ((GeneriqueCimeroFigure)
								myPart.getFigure()).getFigureDimension().width;
						}
					}
					posX += 2 * maxWidth;
				}
			}
			int savBoucle = boucle;
			boolean modification;
			do {
				modification = false;
				boucle = savBoucle;
				//
				// set the Y position component.
				//
				//on parcours les lments dans le sens inverse.
				lstPart.clear();
				lstPart = new ArrayList<GeneriqueCimeroPart>(
						myDiagPart.getChildren());
				while (lstPart.size() != 0) {
					Object [] tabPart = lstPart.toArray();
					List < GeneriqueCimeroPart > partToOrg =
						new ArrayList<GeneriqueCimeroPart>();
					for (int i = 0; i < tabPart.length; i++) {
						GeneriqueCimeroPart myCompo =
							(GeneriqueCimeroPart) tabPart[i];
						List < GeneriqueCimeroModel > lstTmp =
							((GeneriqueCimeroModel) myCompo.getModel())
							.researchInput((GeneriqueCimeroModel)
									myCompo.getModel());
						if (lstTmp.size() >= boucle) {
							//ajout du composant a organiser
							partToOrg.add(myCompo);
							//retire le composant de la liste gnrale
							lstPart.remove(myCompo);
						}
					}
					boucle--;
					//set Y position
					if (partToOrg.size() != 0) {
						//we organise this list.
						Iterator < GeneriqueCimeroPart > iteratorLst =
							partToOrg.iterator();
						while (iteratorLst.hasNext()) {
							GeneriqueCimeroPart myPart = iteratorLst.next();
							//get first predecessor
							List < GeneriqueCimeroModel > firstPred =
								getFirstPredecessor((GeneriqueCimeroModel)
										myPart.getModel());
							int yMoy = 0;
							Iterator < GeneriqueCimeroModel > iteratorModel =
								firstPred.iterator();
							if (firstPred.size() != 0) {
								while (iteratorModel.hasNext()) {
								  yMoy += iteratorModel.next().getLocation().y;
								}
								yMoy = yMoy / firstPred.size();
								GeneriqueCimeroModel myModel =
									(GeneriqueCimeroModel) myPart.getModel();
								// calc new point
								Point myNewP =
									new Point(myModel.getLocation().x, yMoy);
								//test if this point is ok
								while (positionAlreadyUsing(
									myNewP, myModel.getName() ,
									myDiagPart.getChildren())) {
									myNewP.y += 2 * ((GeneriqueCimeroFigure) myPart.getFigure()).getFigureDimension().height;
								}
								//set the new position
								if (myNewP.y != myModel.getLocation().y) {
									modification = true;
									myModel.setLocation(myNewP);
								}
							}
						}
					}
				}
			} while (modification);

		}
	}

	/**
	 * This function return true if MyModel passed in parameter have the
	 * same location like another component.
	 * @param pos : the position to test.
	 * @param modelName : the name of the model to setLocation.
	 * @param components : the list of all components.
	 * @return true if two component have the same position.
	 */
	private boolean positionAlreadyUsing(final Point pos,
			final String modelName,
			final List components) {
		if (components != null) {
			List <GeneriqueCimeroPart> lstPart =
				new ArrayList<GeneriqueCimeroPart>(components);
			Iterator it = lstPart.iterator();
			while (it.hasNext()) {
				GeneriqueCimeroModel modelTmp =
					(GeneriqueCimeroModel) ((GeneriqueCimeroPart)
							it.next()).getModel();
				// si il s'agit du mme lment, on n'effectue pas le test.
				if (modelTmp.getName().compareTo(modelName) != 0) {
					if ((Math.abs(modelTmp.getLocation().y - pos.y ) < 10)
							&& (Math.abs(modelTmp.getLocation().x - pos.x ) < 10)){
						//return true;
						return true;
					}
				}
			}
		}
		return false;
	}


	/**
	 * method to put a String parameter into the title bar.
	 * @param newTitle : the title
	 * @param bExtension : boolean to print or not the extension of the file
	 */
	private void putTitle(final String newTitle, final boolean bExtension) {
		if (bExtension) {
			this.setPartName(newTitle);
		} else {
			//split of the name into 2 parts :
			//the real name and the extension
			this.setPartName(newTitle.split("\\.")[0]);
		}
	}

	/**
	 * method used to get all the part
	 * own by the right diagramEditor.
	 */
	public final void refresh() {
		Iterator iterator = this.getParts().iterator();
		while (iterator.hasNext()) {
			DiagramPart elt = ((DiagramPart) iterator.next());
			refreshDiagramEditPart(elt.getChildren());
		}
	}

	/**
	 * method used to refresh all parts in the
	 * current DiagramEditPart.
	 * @param lstPart : list of the parts
	 */
	private void refreshDiagramEditPart(
			final List lstPart) {
		Iterator iterator = lstPart.iterator();
		while (iterator.hasNext()) {
			GeneriqueCimeroPart elt = ((GeneriqueCimeroPart) iterator.next());
			elt.setBShowErrors(isBShowErrors());
			elt.refreshVisuals();
		}
	}

	/**
	 * method used to reverse showing errors
	 * and to refresh.
	 */
	public final void reverseBShowErrors() {
		//we reverse
		setBShowErrors(!isBShowErrors());
		//and we refresh
		refresh();
	}

	/**
	 * @param showErrorsParam The bShowErrors to set.
	 */
	public final void setBShowErrors(final boolean showErrorsParam) {
		bShowErrors = showErrorsParam;
	}

	/**
	 * set the ifile.
	 * @param file The iFile to set.
	 */
	public final void setIFile(final IFile file) {
		iFile = file;
	}

	/**
	 * set the editor input.
	 * @param input : the input editor
	 */
	@Override
	public final void setInput(final IEditorInput input) {
		IFile file = null;
		if (getEditorInput() != null) {
			file = ((IFileEditorInput) getEditorInput()).getFile();
			file.getWorkspace().removeResourceChangeListener(resourceListener);
		}
		super.setInput(input);
		file =  ((IFileEditorInput) getEditorInput()).getFile();
		file = ((IFileEditorInput) input).getFile();
		try {
			// attempt to read from a file
			InputStream istream = file.getContents(false);
			diagram = Diagram.makeFromStream(istream);

		} catch (Exception e) {
			// but if there's an error, create a new diagram
			CimeroLog.errorLog("Error when loaded the File. "
					+ "The file may be corrupted", e);
			diagram = new Diagram();
		}
		diagram.setCimeroDiagramName(file.getName());
		//call the putTitle function
		//in order to put the name of the file into the title bar
		this.putTitle(file.getName(), true);
		setIFile(file);
		//we call the static refreshAction to set this diagramEditor available
		CimeroRegistry.getInstance().add(this);
		//add the ressource listener
		if (getEditorInput() != null) {
			file.getWorkspace().addResourceChangeListener(resourceListener);
			setPartName(file.getName());
		}
	}

	/**
	 * method to set the palette root.
	 * @param paletteRootParam : the palette to add
	 */
	public final void setPaletteRoot(final PaletteRoot paletteRootParam) {
		this.paletteRoot = paletteRootParam;
	}

	/**
	 * method called to open server and properties views.
	 */
	public final void showNeededViews() {
		//we add the server and properties views and open the last one
		try {
			if (PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage() != null) {
				//first, the server view
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().
				getActivePage().showView(
				"org.eclipse.wst.server.ui.ServersView");
			}
		} catch (Exception e) {
			return;
		}
		try {
			if (PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage() != null) {
				//then the properties one
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().
				getActivePage().showView(
				"org.eclipse.ui.views.PropertySheet");
			}
		} catch (Exception e) {
			return;
		}
	}

	/**
	 * method used to generate xml file associate to the model.
	 * @return boolean : true if valid
	 */
	public final boolean validate() {
		Validator validator = new Validator();
		boolean res = validator.validate(getDiagram(), getFileName());
		//refresh to show or not errors
		refresh();
		return res;
	}

//	-----------------------------------------------------------
	// INNER CLASS
	//-----------------------------------------------------------
	/**
	 * This class represent an instance of the OutLine view.
	 * It permits to add informations
	 * @author Cedric MOUILLERON
	 * @author Pierre NOTEL
	 */
	public class OutlinePage extends ContentOutlinePage {

		/** */
		private PageBook pageBook;
		/** */
		private Control outline;

		/**
		 * Default Constructor.
		 */
		public OutlinePage() {
			super(new TreeViewer());
		}

		/**
		 * method to create controls of the page.
		 * @param parent : the composite parent
		 */
		@Override
		public final void createControl(final Composite parent) {
			pageBook = new PageBook(parent, SWT.NONE);
			outline = getViewer().createControl(pageBook);
			pageBook.showPage(outline);
			getSelectionSynchronizer().addViewer(getViewer());
			getViewer().setEditDomain(getEditDomain());
			getViewer().setEditPartFactory(new TreePartFactory());
			getViewer().setContents(getDiagram());
		}

		/**
		 *
		 */
		@Override
		public final void dispose() {
			getSelectionSynchronizer().removeViewer(getViewer());
			super.dispose();
		}

		/**
		 * method to get the control of the page.
		 * @return Control
		 */
		@Override
		public final Control getControl() {
			return this.pageBook;
		}

		/**
		 * init method.
		 * undo, redo, delete functionnalities
		 * @param pageSite : the current page
		 */
		public final void init(final IPageSite pageSite) {
			super.init(pageSite);
			ActionRegistry registry = getActionRegistry();
			IActionBars bars = pageSite.getActionBars();
			String id = ActionFactory.UNDO.getId();
			bars.setGlobalActionHandler(id, registry.getAction(id));
			id = ActionFactory.REDO.getId();
			bars.setGlobalActionHandler(id, registry.getAction(id));
			id = ActionFactory.DELETE.getId();
			bars.setGlobalActionHandler(id, registry.getAction(id));
			bars.updateActionBars();
		}
	}

	public IFile getWsdlFile() {
		return wsdlFile;
	}


	public void setWsdlFile(IFile wsdlFile) {
		this.wsdlFile = wsdlFile;
	}
	
	public Map<String, Map<String, Object>> getHttpStandardComponentsEndpoints(ServiceAssemblyClass smModel){
		Map<String, Map<String, Object>> httpEndpointMap = new HashMap<String, Map<String, Object>>();
		List <ServiceUnitClass> sus = smModel.getLstSU();

		NameSpaceClass nms = null;
		String endpointNameSpace = null;
		for (ServiceUnitClass su : sus){
			if (su.isStandard()){
				StandardJBIJARClass standardJBI = su.getMonStandard();
				System.out.println(" ======================= StandardJBI =================== ");
				List<StandardJBIClass>  stdJbiComponents = standardJBI.getJbiCompo();
				
				
				for (StandardJBIClass stdJbiClass : stdJbiComponents){
					String serviceName = null;
					String endpoint = null;
					String locationURI = null;
					String service = null;
					if (stdJbiClass instanceof StandardHttpClass) {
						endpoint = ((StandardHttpClass)stdJbiClass).getEndpointValue();
						locationURI = ((StandardHttpClass)stdJbiClass).getLocationURI();
						service = ((StandardHttpClass)stdJbiClass).getServiceName();
						Map<String, Object> epMap = new HashMap<String, Object>();
						epMap.put("locationURI", locationURI);
						epMap.put("service", service);
						epMap.put("jbiObject", stdJbiClass);
						httpEndpointMap.put(endpoint, epMap);
					}
				}
			}
		}
		return  httpEndpointMap;
	}
	
	public void generateWSDLForHttpEndpoint(Shell shell, final IFile fileSelected,String epName, Map<String, Object> epMap){
		final IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		//IContainer container = getFileSelected().getParent();
		
		String selectedFileName = fileSelected.getName();
	
		selectedFileName = selectedFileName.substring(0, selectedFileName.lastIndexOf("."));
		
		final String wsdlFileName = selectedFileName + epName + ".wsdl";
		String saName = selectedFileName;
		String targetNamespace ="http://servicemix.org/cheese/";
		String url = (String)epMap.get("locationURI");
		
		//String portTypeName = saName + epName+ "Port";
		String portTypeName = epName;
		String bindingName = saName+ epName +  "Binding";
		//String serviceName = saName+ epName + "Service";
		String serviceName = (String)epMap.get("service");
		
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		sb.append("<wsdl:definitions targetNamespace=\""+targetNamespace+"\"\n"); 
		sb.append("			xmlns:impl=\""+targetNamespace+"\"\n");
		sb.append("			xmlns:wsdl=\"http://schemas.xmlsoap.org/wsdl/\"\n");
		sb.append("			xmlns:wsdlsoap=\"http://schemas.xmlsoap.org/wsdl/soap/\"\n");
		sb.append("			xmlns:foo=\""+targetNamespace+"\"\n");		
		sb.append("		    xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n");
		
		sb.append("    	<wsdl:types>\n");
		sb.append("	       <schema elementFormDefault=\"qualified\"\n");
	    sb.append("	               targetNamespace=\""+targetNamespace+"\"\n");
	    sb.append("		 		   xmlns=\"http://www.w3.org/2001/XMLSchema\">\n");
		sb.append("		       <element name=\"payload\" type=\"xsd:anyType\" />\n");
		sb.append("			   <element name=\"response\" type=\"xsd:anyType\" />\n");
		sb.append("		    </schema>\n");
		sb.append("	   	</wsdl:types>\n");

		sb.append("	   	<wsdl:message name=\"msgRequest\">\n");
		sb.append("	       	<wsdl:part element=\"impl:payload\" name=\"in\"/>\n");
		sb.append("	   	</wsdl:message>\n");

		sb.append("	   	<wsdl:message name=\"msgResponse\">\n");
		sb.append("			<wsdl:part element=\"impl:response\" name=\"out\"/>\n");
		sb.append("	   	</wsdl:message>\n");
		 
		
		
		
		sb.append("		<wsdl:portType name=\""+portTypeName+"\">\n");
		sb.append("			<wsdl:operation name=\"run\">\n");
		sb.append("				<wsdl:input message=\"impl:msgRequest\" name=\"inRequest\"/>\n");
		sb.append("				<wsdl:output message=\"impl:msgResponse\" name=\"outResponse\"/>\n");
		sb.append("			</wsdl:operation>\n");
		sb.append("		</wsdl:portType>\n");

		sb.append("		<wsdl:binding name=\""+bindingName+"\" type=\"impl:"+portTypeName+"\">\n");
		sb.append("			<wsdlsoap:binding style=\"document\" transport=\"http://schemas.xmlsoap.org/soap/http\"/>\n");
		sb.append("			<wsdl:operation name=\"run\">\n");
		sb.append(" 			<wsdlsoap:operation soapAction=\"\"/>\n");
		sb.append("					<wsdl:input name=\"inRequest\">\n");
		sb.append(" 					<wsdlsoap:body use=\"literal\"/>\n");
		sb.append("					</wsdl:input>");
		sb.append("					<wsdl:output name=\"outResponse\">");
		sb.append("						<wsdlsoap:body use=\"literal\"/>\n");
		sb.append("					</wsdl:output>\n");
		sb.append("			</wsdl:operation>\n");
		sb.append("		</wsdl:binding>\n");

		sb.append("		<wsdl:service name=\""+serviceName+"\">\n");
		sb.append("			<wsdl:port binding=\"impl:"+bindingName+"\" name=\""+portTypeName+"\">\n");
		sb.append("   			<wsdlsoap:address location=\""+url+"\"/>\n");
		sb.append("			</wsdl:port>\n");
		sb.append("		</wsdl:service>\n");

		sb.append("</wsdl:definitions>\n");

		final String wsdlString = sb.toString();
				
		//StandardHttpClass stdHttpClass = (StandardHttpClass)epMap.get("jbiObject");
		
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(final IProgressMonitor monitor)
			throws InvocationTargetException {
				try {
					IProject theSpagicProject = fileSelected.getProject();
					IPath projectPath = theSpagicProject.getFullPath();
					IPath wsdlFilesPath = projectPath.append("WsdlFiles");
					IFolder wsdlFilesPathPathFolder = root.getFolder(wsdlFilesPath);
					createFolderHelper(wsdlFilesPathPathFolder, monitor);
					
					IFile file = wsdlFilesPathPathFolder.getFile(new Path(wsdlFileName));
					ByteArrayInputStream stream = new ByteArrayInputStream(wsdlString.getBytes("UTF-8"));	
//					tests if the file already exist
					if (file.exists()) {
						file.setContents(stream, true, true, monitor);
					} else {
						file.create(stream, true, monitor);
					}
					stream.close();
				} catch (Exception e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		
		try {
			ProgressMonitorDialog dialog = new ProgressMonitorDialog(shell);
			dialog.run(true, true, op);
		} catch (InterruptedException e) {
			return; 			
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();	
			e.printStackTrace();
		}
		
	}
	
	private void createFolderHelper(IFolder folder,IProgressMonitor monitor)
    throws CoreException
 {
    if(!folder.exists())
    {
       IContainer parent = folder.getParent();
       if(parent instanceof IFolder
          && (!((IFolder)parent).exists()))
          createFolderHelper((IFolder)parent,monitor);
       folder.create(false,true,monitor);
    }
 }
}
