package com.bull.cimero.pluginEditor.editors.commands;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * This class is used to create the command to add.
 * an node object into the graphical Editor
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class NodeCreateCommand extends Command {

	/** current diagram. */
	private Diagram diagram;
	/** current node. */
	private GeneriqueCimeroModel node;
	/** current location. */
	private Point location;

   /**
	 * set the diagram.
	 * @param diagramParam the diagram given in parameter
	 */
   public final void setDiagram(final Diagram diagramParam) {
      this.diagram = diagramParam;
   }

   /**
	 * set the node of the connection.
	 * @param nodeParam the node given in parameter
	 */
   public final void setNode(final GeneriqueCimeroModel nodeParam) {
      this.node = nodeParam;
   }

   /**
	 * set the location of the connection.
	 * @param locationParam the location given in parameter
	 */
   public final void setLocation(final Point locationParam) {
      this.location = locationParam;
   }

   /**
    * get the label of the method.
    * @return String : the label
    */
   public final String getLabel() {
      return "Create Node";
   }

   /**
    * execute the method.
    */
   public final void execute() {
      if (this.location != null) {
         this.node.setLocation(this.location);
      }
      try {
		this.diagram.addNode(this.node);
	} catch (Exception e) {
		CimeroLog.errorLog("impossible to add a new element", e);
	}
   }

   /**
    * undo the method.
    */
   public final void undo() {
      this.diagram.removeNode(this.node);
   }

   /**
    * redo the method.
    */
   public final void redo() {
      this.execute();
   }
}
