package com.bull.cimero.pluginEditor.editors.commands;

import org.eclipse.gef.commands.Command;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * This class is used to create the command to delete
 * an object into the graphical editor.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class NodeDeleteCommand extends Command {

	/** current diagram. */
	private Diagram diagram;
	/** current node. */
	private GeneriqueCimeroModel node;

	/**
	 * method to set the diagram.
	 * @param diagramParam : the diagram to set
	 */
	public final void setDiagram(final Diagram diagramParam) {
		this.diagram = diagramParam;
	}

	/**
	 * method to set the node.
	 * @param nodeParam : the node to set
	 */
	public final void setNode(final GeneriqueCimeroModel nodeParam) {
		this.node = nodeParam;
	}

	/**
	 * method to get the label of the class.
	 * @return String : the label
	 */
	public final String getLabel() {
		return "Delete Node";
	}

	/**
	 * this function is called to determine
	 * if the execute function can be apply.
	 * @return boolean
	 */
	public final boolean canExecute() {
		return true;
	}

	/**
	 * execute method.
	 */
	public final void execute() {
		this.diagram.removeNode(node);
	}

	/**
	 * undo method.
	 */
	public final void undo() {
		try {
			this.diagram.addNode(node);
		} catch (Exception e) {
			CimeroLog.errorLog("impossible to add this element", e);
		}
	}

	/**
	 * redo method.
	 */
	public final void redo() {
		this.execute();
	}
}
