package com.bull.cimero.pluginEditor.editors.commands;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;

/**
 * This class is used to move an object in the graphical flow Editor.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class NodeMoveCommand extends Command {

   /** current node. */
   private GeneriqueCimeroModel node;
   /** new position of the node. */
   private Point newPos;
   /** curernt position of the node. */
   private Point oldPos;

   /**
    * To set the actual location.
    * @param pt : new point location
    */
   public final void setLocation(final Point pt) {
      this.newPos = pt;
   }

   /**
    * To set the actual object to move.
    * @param nodeParam : the current node to set
    */
   public final void setNode(final GeneriqueCimeroModel nodeParam) {
      this.node = nodeParam;
   }

   /**
    * This action is called when an object is moved
    * to his new destination.
    */
   @Override
   public final void execute() {
      oldPos = this.node.getLocation();
      this.node.setLocation(newPos);
   }

   /**
    * This is the label show at the user.
    * @return String : the label
    */
   public final String getLabel() {
      return "Move Node";
   }

   /**
    * What we must to do is the user uses the redo fionctionnality.
    */
   @Override
   public final void redo() {
      this.node.setLocation(newPos);
   }

   /**
    * What we must to do if the user uses the undo fonctionnality.
    */
   @Override
   public final void undo() {
      this.node.setLocation(oldPos);
   }
}
