
package com.bull.cimero.pluginEditor.editors.figure;

import org.eclipse.draw2d.AbstractConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionPoint;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * This class is the class wihch permit to add in a figure,
 * an connection point.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class PortConnectionAnchor extends AbstractConnectionAnchor {

	/** fudge factor. */
	private static final int FUDGEFACTOR = 10;

	/**
	 * The default contructor.
	 */
	public PortConnectionAnchor() {
		super();
	}

	/**
	 * The constructor with a Figure in parameter.
	 * @param owner : the owner of the connection
	 */
	public PortConnectionAnchor(final IFigure owner) {
		super(owner);
	}

	/**
	 * This function return the nearest figure point
	 * with the param point.
	 * @param arg0 : the current location
	 * @return Point : current location
	 */
	public final Point getLocation(final Point arg0) {
		boolean onLeft;
		Rectangle parent = getOwner().getParent().getBounds();
		Point parentLeft = new PrecisionPoint(parent.getTopLeft());
		getOwner().getParent().translateToAbsolute(parentLeft);

		Rectangle r = getOwner().getBounds();
		Point left = new PrecisionPoint(r.getTopLeft());
		getOwner().translateToAbsolute(left);

		onLeft = Math.abs(parentLeft.x - left.x) < FUDGEFACTOR;
		Point result = new PrecisionPoint(
				onLeft ? r.x : r.x + r.width, r.y + r.height / 2);
		getOwner().translateToAbsolute(result);

		return result;
	}
}
