package com.bull.cimero.pluginEditor.editors.figure.SEFigure;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.ImageFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Image;
import cimeroEditor.CimeroEditorPlugin;
import com.bull.cimero.pluginEditor.editors.figure.GeneriqueCimeroFigure;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * The SECimeroFigure Class.
 * This class is the representation of a generic figure.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class SECimeroFigure extends GeneriqueCimeroFigure {
	/** width of the icone. */
	private static final int WIDTH_ICONE = 150;
	/** height of the icone. */
	private static final int HEIGHT_ICONE = 10;
	/**size of a rectangle.*/
	private static final int RECTSIZE = 15;
	/** image representing the icone. */
	private Image monImage;
	/** normal figure. */
	private ImageInnerClass monInnerClass;
	/** error figure. */
	private ImageInnerClass monInnerClassError;
	/** figure path. */
	private String pathToFigure = "";
	/** figure error path. */
	private String pathToErrorFigure = "";
	/** figure icon. */
	private String pathToIcon = "";

	/**
	 * copy constructor.
	 * @param figure : path to the figure
	 * @param errorFigure : path to the error figure
	 * @param icon : path to the icon
	 */
	public SECimeroFigure(final String figure, final String errorFigure
											, final String icon) {
		super();
		monInnerClass = new ImageInnerClass(figure);
		add(monInnerClass);
		monInnerClassError = new ImageInnerClass(errorFigure);
		addDefaultInput();
		addDefaultOutput();
		setPathToFigure(figure);
		setPathToErrorFigure(errorFigure);
		setPathToIcon(icon);
	}

	/**
	 * method used to separate valide component to others.
	 * @param valide : validation result
	 */
	@Override
	public final void setValideDraw(final boolean valide) {
		super.setValideDraw(valide);
		if (valide) {
			LayoutManager layout = getLayoutManager();
			//try to remove the old
			try {
				layout.remove(monInnerClassError);
				remove(monInnerClassError);
			} catch (Exception e) {
				//add the new
				add(monInnerClass);
			}
			add(monInnerClass);
		} else {
			LayoutManager layout = getLayoutManager();
			//try to remove the old
			try {
				layout.remove(monInnerClass);
				remove(monInnerClass);
			} catch (Exception e) {
				//add the new
				add(monInnerClassError);
			}
			add(monInnerClassError);
		}
	}

	/**
	 * this function is used to position this figure in the parents bound.
	 */
	public final void validate() {
		LayoutManager layout = getLayoutManager();
		if (isValideDraw()) {
			//normal print
			layout.setConstraint(
					monInnerClass, new Rectangle(RECTSIZE, 0, -1, -1));
		} else {
			//error print
			layout.setConstraint(
					monInnerClassError, new Rectangle(RECTSIZE, 0, -1, -1));
		}
		super.validate();
	}

	/**
	 * method to get the dimension of the icon.
	 * @return  Dimension : the dimension of the icon
	 */
	public final Dimension getFigureDimension() {
		if (monImage != null) {
			return new Dimension(
					monImage.getBounds().width + RECTSIZE * 2
					, monImage.getBounds().height);
		} else {
			return new Dimension(
					WIDTH_ICONE, HEIGHT_ICONE);
		}
	}

	/**
	 * This class is used to add an image in a Figure.
	 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
	 */
	public class ImageInnerClass extends Figure {

		/** path to the icone. */
		private String iconePath = "";

		/**
		 * default constructor of this inner class.
		 * @param iconepath : path to icon
		 */
		public ImageInnerClass(final String iconepath) {
			super();
			FlowLayout layout = new FlowLayout();
			setLayoutManager(layout);
			//init variable
			this.iconePath = iconepath;
			try{
			monImage = CimeroEditorPlugin.getImageDescriptor(
					this.iconePath).createImage();
			ImageFigure img = new ImageFigure(monImage);
			add(img);
			}catch(Throwable t){
				CimeroLog.errorLog("Not found image"+ iconePath,t);
				throw new RuntimeException(t);
			}
		}
	}

	/**
	 * get the path to the error figure.
	 * @return Returns the pathToErrorFigure.
	 */
	@Override
	public final String getPathToErrorFigure() {
		return pathToErrorFigure;
	}

	/**
	 * set the path to the error figure.
	 * @param pathToErrorFigureP The pathToErrorFigure to set.
	 */
	@Override
	public final void setPathToErrorFigure(final String pathToErrorFigureP) {
		this.pathToErrorFigure = pathToErrorFigureP;
	}

	/**
	 * get the path to the figure.
	 * @return Returns the pathToFigure.
	 */
	@Override
	public final String getPathToFigure() {
		return pathToFigure;
	}

	/**
	 * set the path to the figure.
	 * @param pathToFigureParam The pathToFigure to set.
	 */
	@Override
	public final void setPathToFigure(final String pathToFigureParam) {
		this.pathToFigure = pathToFigureParam;
	}

	/**
	 * get the path to the icon.
	 * @return Returns the pathToIcon.
	 */
	@Override
	public final String getPathToIcon() {
		return pathToIcon;
	}

	/**
	 * set the path to the figure.
	 * @param pathToIconParam The pathToFigure to set.
	 */
	@Override
	public final void setPathToIcon(final String pathToIconParam) {
		this.pathToIcon = pathToIconParam;
	}

}
