package com.bull.cimero.pluginEditor.editors.model.BCModel;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * This class is the FTP model.
 * It contain the properties of a FTP server
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class FTP extends BindingComponentModel {

	/** serialUID. */
	private static final long serialVersionUID = 3182004700564702781L;
	//
	//CONSTANT
	//
	/**The default name of the ftp component.*/
	private static final String DEFAULTNAME = "myFTPServer";
	//
	//properties
	//
	/**The host adress id.*/
	private static final String HOSTADRESS = "host adress";
	/**The login id.*/
	private static final String LOGIN = "login";
	/**The password id.*/
	private static final String PASSWORD = "password";
	/**The path id.*/
	private static final String PATH = "path";
	/**The destination id.*/
	private static final String NAMEDEST = "destination";
	/**The xpath rule id.*/
	private static final String XPATHTITLE = "xpath rule to title";
	// variables

	/**The id of this element.*/
	private static final String ID = "Ftp";

	/**
	 * The default constructor.
	 */
	public FTP() {
		super(DEFAULTNAME);
		if (setInput(false)) {
			setOutput(true);
		}
		//set the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(INOUT, "");
		setProperty(HOSTADRESS, "");
		setProperty(LOGIN, "");
		setProperty(PASSWORD, "");
		setProperty(PATH, "");
		setProperty(NAMEDEST, "");
		setProperty(XPATHTITLE, "");
	}

	/**
	 * This function is used to determine if this element is valide.
	 * @return true is this element is valide.
	 * @param showLog : true if the log must to be show.
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		if (getName().compareTo("") == 0) {
			msg += "\n- Name => the element name cannot be null";
			returnResult = false;
		}
		if (getHostName().compareTo("") == 0) {
			msg += "\n- host adress => set the server adress";
			returnResult = false;
		}
		if (getLogin().compareTo("") == 0) {
			msg += "\n- login => The login cannot be null. If the server is a"
				+ " public server, the login may be 'anonymous'";
			returnResult = false;
		}
		if (isOutput()) {
			if (getNameFirstDestination().compareTo("") == 0) {
				msg += "\n- Destination service => the destination of this "
							+ "element cannot be null!";
				returnResult = false;
			}
		} else {
			if ((getXpathTitle().compareTo("") == 0)
					|| (!Ressources.testXPath(getXpathTitle()))) {
				msg += "\n- xpathTitle => The xpath rule to determine the "
					+ "name of the file is not valide";
				returnResult = false;
			}
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}

	/**
	 * Create the descriptor to the property views.
	 */
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, indice++ + " - name : ");
		addTextDescriptor(HOSTADRESS, indice++ + " - host adress : ");
		addTextDescriptor(LOGIN, indice++ + " - login : ");
		addTextDescriptor(PASSWORD, indice++ + " - password : ");
		addTextDescriptor(PATH, indice++ + " - path (optional) : ");
		addComboBoxDescriptor(INOUT,
	    		indice++ + " - mode", new String[]{"IN", "OUT"});
		if (isOutput()) {
			String[] lst = getLstObjectDrawWithInputConnection();
			addComboBoxDescriptor(NAMEDEST,
					indice++ + " - destination : ", lst);
		} else {
			addTextDescriptor(XPATHTITLE,
					indice++ + " - " + XPATHTITLE + " : ");
		}
	}

	/**
	 * This function is used to set the variable. It is an abstract method
	 * of IpropertySource.
	 * @param id : the id of the object
	 * @param value : the value of this object
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == INOUT) {
			if (isInput()) {
				if (setInput(false)) {
					setOutput(true);
				}
			} else {
				if (setOutput(false)) {
					setInput(true);
				}
			}
		} else if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		} else {
			super.setPropertyValue(id, value);
		}
	}

	/**
	 * This function is the implementation of an abstract method of
	 * IPropertySource class.
	 * It is used to return the value of selected element identified by id.
	 * @param id : the id of the element
	 * @return Object : the value of this element
	 */
	public final Object getPropertyValue(final Object id) {
		if (INOUT.equals(id)) {
			if (!isInput()) {
				return 0;
			} else {
				return 1;
			}
		} else if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
		} else {
			return super.getPropertyValue(id);
		}
		return null;
	}

	/**
	 * This function is the implementation of an abstract method of
	 * IPropertySource class.
	 * It is used to reset the value of all property.
	 * @param id : the id of the selected element
	 */
	public final void resetPropertyValue(final Object id) {
	}

	/**
	 * get the hostname.
	 * @return Returns the hostName.
	 */
	public final String getHostName() {
		return (String) getProperty(HOSTADRESS);
	}

	/**
	 * get the login.
	 * @return Returns the login.
	 */
	public final String getLogin() {
		return (String) getProperty(LOGIN);
	}

	/**
	 * get the password.
	 * @return Returns the password.
	 */
	public final String getPassword() {
		return (String) getProperty(PASSWORD);
	}

	/**
	 * get the path.
	 * @return Returns the path.
	 */
	public final String getPath() {
		return (String) getProperty(PATH);
	}

	/**
	 * get the xpath rule to title.
	 * @return Returns the xpathTitle.
	 */
	public final String getXpathTitle() {
		return (String) getProperty(XPATHTITLE);
	}

	/**
	 * get the id.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}

	/**
	 * set the hostname.
	 * @param hostNameParam The hostName to set.
	 */
	public final void setHostName(final String hostNameParam) {
		setProperty(HOSTADRESS, hostNameParam);
	}

	/**
	 * set the login.
	 * @param loginParam The login to set.
	 */
	public final void setLogin(final String loginParam) {
		setProperty(LOGIN, loginParam);
	}

	/**
	 * set the password.
	 * @param passwordParam The password to set.
	 */
	public final void setPassword(final String passwordParam) {
		setProperty(PASSWORD, passwordParam);
	}

	/**
	 * set the path.
	 * @param pathParam The path to set.
	 */
	public final void setPath(final String pathParam) {
		setProperty(PATH, pathParam);
	}

	/**
	 * set the xpath rule to title.
	 * @param xpathTitleParam The xpathTitle to set.
	 */
	public final void setXpathTitle(final String xpathTitleParam) {
		setProperty(XPATHTITLE, xpathTitleParam);
	}
}
