package com.bull.cimero.pluginEditor.editors.model.BCModel;

import java.util.ArrayList;
import java.util.List;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import com.bull.cimero.pluginEditor.editors.model.Element;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * The routeur model class.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class FileBC extends BindingComponentModel  {

	
	private static final long serialVersionUID = 1L;
	public static final String DEFAULTNAME = "myFileBC";
	private static final String NAMEDEST = "destination service";
	public static final String DIRECTORY = "directory";
	public static final String FILE = "file";
	private static final String ID = "FILEBC";

	/**.
	 * The default contructor
	 */
	public FileBC() {
		super(DEFAULTNAME);
		if (setInput(false)) {
			setOutput(true);
		}
		
		//add name properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(INOUT, "");
		setProperty(DIRECTORY, "directory:");
		setProperty(FILE, "file:");
	}

	

	/**.
	 * This function is used by the validator. It is used to determine
	 * if this class is validate or not
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		if (getName().compareTo("") == 0) {
			msg += "\n- Name => the element name cannot be null";
			returnResult = false;
		}
		if (isOutput()) { 
			//mode IN
			if (getNameFirstDestination().compareTo("") == 0) {
				msg += "\n- destination => "
					+ "the destination service cannot be null";
				returnResult = false;
			}
			if (getFile().equalsIgnoreCase("")) {
				msg += "\n- file => "
					+ "the file cannot be null";
				returnResult = false;
			}
		} else if (getDirectory().equalsIgnoreCase("")) {
			msg += "\n- directory => "
				+ "the directory cannot be null";
			returnResult = false;
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}
	
	private String getString(int indice){
		if (indice >= 10)
			return String.valueOf(indice);
		else
			return "0" + String.valueOf(indice);
	}

	/**
	 * reset all property value.
	 * @param id : the id of the element
	 * @see org.eclipse.ui.views.properties.
	 * IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public final void resetPropertyValue(final Object id) {
		//do nothing
	}
	
	public String getDirectory(){
		return (String)getProperty(DIRECTORY);
	}

	public String getFile(){
		return (String)getProperty(FILE);
	}
	/**
	 * method to create descriptors.
	 * @see com.bull.cimero.pluginEditor.editors.model
	 * .GeneriqueModel#createDescriptor()
	 */
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, getString(indice++) + " - name : ");
		addComboBoxDescriptor(INOUT,
				getString(indice++) + " - mode", new String[]{"IN", "OUT"});
		 
		if (!isInput()) {
				//consumer
			addTextDescriptor(FILE, getString(indice++) + " - file ");
			String [] lst = getLstObjectDrawWithInputConnection();
			addComboBoxDescriptor(NAMEDEST, getString(indice++) + " - destination name :", lst);
				 
		} else
			addTextDescriptor(DIRECTORY, getString(indice++) + " - directory ");
	}	
	/**
	 * Abstract method. Permit to set the property value.
	 * @param id : the id of the property
	 * @param value : the value of this property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		
		if (id == INOUT) {
			System.out.print("Called setPropertyValue -> [" + value +"] isInput["+isInput()+"] isOutput ["+isOutput()+"]\n");
		
			if (isInput()) {
				if (setInput(false)) {
					setOutput(true);
				}
			} else {
				if (setOutput(false)) {
					setInput(true);
				}
			}
		} else if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * Return the value of a property.
	 * @param id : the id of the property
	 * @return Object
	 */
	public final Object getPropertyValue(final Object id) {
		if (INOUT.equals(id)) {
			System.out.print("Called getPropertyValue isInput() [" + isInput()+"] isOutput ["+isOutput()+"]\n");
			if (!isInput()) {
				return 0;
			} else {
				return 1;
			}
		} else if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
		}
		return super.getPropertyValue(id);
	}

	/**
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
	
}
