package com.bull.cimero.pluginEditor.editors.model.BCModel;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * This class is the ServiceWeb model.
 * It contains the properties of a ServiceWeb
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ServiceWeb extends BindingComponentModel {

	/** serialVersionUID. */
	private static final long serialVersionUID = -7441876240933138357L;
	/** the default name. */
	private static final String DEFAULTNAME = "myWebService";
	/** the default hostname. */
	public static final String LOCATIONURI = "serveurName";
	/** the default destination service. */
	private static final String NAMEDEST = "destination service";
	/**The soap request name.*/
	private static final String SOAPREQUEST = "soapRequets";
	
	/**The soap request name.*/
	private static final String SSLSUPPORT = "sslSupport";
	
	private static final String KS = "keystore";
	private static final String KS_PASS = "keystorePass";
	private static final String KS_TYPE = "keystoreType";
	public static final String[] KS_TYPE_COMBO_VALUES =
		new String[]{"JKS","PKCS12"};
	private static final String CLIENT_AUTH = "clientAuthentication";
	private static final String TRUST_STORE = "trustStore";
	private static final String TRUST_PASS = "trustPass";
	private static final String TRUST_TYPE = "trustType";
	

	/** The soap version */
	private static final String SOAP_VERSION="SOAP_VERSION";
	private static final String[] SOAP_VERSION_COMBO = {"1.2", "1.1"};
	
	/** The soap action */
	private static final String SOAP_ACTION="SOAP_ACTION";
	
	/**The id of this element.*/
	private static final String ID = "WebService";
	private boolean createdByPipeline = false;

	/**
	 * The default constructor.
	 */
	public ServiceWeb() {
		super(DEFAULTNAME);
		if (setInput(false)) {
			setOutput(true);
		}
	
		
		//set the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(SOAPREQUEST, 0);
		setProperty(SSLSUPPORT, 1);
		setProperty(LOCATIONURI, "");
		setProperty(KS, "");
		setProperty(KS_PASS, "");
		setProperty(TRUST_STORE, "");
		setProperty(TRUST_PASS, "");
		setProperty(CLIENT_AUTH, 0);
		setProperty(INOUT, "");
		setProperty(FLAG_INPUT_OUTPUT, "INPUT");
		setProperty(SOAP_VERSION, 0);
		setProperty(SOAP_ACTION, "");

		setStandard(true);
	}

	/**
	 * The default constructor.
	 * @param name : the name of this component.
	 */
	public ServiceWeb(final String name) {
		super(name);
		if (setInput(false)) {
			setOutput(true);
		}
		//set the properties
		setProperty(NAMEDEST, "");
		setProperty(SOAPREQUEST, 0);
		setProperty(SSLSUPPORT, 1);
		setProperty(KS, "");
		setProperty(KS_PASS, "");
		setProperty(CLIENT_AUTH, 0);
		setProperty(LOCATIONURI, "");
		setProperty(INOUT, "");
		setProperty(TRUST_STORE, "");
		setProperty(TRUST_PASS, "");
		setProperty(FLAG_INPUT_OUTPUT, "INPUT");
		setProperty(KS_TYPE, 0);
		setProperty(TRUST_TYPE, 0);
		setProperty(SOAP_VERSION, 0);
		setProperty(SOAP_ACTION, "");
		
		setStandard(true);
	}

	/**
	 * This function is used to validate this class. return true
	 * if all the variable are specifies.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		if (getName().compareTo("") == 0) {
			msg += "\n- Name => the element name cannot be null";
			returnResult = false;
		}
		if (getHostNameAdress().compareTo("") == 0) {
			msg += "\n- HostName => the server adress is required !";
			returnResult = false;
		}
		if (isOutput()) {
			//mode IN
			if (getNameFirstDestination().compareTo("") == 0) {
				msg += "\n- destination => "
					+ "the destination service cannot be null";
				returnResult = false;
			}
		}
		
		if (isSslRequested()){
			String locationURI = getHostNameAdress();
			if (!locationURI.startsWith("https")) {
				msg += "\n Location URI => the protocol must be https if enabling" +
						" https negotiation";
				returnResult = false;
			}
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}

	private String getString(int indice){
		if (indice >= 10)
			return String.valueOf(indice);
		else
			return "0" + String.valueOf(indice);
	}
	/**
	 * Create the descriptor of this function.
	 */
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, getString(indice++) + " - name : ");
		addTextDescriptor(LOCATIONURI, getString(indice++) + " - Location URI : ");
		addComboBoxDescriptor(SOAPREQUEST, getString(indice++) + " - soap message",
				new String[]{"TRUE", "FALSE"});
		addComboBoxDescriptor(SOAP_VERSION, getString(indice++) + " - soap version",
				SOAP_VERSION_COMBO);
		addTextDescriptor(SOAP_ACTION, getString(indice++) + " - soap action");
		
		addComboBoxDescriptor(INOUT,
				getString(indice++) + " - mode", new String[]{"CONSUMER", "PROVIDER"});
		
		 addComboBoxDescriptor(FLAG_INPUT_OUTPUT, getString(indice++) + " - MEP ",
					new String[]{"IN", "OUT", "IN-OUT"});
		 
		 addComboBoxDescriptor(SSLSUPPORT, getString(indice++) + " - enable https(ssl)",
					new String[]{"TRUE", "FALSE"});
		 if (isSslRequested()){
			 if (!isInput()) {
				 // Consumer
			
				 if (isSslRequested()){
					 addComboBoxDescriptor(CLIENT_AUTH, getString(indice++) + " - Client Authentication ",
								new String[]{"No Client Authentication", "Optional Client Authentication", "Mandatory Client Authentication"});
					 
					 addTextDescriptor(KS, getString(indice++) + " - Keystore: ");
					 addPasswordTextDescriptor(KS_PASS, getString(indice++) + " - Keystore Password: ");
					 addComboBoxDescriptor(KS_TYPE,
						     getString(indice++) + " - Keystore type :", KS_TYPE_COMBO_VALUES);

					 if (isClientAuthenticationOptionalOrMandatory()){
						 addTextDescriptor(TRUST_STORE, getString(indice++) + " - Trust Store: ");
						 addPasswordTextDescriptor(TRUST_PASS, getString(indice++) + " - Trust Password: ");
						 addComboBoxDescriptor(TRUST_TYPE,
							     getString(indice++) + " - Truststore type :", KS_TYPE_COMBO_VALUES);
					 }
				 }
			 }else{
			
				 addTextDescriptor(KS, getString(indice++) + " - Keystore: ");
				 addPasswordTextDescriptor(KS_PASS, getString(indice++) + " - Keystore Password: ");
				 addTextDescriptor(TRUST_STORE, getString(indice++) + " - Trust Store: ");
				 addPasswordTextDescriptor(TRUST_PASS, getString(indice++) + " - Trust Password: ");
				 
			 }
		 }
		 if (!isInput()) {
			 String [] lst = getLstObjectDrawWithInputConnection();
			 addComboBoxDescriptor(NAMEDEST,
		     getString(indice++) + " - destination name :", lst);
		 }
			 
		 }

	/**
	 * Abstract method. Permit to set the property value.
	 * @param id : the id of the property
	 * @param value : the value of this property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		
		if (id == INOUT) {
			System.out.print("Called setPropertyValue -> [" + value +"] isInput["+isInput()+"] isOutput ["+isOutput()+"]\n");
			
		
			if (isInput()) {
				if (setInput(false)) {
					setOutput(true);
				}
			} else {
				if (setOutput(false)) {
					setInput(true);
				}
			}
		}else if (id == FLAG_INPUT_OUTPUT) {
			int intValue = ((Integer)value).intValue();
			if (intValue == 0){
				setProperty(FLAG_INPUT_OUTPUT, "INPUT");
			}else if (intValue == 1){
				setProperty(FLAG_INPUT_OUTPUT, "OUTPUT");
			}else{
				setProperty(FLAG_INPUT_OUTPUT, "INPUT-OUTPUT");
			}
			return;
		} else if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * Return the value of a property.
	 * @param id : the id of the property
	 * @return Object
	 */
	public final Object getPropertyValue(final Object id) {
		if (INOUT.equals(id)) {
			System.out.print("Called getPropertyValue isInput() [" + isInput()+"] isOutput ["+isOutput()+"]\n");
			if (!isInput()) {
				return 0;
			} else {
				return 1;
			}
		}	else if (FLAG_INPUT_OUTPUT.equals(id)) {
			String io = (String)getProperty(FLAG_INPUT_OUTPUT);
			if (io.equalsIgnoreCase("INPUT")){
				return 0;
			}else if (io.equalsIgnoreCase("OUTPUT")){
				return 1;
			}else if (io.equalsIgnoreCase("INPUT-OUTPUT")){
				return 2;
			}
		
		} else if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
		} else if (KS_TYPE.equals(id)) {
			Object value = getProperty(KS_TYPE);
			if (value == null || !(value instanceof Integer)) {
				setProperty(KS_TYPE, 0);
				return 0;
			}
		} else if (TRUST_TYPE.equals(id)) {
			Object value = getProperty(TRUST_TYPE);
			if (value == null || !(value instanceof Integer)) {
				setProperty(TRUST_TYPE, 0);
				return 0;
			}
		} else if (id == SOAP_VERSION) {
			Object soapVersion = getProperty(SOAP_VERSION);
			return (soapVersion != null &&(soapVersion instanceof Integer))? soapVersion : 0;
		} else if (id == SOAP_ACTION) {
			return getSoapAction();
		}
		return super.getPropertyValue(id);
	}

	/**
	 * reset the value of the property.
	 * @param id : the id property
	 */
	public final void resetPropertyValue(final Object id) {
	}

	/**
	 * get the hostname.
	 * @return Returns the hostNameAdress.
	 */
	public final String getHostNameAdress() {
		return (String) getProperty(LOCATIONURI);
	}

	/**
	 * set the hostname.
	 * @param hostNameAdressParam The hostNameAdress to set.
	 */
	public final void setHostNameAdress(final String hostNameAdressParam) {
		setProperty(LOCATIONURI, hostNameAdressParam);
	}

	/**
	 * get the soaprequest.
	 * @return Returns the soapRequest.
	 */
	public final boolean isSoapRequest() {
		return ((Integer) getProperty(SOAPREQUEST) == 0);
	}
	
	/**
	 * get the soaprequest.
	 * @return Returns the soapRequest.
	 */
	public final boolean isSslRequested() {
		return ((Integer) getProperty(SSLSUPPORT) == 0);
	}
	
	/**
	 * get the soaprequest.
	 * @return Returns the soapRequest.
	 */
	public final boolean isInputOutput() {
		return ((Integer) getProperty(FLAG_INPUT_OUTPUT) == 0);
	}

	/**
	 * get the id.
	 * @return the id of this component.
	 */
	public final String getComponentId() {
		return ID;
	}

	/**
	 * set the soap request.
	 * @param soapRequestP the soapRequest to set
	 */
	public final void setSoapRequest(final boolean soapRequestP) {
		if (soapRequestP) {
			setProperty(SOAPREQUEST, 0);
		} else {
			setProperty(SOAPREQUEST, 1);
		}
	}
	
	
	public final void setSslSupport(final boolean sslSupport) {
		if (sslSupport) {
			setProperty(SSLSUPPORT, 0);
		} else {
			setProperty(SSLSUPPORT, 1);
		}
	}
	
	public final boolean isClientAuthenticationOptionalOrMandatory() {
		Integer o = (Integer)getProperty(CLIENT_AUTH);
		
		return o > 0;
	}
	
	public final boolean isClienAuthenticationOptional(){
		Integer o = (Integer)getProperty(CLIENT_AUTH);
		
		return o == 1;
	}
	
	public final boolean isClienAuthenticationMandatory(){
		Integer o = (Integer)getProperty(CLIENT_AUTH);
		
		return o == 2;
	}
	/**
	 * set the soap request.
	 * @param soapRequestP the soapRequest to set
	 */
	public final void setFlagInputOutput(final boolean flgInputOutput) {
		if (flgInputOutput) {
			setProperty(FLAG_INPUT_OUTPUT, 0);
		} else {
			setProperty(FLAG_INPUT_OUTPUT, 1);
		}
	}
	
	public final String getMep(){
		return (String)getProperty(FLAG_INPUT_OUTPUT);
	}
	public final String getKeyStore(){
		return (String)getProperty(KS);
	}
	public final String getKeyStoreType(){
		Integer intType = (Integer)getProperty(KS_TYPE);
		if (intType != null) {
			return KS_TYPE_COMBO_VALUES[intType];
		}
		return KS_TYPE_COMBO_VALUES[0];
	}
	
	public final String getKeyStorePass(){
		return (String)getProperty(KS_PASS);
	}
	
	public final String getTrustStore(){
		return (String)getProperty(TRUST_STORE);
	}
	
	public final String getTrustStorePass(){
		return (String)getProperty(TRUST_PASS);
	}

	public boolean isCreatedByPipeline() {
		return createdByPipeline;
	}

	public void setCreatedByPipeline(boolean createdByPipeline) {
		this.createdByPipeline = createdByPipeline;
	}
	
	public final String getTrustStoreType(){
		Integer intType = (Integer)getProperty(TRUST_TYPE);
		if (intType != null) {
			return KS_TYPE_COMBO_VALUES[intType];
		}
		return KS_TYPE_COMBO_VALUES[0];
	}
	public final String getSoapVersion(){
		Integer intType = (Integer)getProperty(SOAP_VERSION);
		if (intType != null) {
			return SOAP_VERSION_COMBO[intType];
		}
		return SOAP_VERSION_COMBO[0];
	}	
	
	public final String getSoapAction() {
		Object objValue = getProperty(SOAP_ACTION);
		return (objValue != null)? (String)objValue : "";
	}
	
	public final void setSoapAction(String action) {
		setProperty(SOAP_ACTION, action);
	}
	
	public final void setSoapVersion(String version) {
		for (int idx = 0; idx < SOAP_VERSION_COMBO.length; idx++) {
			if (SOAP_VERSION_COMBO[idx].equals(version)) {
				setProperty(SOAP_VERSION, idx);
			}
		}
	}
}
