package com.bull.cimero.pluginEditor.editors.model.BCModel;

import it.eng.cimero.plugin.descriptor.PasswordTextPropertyDescriptor;

import java.io.Serializable;
import java.net.InetAddress;
import java.nio.ByteOrder;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
/**
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class TCPModelHelper implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	public static final String POINT_NAME = "PointName".intern();
	/* it instead use the BindingComponentModel.INOUT key  */
	//	public static final String POINT_TYPE = "PointType".intern();
		public static final String POINT_MODE = "PointMode".intern();
	/* low implementation priority */
	public static final String INPUT_THROTTLING = "InputThrottling".intern();
	public static final String INPUT_THROTTLING_INDIVIDUAL_CONNECTION = "InputThrottlingIndividualConnection".intern();
	public static final String OUT_THROTTLING = "OutThrottling".intern();
	public static final String OUT_THROTTLING_INDIVIDUAL_CONNECTION = "OutThrottlingIndividualConnection".intern();
	/* Response communication settings */
	public static final String RESPONSE_TIMEOUT = "ResponseTimeout".intern();
	public static final String RETRY_COUNT = "RetryCount".intern();
	public static final String FAIL_ACTION = "FailAction".intern();
	public static final String CONN_NUMBER = "ConnNumber".intern();
	/* Connection initialization settings */
	public static final String RETRY_NUMBER = "RetryNumber".intern();
	public static final String RETRY_TYPE = "RetryType".intern();
	public static final String RETRY_DELAY = "RetryDelay".intern();
	/* JBI Marshalling/Unmarshalling */
	public static final String OUT_NM_ENVELOPE = "OutNmEnvelope".intern();
	public static final String IN_NM_ENVELOPE = "InNmEnvelope".intern();
	public static final String DEFAULT_MEP = "DefaultMep".intern();
	public static final String BASE64_ENCODE = "BASE64_ENCODE".intern();
	public static final String BASE64_DECODE = "BASE64_DECODE".intern();
	public static final String IN_OUT = "IN-OUT".intern();
	public static final String IN_ONLY = "IN-ONLY".intern();
	public static final String OUT_ONLY = "OUT-ONLY".intern();
	public static final String[] MEP_COMBO_VALUES =
	new String[]{IN_OUT,IN_ONLY,OUT_ONLY};
	/* INPUT MODE */
	// listening config
	public static final String LOCAL_PORT = "LocalPort".intern();
	public static final String LOCAL_ADDRESS = "LocalAddress".intern();
	public static final String CONNECTION_MODE = "CONNECTION_MODE".intern();
	public static final String CONN_MODE_DISCONNECT = "CONN_MODE_DISCONNECT".intern();
	public static final String CONN_MODE_MAINTAIN = "CONN_MODE_MAINTAIN".intern();
	public static final String[] COMBO_CONNECTION_MODES = {
		CONN_MODE_MAINTAIN, CONN_MODE_DISCONNECT
	};
	
	// server listening only
	public static final String LISTEN_BACKLOG = "ListenBacklog".intern();
	// client listening only
	public static final String REMOTE_HOST = "RemoteHost".intern();
	public static final String REMOTE_PORT = "RemotePort".intern();
	public static final String USE_SSL = "UseSSL".intern();
	public static final String USE_SSL_CLIENT_MODE = "UseSSLClientMode".intern();
	public static final String KEY_STORE_FILE_NAME = "KeyStoreFilename".intern();
	public static final String KEY_STORE_PASSWORD = "KeyStorePassword".intern();
	public static final String TRUST_STORE_FILE_NAME = "TrustStoreFilename".intern();
	public static final String TRUST_STORE_PASSWORD = "TrustStorePassword".intern();
	// log config
	public static final String LOG_CONNECTIONS = "LogConnections".intern();
	public static final String LOG_DATA = "LogData".intern();
	public static final String LOG_DATA_AS_HEX = "LogDataAsHex".intern();
	public static final String CONNECTION_LOG_FILE_NAME = "ConnectionLogFilename".intern();
	public static final String EXTRA_INFORMATION = "ExtraInformation".intern();
	// protocol config
	public static final String INCOMING_WRAPPER = "IncomingWrapper".intern();
	public static final String STRIP_WRAPPING = "StripWrapping".intern();
	public static final String INCOMING_HEADER = "IncomingHeader".intern();
	public static final String INCOMING_TRAILER = "IncomingTrailer".intern();
	public static final String INCOMING_ENDIANNESS = "IncomingEndianness".intern();
	/* OUTPUT MODE */
	public static final String OUTGOING_WRAPPER  = "OutgoingWrapper".intern();
	public static final String OUTGOING_HEADER = "OutgoingHeader".intern();
	public static final String OUTGOING_TRAILER = "OutgoingTrailer".intern();
	public static final String OUTGOING_ENDIANNESS = "OutgoingEndianness".intern();
	/* Client only config */
	public static final String TCP_OUT_IN_RECEIVER_CLASSNAME = "TcpInOutReceiverClassname".intern();
	public static final String EXTRA_INFO_NONE = "EXTRA_INFO_NONE".intern();
	public static final String EXTRA_INFO_LOG_TIME = "EXTRA_INFO_LOG_TIME".intern();
	public static final String WRAPPER_MINIMAL = "WRAPPER_MINIMAL".intern();
	public static final String WRAPPER_USER = "WRAPPER_USER".intern();
	public static final String[] WRAPPER_COMBO_VALUES =
	new String[]{WRAPPER_USER, WRAPPER_MINIMAL};
	public static final String BIG_ENDIAN_STR = ByteOrder.BIG_ENDIAN.toString();
	public static final String LITTLE_ENDIAN_STR = ByteOrder.LITTLE_ENDIAN.toString();
	public static final String[] ENDIANNESS_COMBO_VALUES =
	new String[]{BIG_ENDIAN_STR, LITTLE_ENDIAN_STR};
	public static final String OPERATION_MODE_BIDIRECTIONAL = "OPERATION_MODE_BIDIRECTIONAL".intern();
	public static final String OPERATION_MODE_OUT_IN = "OPERATION_MODE_OUT_IN".intern();
	public static final String OPERATION_MODE_IN_OUT = "OPERATION_MODE_IN_OUT".intern();
	public static final String OPERATION_MODE_IN = "OPERATION_MODE_IN".intern();
	public static final String OPERATION_MODE_OUT = "OPERATION_MODE_OUT".intern();
	public static final String[] OPERATION_MODE_COMBO_VALUES = new String[]{OPERATION_MODE_BIDIRECTIONAL, OPERATION_MODE_IN,
	OPERATION_MODE_IN_OUT, OPERATION_MODE_OUT,
	OPERATION_MODE_OUT_IN};
	public static final String POINT_TYPE_CLIENT = "POINT_TYPE_CLIENT".intern();
	public static final String POINT_TYPE_SERVER = "POINT_TYPE_SERVER".intern();
	public static final String[] POINT_TYPE_COMBO_VALUES = new String[]{POINT_TYPE_SERVER, POINT_TYPE_CLIENT};
	public static final String FAIL_ACTION_MSG_TO_QUEUE = "FAIL_ACTION_MSG_TO_QUEUE".intern();
	public static final String FAIL_ACTION_CLOSE_CONN = "FAIL_ACTION_CLOSE_CONN".intern();
	public static final String FAIL_ACTION_MSG_TO_QUEUE_CLOSE_CONNECTION = "FAIL_ACTION_MSG_TO_QUEUE_CLOSE_CONNECTION".intern();
	public static final String[] FAIL_ACTION_COMBO_VALUES =
	new String[]{FAIL_ACTION_CLOSE_CONN, FAIL_ACTION_MSG_TO_QUEUE,
		FAIL_ACTION_MSG_TO_QUEUE_CLOSE_CONNECTION};
	public static final String RETRY_TYPE_NO_RETRY = "RETRY_TYPE_NO_RETRY";
	public static final String RETRY_TYPE_IMMEDIATE = "RETRY_TYPE_IMMEDIATE";
	public static final String RETRY_TYPE_LINEAR = "RETRY_TYPE_LINEAR";
	public static final String RETRY_TYPE_EXPONENTIAL = "RETRY_TYPE_EXPONENTIAL";
	public static final String[] RETRY_TYPE_COMBO_VALUES = new String[]{RETRY_TYPE_EXPONENTIAL, RETRY_TYPE_IMMEDIATE,
	RETRY_TYPE_LINEAR, RETRY_TYPE_NO_RETRY};
	static final String[] BOOLEAN_COMBO_VALUES = new String[]{"TRUE".intern(), "FALSE".intern()};

	
	/* state properties */
	private boolean isInput = false;
	private boolean isOutput = false;
	private boolean isBindingComponent = false;
	
	public boolean isInput() {
		return isInput;
	}

	public void setInput(boolean isInput) {
		this.isInput = isInput;
	}

	public boolean isOutput() {
		return isOutput;
	}

	public void setOutput(boolean isOutput) {
		this.isOutput = isOutput;
	}

	public void setDefaultProperties(GeneriqueCimeroModel model) {
		model.setProperty(TCPModelHelper.POINT_MODE, 0);
		model.setProperty(BindingComponentModel.INOUT, 0);
		
		/* low implementation priority */
//		model.setProperty(INPUT_THROTTLING, "");
//		model.setProperty(INPUT_THROTTLING_INDIVIDUAL_CONNECTION, "");
//		model.setProperty(OUT_THROTTLING, "");
//		model.setProperty(OUT_THROTTLING_INDIVIDUAL_CONNECTION, "");

		/* Response communication settings */
		model.setProperty(TCPModelHelper.RESPONSE_TIMEOUT, String.valueOf(10*1000));
		model.setProperty(TCPModelHelper.RETRY_COUNT, String.valueOf(1));
		model.setProperty(TCPModelHelper.FAIL_ACTION, 0);

		model.setProperty(TCPModelHelper.CONN_NUMBER, String.valueOf(-1));
		
		/* Connection initialization settings */
		model.setProperty(TCPModelHelper.RETRY_NUMBER, String.valueOf(0));
		model.setProperty(TCPModelHelper.RETRY_TYPE, 1);
		model.setProperty(TCPModelHelper.RETRY_DELAY, "0");
		
		/* JBI Marshalling/Unmarshalling */
		model.setProperty(TCPModelHelper.OUT_NM_ENVELOPE, "tcp-message");
		model.setProperty(TCPModelHelper.IN_NM_ENVELOPE, "tcp-message");
		model.setProperty(TCPModelHelper.DEFAULT_MEP, 0);
		model.setProperty(TCPModelHelper.BASE64_ENCODE, 0);
		model.setProperty(TCPModelHelper.BASE64_DECODE, 0);
		
		/* INPUT MODE */
		// listening config
		model.setProperty(TCPModelHelper.LOCAL_PORT, String.valueOf(10222));
		String localhostName = "";
		try {
			String lhName = InetAddress.getLocalHost().getHostName(); 
			String lhAddress = InetAddress.getLocalHost().getHostAddress(); 
			localhostName = ( lhName != null && !"".equals(lhName) )?
				lhName : lhAddress;
			if (localhostName == null || "".equals(localhostName)) {
				localhostName = "localhost";
			}
		} catch (Exception e) { /*ignore*/ }
		model.setProperty(TCPModelHelper.LOCAL_ADDRESS, localhostName);
		model.setProperty(TCPModelHelper.CONNECTION_MODE, 1);
		
		// server listening only
		model.setProperty(TCPModelHelper.LISTEN_BACKLOG, "");
		
		// client listening only
		model.setProperty(TCPModelHelper.REMOTE_HOST, "");
		model.setProperty(TCPModelHelper.REMOTE_PORT, String.valueOf(10222));
		model.setProperty(TCPModelHelper.USE_SSL, 1);
		model.setProperty(TCPModelHelper.USE_SSL_CLIENT_MODE, 1);
		model.setProperty(TCPModelHelper.KEY_STORE_FILE_NAME, "classpath:");
		model.setProperty(TCPModelHelper.KEY_STORE_PASSWORD, "");
		model.setProperty(TCPModelHelper.TRUST_STORE_FILE_NAME, "classpath:");
		model.setProperty(TCPModelHelper.TRUST_STORE_PASSWORD, "");
		
		// log config
		model.setProperty(LOG_CONNECTIONS, 1);
		model.setProperty(LOG_DATA, 1);
		model.setProperty(LOG_DATA_AS_HEX, 1);
		model.setProperty(CONNECTION_LOG_FILE_NAME, "");
		model.setProperty(EXTRA_INFORMATION, 1);
		
		// protocol config
		model.setProperty(TCPModelHelper.INCOMING_WRAPPER, 0);
		model.setProperty(TCPModelHelper.STRIP_WRAPPING, 0);
		model.setProperty(TCPModelHelper.INCOMING_HEADER, "0x0B");
		model.setProperty(TCPModelHelper.INCOMING_TRAILER, "0x1C 0x0D");
		model.setProperty(TCPModelHelper.INCOMING_ENDIANNESS, 0);
		
		/* OUTPUT MODE */
		model.setProperty(TCPModelHelper.OUTGOING_WRAPPER, 0);
		model.setProperty(TCPModelHelper.OUTGOING_HEADER, "0x0B");
		model.setProperty(TCPModelHelper.OUTGOING_TRAILER, "0x1C 0x0D");
		model.setProperty(TCPModelHelper.OUTGOING_ENDIANNESS, 0);
		
		/* Client only config */
		model.setProperty(TCPModelHelper.TCP_OUT_IN_RECEIVER_CLASSNAME, "it.eng.spagosoa.smx.components.tcp.ConsoleTCPReceiver");
		
	}
	
	public void setDescriptors(GeneriqueCimeroModel model) {
		int mainIdx = 0;
		/* COMMON CONFIG */
		int categoryIdx = 0;
		String category;
//		category = getString(++mainIdx)+" TCPBC";
		IPropertyDescriptor pDescr;
//		pDescr = new TextPropertyDescriptor(NAME, getString(++categoryIdx)+" Component Name");
//		((TextPropertyDescriptor) pDescr).setCategory(category);
//		model.addDescriptor(pDescr);
		
		categoryIdx = 0;
//		pDescr = new TextPropertyDescriptor(POINT_NAME, getString(++categoryIdx)+" Connection Point name");
		category = getString(++mainIdx)+" COMMON";
//		((TextPropertyDescriptor) pDescr).setCategory(category);
//		model.addDescriptor(pDescr);
		
		/* correspond to rapsody POINT_TYPE */
		if (isBindingComponent) { // otherwise will be a pipeline with setted in client mode
			pDescr = new ComboBoxPropertyDescriptor(BindingComponentModel.INOUT, getString(++categoryIdx)+" Connection Point type",
				POINT_TYPE_COMBO_VALUES);
			((PropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
		}

		pDescr = new ComboBoxPropertyDescriptor(POINT_MODE, getString(++categoryIdx)+" Connection Point mode",
			OPERATION_MODE_COMBO_VALUES);
		((PropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		/* low implementation priority */
//		pDescr = new TextPropertyDescriptor(INPUT_THROTTLING, String.valueOf(subIdx++));
//		((TextPropertyDescriptor) pDescr).setCategory("COMMON");
//		model.addDescriptor(pDescr);
//
//		pDescr = new TextPropertyDescriptor(INPUT_THROTTLING_INDIVIDUAL_CONNECTION, String.valueOf(subIdx++));
//		((TextPropertyDescriptor) pDescr).setCategory("COMMON");
//		model.addDescriptor(pDescr);
//
//		pDescr = new TextPropertyDescriptor(OUT_THROTTLING, String.valueOf(subIdx++));
//		((TextPropertyDescriptor) pDescr).setCategory("COMMON");
//		model.addDescriptor(pDescr);
//
//		pDescr = new TextPropertyDescriptor(OUT_THROTTLING_INDIVIDUAL_CONNECTION, String.valueOf(subIdx++));
//		((TextPropertyDescriptor) pDescr).setCategory("COMMON");
//		model.addDescriptor(pDescr);

		/* Response communication settings */
		pDescr = new TextPropertyDescriptor(RESPONSE_TIMEOUT, getString(++categoryIdx)+" Response timeout");
		((TextPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new TextPropertyDescriptor(RETRY_COUNT, getString(++categoryIdx)+" Retry count");
		((TextPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new ComboBoxPropertyDescriptor(FAIL_ACTION, getString(++categoryIdx)+" Fail action",
			FAIL_ACTION_COMBO_VALUES);
		((PropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new TextPropertyDescriptor(CONN_NUMBER, getString(++categoryIdx)+" Connection number");
		((TextPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		/* Connection initialization settings */
		pDescr = new TextPropertyDescriptor(RETRY_NUMBER, getString(++categoryIdx)+" Retry number");
		((TextPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new ComboBoxPropertyDescriptor(RETRY_TYPE, getString(++categoryIdx)+" Retry type",
			RETRY_TYPE_COMBO_VALUES);
		((PropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new TextPropertyDescriptor(RETRY_DELAY, getString(++categoryIdx)+" Retry delay");
		((TextPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		/* JBI Marshalling/Unmarshalling */
		categoryIdx = 0;
		pDescr = new TextPropertyDescriptor(OUT_NM_ENVELOPE, getString(++categoryIdx)+" Normalize message output envelope");
		category = getString(++mainIdx)+" JBI";
		((TextPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new TextPropertyDescriptor(IN_NM_ENVELOPE, getString(++categoryIdx)+" Normalize message intput envelope");
		((TextPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new ComboBoxPropertyDescriptor(BASE64_ENCODE, getString(++categoryIdx)+" Base64 encode message",
				BOOLEAN_COMBO_VALUES);
				((PropertyDescriptor) pDescr).setCategory(category);
				model.addDescriptor(pDescr);

		pDescr = new ComboBoxPropertyDescriptor(BASE64_DECODE, getString(++categoryIdx)+" Base64 decode message",
				BOOLEAN_COMBO_VALUES);
				((PropertyDescriptor) pDescr).setCategory(category);
				model.addDescriptor(pDescr);

		pDescr = new ComboBoxPropertyDescriptor(DEFAULT_MEP, getString(++categoryIdx)+" MEP",
			MEP_COMBO_VALUES);
			((PropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
		
		/* INPUT MODE */
		// server listening config
		if (isOutput()) {
			categoryIdx = 0;
			pDescr = new TextPropertyDescriptor(LOCAL_PORT, getString(++categoryIdx)+" Local port");
			category = getString(++mainIdx)+" LOCAL CONNECTION";
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
	
			pDescr = new TextPropertyDescriptor(LOCAL_ADDRESS, getString(++categoryIdx)+" Local address");
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);

			pDescr = new TextPropertyDescriptor(LISTEN_BACKLOG, getString(++categoryIdx)+" Listen backlog");
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
		}

		// client sending config
		if (isInput()) {
			pDescr = new TextPropertyDescriptor(REMOTE_HOST, getString(++categoryIdx)+" Remote host");
			category = getString(++mainIdx)+" REMOTE CONNECTION";
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
	
			pDescr = new TextPropertyDescriptor(REMOTE_PORT, getString(++categoryIdx)+" Remote port");
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);

			pDescr = new ComboBoxPropertyDescriptor(CONNECTION_MODE,
				getString(++categoryIdx)+" Connection modes",
				COMBO_CONNECTION_MODES);
			((ComboBoxPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
		}

		/* SECURITY */
		categoryIdx = 0;
		pDescr = new ComboBoxPropertyDescriptor(USE_SSL, getString(++categoryIdx)+" Use SSL",
			BOOLEAN_COMBO_VALUES);
		category = getString(++mainIdx)+" SECURITY";
		((PropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		if (isUseSSL(model)) {
			pDescr = new ComboBoxPropertyDescriptor(USE_SSL_CLIENT_MODE, getString(++categoryIdx)+" Use SSL client mode",
				BOOLEAN_COMBO_VALUES);
			((PropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
	
			pDescr = new TextPropertyDescriptor(KEY_STORE_FILE_NAME, getString(++categoryIdx)+" Keystore filename");
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
	
			pDescr = new PasswordTextPropertyDescriptor(KEY_STORE_PASSWORD, getString(++categoryIdx)+" Keystore password");
			((PropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
	
			pDescr = new TextPropertyDescriptor(TRUST_STORE_FILE_NAME, getString(++categoryIdx)+" Truststore filename");
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
	
			pDescr = new PasswordTextPropertyDescriptor(TRUST_STORE_PASSWORD, getString(++categoryIdx)+" Truststore password");
			((PropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
		}
		
		// log config
		categoryIdx = 0;
		category = getString(++mainIdx)+" LOGGING";
		pDescr = new ComboBoxPropertyDescriptor(LOG_CONNECTIONS, getString(++categoryIdx)+" Log connections",
			BOOLEAN_COMBO_VALUES);
		((ComboBoxPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new ComboBoxPropertyDescriptor(LOG_DATA, getString(++categoryIdx)+" Log data",
				BOOLEAN_COMBO_VALUES);
		((ComboBoxPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new ComboBoxPropertyDescriptor(LOG_DATA_AS_HEX, getString(++categoryIdx)+" Log data in hexadecimal",
				BOOLEAN_COMBO_VALUES);
		((ComboBoxPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new TextPropertyDescriptor(CONNECTION_LOG_FILE_NAME, getString(++categoryIdx)+" Log filename");
		((TextPropertyDescriptor) pDescr).setCategory(category);
		setNotNullStringProperty(model, CONNECTION_LOG_FILE_NAME);
		model.addDescriptor(pDescr);

		pDescr = new ComboBoxPropertyDescriptor(EXTRA_INFORMATION, getString(++categoryIdx)+" Log extra info",
				BOOLEAN_COMBO_VALUES);
		((ComboBoxPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		// protocol config
		categoryIdx = 0;
		pDescr = new ComboBoxPropertyDescriptor(INCOMING_WRAPPER, getString(++categoryIdx)+" Incoming wrapper",
			WRAPPER_COMBO_VALUES);
		category = getString(++mainIdx)+" PROTOCOL";
		((PropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		pDescr = new ComboBoxPropertyDescriptor(STRIP_WRAPPING, getString(++categoryIdx)+" Strip wrapping",
			BOOLEAN_COMBO_VALUES);
			((PropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);

		if (((Integer)model.getProperty(INCOMING_WRAPPER)).intValue() == 0) {
			pDescr = new TextPropertyDescriptor(INCOMING_HEADER, getString(++categoryIdx)+" Incoming header");
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
	
			pDescr = new TextPropertyDescriptor(INCOMING_TRAILER, getString(++categoryIdx)+" Incoming trailer");
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
	
			pDescr = new ComboBoxPropertyDescriptor(INCOMING_ENDIANNESS, getString(++categoryIdx)+" Incoming endianness",
				ENDIANNESS_COMBO_VALUES);
			((PropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
		}
		
		/* OUTPUT MODE */
		pDescr = new ComboBoxPropertyDescriptor(OUTGOING_WRAPPER, getString(++categoryIdx)+" Outgoing wrapper",
				WRAPPER_COMBO_VALUES);
		((PropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);

		if (((Integer)model.getProperty(OUTGOING_WRAPPER)).intValue() == 0) {
			pDescr = new TextPropertyDescriptor(OUTGOING_HEADER, getString(++categoryIdx)+" Outgoing header");
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
	
			pDescr = new TextPropertyDescriptor(OUTGOING_TRAILER, getString(++categoryIdx)+" Outgoing trailer");
			((TextPropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
	
			pDescr = new ComboBoxPropertyDescriptor(OUTGOING_ENDIANNESS, getString(++categoryIdx)+" Outgoing endianness",
					ENDIANNESS_COMBO_VALUES);
			((PropertyDescriptor) pDescr).setCategory(category);
			model.addDescriptor(pDescr);
		}
			
		/* Client only config */
		categoryIdx = 0;
		pDescr = new TextPropertyDescriptor(TCP_OUT_IN_RECEIVER_CLASSNAME, getString(++categoryIdx)+" Tcp out-in receiver classname");
		category = getString(++mainIdx)+" RECEIVER HANDLER";
		((TextPropertyDescriptor) pDescr).setCategory(category);
		model.addDescriptor(pDescr);
	}
	
	public boolean isValide(GeneriqueCimeroModel model, boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		if (model.getName().compareTo("") == 0) {
			msg += "\n1 - Name => the element name cannot be null";
			returnResult = false;
		}
		String stringProperty;
		
		// ssl settings
		if (isUseSSL(model)) {
			stringProperty = (String)model.getProperty(KEY_STORE_FILE_NAME);
			if (stringProperty.compareTo("") == 0) {
				msg += "\n2 - Key Store => the Key Store cannot be empty";
				returnResult = false;
			}
			stringProperty = (String)model.getProperty(KEY_STORE_PASSWORD);
			if (stringProperty.compareTo("") == 0) {
				msg += "\n3 - Key Store password => the Key Store password cannot be empty";
				returnResult = false;
			}
			// TODO check if actual implementation about TCP component override JVM
			// trust store configuration or can use that
			stringProperty = (String)model.getProperty(TRUST_STORE_FILE_NAME);
			if (stringProperty.compareTo("") == 0) {
				msg += "\n4 - Trust Store => the Trust Store cannot be empty";
				returnResult = false;
			}
			stringProperty = (String)model.getProperty(TRUST_STORE_PASSWORD);
			if (stringProperty.compareTo("") == 0) {
				msg += "\n5 - Trust Store password => the Trust Store password cannot be empty";
				returnResult = false;
			}
			
		}
		// client mode
		if (isInput()) {
			stringProperty = (String)model.getProperty(REMOTE_HOST);
			if (stringProperty.compareTo("") == 0) {
				msg += "\n5 - Remote Host => the Remote Host cannot be empty";
				returnResult = false;
			}
			stringProperty = (String)model.getProperty(REMOTE_PORT);
			if (stringProperty.compareTo("") == 0) {
				msg += "\n5 - Remote Port => the Remote Port cannot be empty";
				returnResult = false;
			}
		}
		if (isOutput()) {
			stringProperty = (String)model.getProperty(LOCAL_ADDRESS);
			if (stringProperty.compareTo("") == 0) {
				msg += "\n5 - Local Address => the Local Address cannot be empty";
				returnResult = false;
			}
			stringProperty = (String)model.getProperty(LOCAL_PORT);
			if (stringProperty.compareTo("") == 0) {
				msg += "\n5 - Local Port => the Local Port cannot be empty";
				returnResult = false;
			}
		}
		
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "\nELEMENT NAME : " + model.getName() + "\n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}
	
	public boolean isUseSSL(GeneriqueCimeroModel model) {
		return ((Integer)model.getProperty(TCPModelHelper.USE_SSL)).intValue() == 0;
	}
	
	public String getString(int indice){
		if (indice >= 10)
			return String.valueOf(indice);
		else
			return "0" + String.valueOf(indice);
	}

	public boolean isBindingComponent() {
		return isBindingComponent;
	}

	public void setBindingComponent(boolean isBindingComponent) {
		this.isBindingComponent = isBindingComponent;
	}
	
	private void setNotNullStringProperty(GeneriqueCimeroModel model, String propName) {
		Object value = model.getProperty(propName);
		if (value == null) {
			model.setProperty(propName, "");
			// seem not to work (I'd like to notify the gui about the change)
			model.firePropertyChange(propName, null, "");
		}
	}
}
