package com.bull.cimero.pluginEditor.editors.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;

/**
 * The Element class. This class is used to add the serialization.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class Element implements Cloneable, Serializable {

   /** serialization version. */
   static final long serialVersionUID = 1;
   /** property change listeners. */
   private transient PropertyChangeSupport listeners =
	   						new PropertyChangeSupport(this);

   /**
	* method to add a listener.
	* @param listener : the listener
	*/
   public final void addPropertyChangeListener(
		   final PropertyChangeListener listener) {
      listeners.addPropertyChangeListener(listener);
   }

   /**
     * method to change value of a property.
	 * @param prop : property to change
	 * @param old : old value
	 * @param newValue : new value
	 */
   public final void firePropertyChange(
		   final String prop, final Object old, final Object newValue) {
      listeners.firePropertyChange(prop, old, newValue);
   }

   /**
	 * method to change a structure.
	 * @param prop : the property
	 * @param child : the object
	 */
	protected final void fireStructureChange(
			final String prop, final Object child) {
      listeners.firePropertyChange(prop, null, child);
   }

   /**
	 * method to read an object.
	 * @param in : object in input
	 * @throws IOException : if pb
	 * @throws ClassNotFoundException : if pb
	 */
	private void readObject(
			final ObjectInputStream in)
				throws IOException, ClassNotFoundException {
      in.defaultReadObject();
      listeners = new PropertyChangeSupport(this);
   }

   /**
    * method to remove a listener.
    * @param listener : listener to remove
 	*/
   public final void removePropertyChangeListener(
		   final PropertyChangeListener listener) {
      listeners.removePropertyChangeListener(listener);
   }
}
