package com.bull.cimero.pluginEditor.editors.model.SEModel;


import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * The pipeline model class.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class Pipeline extends ServiceEngineModel {

	/** Serial UID. */
	private static final long serialVersionUID = 1L;
	/** Default name. */
	private static final String DEFAULTNAME = "myEnricher";
	/**the id of this element.*/
	private static final String ID = "ENRICHER";
	/**The location uri of the WS transformeur path.*/
	private static final String LOCATIONURI = "Location URI";
	/**The name of the destination service property.*/
	private static final String NAMEDEST = "destination service";
	/**The operation to call on the WS property.*/
	private static final String SOAPREQUEST = "Soap request";
	private static final String SOAP_ACTION = "SOAP_ACTION";
	private static final String SOAP_VERSION = "SOAP_VERSION";
	private static final String[] SOAP_VERSION_COMBO = {"1.2", "1.1"};

	/**
	 * default constructor.
	 */
	public Pipeline() {
		super(DEFAULTNAME, 1, DEFAULTOUTIN);
		//set the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(LOCATIONURI, "");
		setProperty(NAMEDEST, "");
		setProperty(SOAPREQUEST, 0);
		setProperty(SOAP_ACTION, "");
		setProperty(SOAP_VERSION, 0);
		setStandard(true);
	}

	/**
	 * get the location URI.
	 * @return Returns the locationURI.
	 */
	public final String getLocationURI() {
		return (String) getProperty(LOCATIONURI);
	}

	/**
	 * set the location URI.
	 * @param locationURIParam The locationURI to set.
	 */
	public final void setLocationURI(final String locationURIParam) {
		setProperty(LOCATIONURI, locationURIParam);
	}

	/**
	 * get the soapRequest.
	 * @return the soapRequest
	 */
	public final boolean isSoapRequest() {
		return ((Integer) getProperty(SOAPREQUEST) == 0);
	}

	/**
	 * set the soapRequest.
	 * @param soapRequestP the soapRequest to set
	 */
	public final void setSoapRequest(final boolean soapRequestP) {
		setProperty(SOAPREQUEST, soapRequestP);
	}
	
	public final String getSoapAction() {
		Object objValue = getProperty(SOAP_ACTION);
		return (objValue != null)? (String)objValue : "";
	}
	
	public final void setSoapAction(String action) {
		setProperty(SOAP_ACTION, action);
	}
	
	public final String getSoapVersion(){
		Integer intType = (Integer)getProperty(SOAP_VERSION);
		if (intType != null) {
			return SOAP_VERSION_COMBO[intType];
		}
		return SOAP_VERSION_COMBO[0];
	}	

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	@Override
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the enricher name cannot be null";
			returnResult = false;
		}
		//location URI verification
		if (getLocationURI().compareTo("") == 0) {
			msg += "\n2 - location URI => set the location URI";
			returnResult = false;
		}
		if (getNameFirstDestination().compareTo("") == 0) {
			msg += "\n4 - Destination service => the destination of this "
						+ "element cannot be null!";
			returnResult = false;
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	@Override
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, indice++ + " - name");
		addTextDescriptor(LOCATIONURI, indice++
				+ " - select the location URI of the WS");
		addComboBoxDescriptor(SOAPREQUEST, indice++ + " - soap request ? :",
						new String[]{"true", "false"});
		addComboBoxDescriptor(SOAP_VERSION, indice++ + " - soap version",
				SOAP_VERSION_COMBO);
		addTextDescriptor(SOAP_ACTION, indice++ + " - soap action");
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);

	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	@Override
	public final String getComponentId() {
		return ID;
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else if (id == SOAP_VERSION) {
			Object soapVersion = getProperty(SOAP_VERSION);
			return (soapVersion != null &&(soapVersion instanceof Integer))? soapVersion : 0;
		} else if (id == SOAP_ACTION) {
			return getSoapAction();
		} else {
			return super.getPropertyValue(id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		}
		if (id == SOAP_VERSION) {
			if (value == null) { setProperty(SOAP_VERSION, 0); }
			return;
		}
		if (id == SOAP_ACTION) {
			setSoapAction( value != null? (String)value : "");
			return;
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public final void resetPropertyValue(final Object id) {
	}

}
