package com.bull.cimero.pluginEditor.editors.model.SEModel;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * The splitter model class.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class Splitter extends ServiceEngineModel {

	/**serialization number.*/
	private static final long serialVersionUID = 1L;
	/**The xsl splitter path.*/
	private static final String XPATH = "XPATH rule";
	/**The name of the destination service property.*/
	private static final String NAMEDEST = "destination service";
	/**The default transformer name.*/
	private static final String DEFAULTNAME = "mySplitter";
	/**the id of this element.*/
	private static final String ID = "SPLITTER";

	/**
	 * The default constructor.
	 */
	public Splitter() {
		super(DEFAULTNAME, 1 , DEFAULTOUTIN);
		//add the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(XPATH, "");
		setProperty(NAMEDEST, "");
		setStandard(true);
	}

	/**
	 * XPath getter function.
	 * @return String
	 */
	public final String getXPath() {
		return (String) getProperty(XPATH);
	}

	/**
	 * path setter function.
	 * @param xPathParam : the path of the xslt file
	 */
	public final void setXPath(final String xPathParam) {
		setProperty(XPATH, xPathParam);
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the transformer name cannot be null";
			returnResult = false;
		}
		//xsl path verification
		if (getXPath().compareTo("") == 0) {
			msg += "\n2 - XPath => set the XPATH rule";
			returnResult = false;
		} else {
			//we test if XPath expression is valid
			if (!Ressources.testXPath(getXPath())) {
				msg += "\n2 - XPath rule => expression not valid.";
				returnResult = false;
			}
		}
		if (getNameFirstDestination().compareTo("") == 0) {
			msg += "\n3 - Destination service => the destination of this "
						+ "element cannot be null!";
			returnResult = false;
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + " \n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else {
			return super.getProperty((String) id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public void resetPropertyValue(final Object id) {
		// do nothing
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	protected final void createDescriptor() {
		int indice = 1;
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addTextDescriptor(NAME, indice++ + " - name");
		addTextDescriptor(XPATH, indice++ + " - select the XPath rule");
		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);

	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
}
