package com.bull.cimero.pluginEditor.editors.policies;

import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editpolicies.SelectionEditPolicy;

/**
 * policy to select connections.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ConnectionSelectionEditPolicy extends SelectionEditPolicy {

	/** width of an unselected arrow. */
	private static final int WIDTHUNSELECTEDARROW = 1;
	/** width of a selected arrow. */
	private static final int WIDTHSELECTEDARROW = 3;

	/**
	 * default constructor.
	 */
	public ConnectionSelectionEditPolicy() {
		super();
	}

	/**
	 * hide the current connection (bold).
	 */
	@Override
	protected final void hideSelection() {
		//method calld when a connection is now more selected
		getConnectionFigure().setLineWidth(WIDTHUNSELECTEDARROW);
	}

	/**
	 * show the current connection (bold).
	 */
	@Override
	protected final void showSelection() {
		//method calld when a connection is selected
		getConnectionFigure().setLineWidth(WIDTHSELECTEDARROW);
	}

	/**
	 * get the figure of the connection.
	 * @return PolylineConnection the figure
	 */
	protected final PolylineConnection getConnectionFigure() {
		return (PolylineConnection) (
				(GraphicalEditPart) getHost()).getFigure();
	}
}
