package com.bull.cimero.pluginEditor.editors.policies;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;
import com.bull.cimero.pluginEditor.editors.commands.NodeCreateCommand;
import com.bull.cimero.pluginEditor.editors.commands.NodeMoveCommand;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;

/**
 * XYLayoutEditPolicy is a policy that constrains figures
 * by a location and size.
 * This extends it with the capability to change the location
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class DiagramLayoutEditPolicy extends XYLayoutEditPolicy {

    /**
     * function to create child edit policy.
     * @param child : the child (not used)
     * @return EditPolicy : the new policy
     */
	@Override
	protected final EditPolicy createChildEditPolicy(final EditPart child) {
	   //we need to do this because the default is resizable
	   return new ResizableEditPolicy();
	}

	/**
	 * function to create add command.
	 * @param child : the child (not used)
	 * @param constraint : the constraint (not used)
	 * @return Command : the command created
	 */
	@Override
	protected final Command createAddCommand(
			final EditPart child, final Object constraint) {
      return null; //no support for adding
    }

	/**
	 * function to create change command.
	 * @param child : the child
	 * @param constraint : the constraint
	 * @return Command : the command created
	 */
	@Override
	protected final Command createChangeConstraintCommand(
			final EditPart child, final Object constraint) {
      //return a command to move the part to the
	  //location given by the constraint
      NodeMoveCommand locationCommand = new NodeMoveCommand();
      locationCommand.setNode((GeneriqueCimeroModel) child.getModel());
      locationCommand.setLocation(((Rectangle) constraint).getLocation());
      return locationCommand;
    }

   /**
    * function which is called when user
    * go to the draw vue after selected element of the palette.
    * @param request : the request sent by Eclipse
    * @return Command : the command to execute
    */
	@Override
   protected final Command getCreateCommand(final CreateRequest request) {
      //System.out.println(request.getNewObjectType().toString());
      NodeCreateCommand command = null;
      if (request.getNewObject() instanceof GeneriqueCimeroModel) {
    	  //add an Node object to the interface
    	  command = new NodeCreateCommand();
          command.setDiagram((Diagram) getHost().getModel());
          command.setNode((GeneriqueCimeroModel) request.getNewObject());
          Rectangle constraint = (Rectangle) getConstraintFor(request);
          command.setLocation(constraint.getLocation());
      }
	  return command;
   }

   /**
    * function which is called when user
    * wants to delete one dependant element.
    * @param request : the request sent by Eclipse
    * @return Command : the command to execute
    */
	@Override
   protected final Command getDeleteDependantCommand(final Request request) {

	  return null; //no support for deleting a dependant
   }
}
