package com.bull.cimero.pluginEditor.editors.policies;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;


import com.bull.cimero.pluginEditor.editors.commands.ConnectionCreateCommand;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;

/**
 * This class is the Policies which permit to make aa connection
 * between two figures.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */

public class NodeGraphicalNodeEditPolicy extends GraphicalNodeEditPolicy {

	/**
    * function used to get the complete command.
    * @param request : the request sent by Eclipse
    * @return Command : the command to execute
    */
	@Override
   protected final Command getConnectionCompleteCommand(
		   final CreateConnectionRequest request) {
		//the command which will permit to create a connection
		ConnectionCreateCommand command = null;
		//we test is the target element can receive input connection
		GeneriqueCimeroModel target =
			(GeneriqueCimeroModel) getHost().getModel();
		if (target.isInput()) {
	      command = (ConnectionCreateCommand) request.getStartCommand();
	      command.setTarget((GeneriqueCimeroModel) getHost().getModel());
		}
      return command;
   }

   /**
    * function used to get the create command.
    * @param request : the request sent by Eclipse
    * @return Command : the command to execute
    */
	@Override
   protected final Command getConnectionCreateCommand(
		   final CreateConnectionRequest request) {
		//we test if the selected source figure has an input connection
		//if no, we return null
		GeneriqueCimeroModel source =
			(GeneriqueCimeroModel) getHost().getModel();
		if (source.isOutput()) {
			//there is an output but if this component can send message
			// to more 1 element, the creation of connection is forbiddened
			if (source.getNbOutputs() == 1) {
			      ConnectionCreateCommand command =
			    	  			new ConnectionCreateCommand();
			      command.setSource(
			    		  (GeneriqueCimeroModel) getHost().getModel());
			      request.setStartCommand(command);
			      return command;
			} else {
				return null;
			}
		} else {
			return null;
		}
   }

   /**
    * function used to get the reconnect target command.
    * @param request : the request sent by Eclipse
    * @return Command : the command to execute
    */
	@Override
   protected final Command getReconnectTargetCommand(
		   final ReconnectRequest request) {
      return null;
   }

   /**
    * function used to get the reconnect source command.
    * @param request : the request sent by Eclipse
    * @return Command : the command to execute
    */
	@Override
   protected final Command getReconnectSourceCommand(
		   final ReconnectRequest request) {
      return null;
   }
}
