package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory;

import java.lang.reflect.Constructor;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.figure.BCFigure.BCConcreteFigure;
import com.bull.cimero.pluginEditor.editors.figure.SEFigure.SEConcreteFigure;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.BindingComponentModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.utils.classLoader.CimeroClassLoader;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * This class is the factory of the servicemix factory model.
 * @author Cedric Mouilleron
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public final class FactoryFactory {

//	---------------------------------------------
	//CONSTANTES
	/**Current factory package.*/
	private static final String CURRENTPACKAGE =
		"com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory";
	/**LW factory package.*/
	private static final String LWFACTORYPACKAGE = "lWFactory";
	/**Standard factory package.*/
	private static final String STDFACTORYPACKAGE = "standardFactory";
	/**BC package factory.*/
	private static final String BC = "BC";
	/**SE package factory.*/
	private static final String SE = "SE";
	/**The suffixe factory.*/
	private static final String SUFFIXEFACTORY = "Factory";

	/**
	 * private constructor.
	 *
	 */
	private FactoryFactory() {
	}

	public static String getFactoryClassNameFromConfiguration(GeneriqueCimeroModel model){
		String customerComponentId = model.getCustomComponentId();
		Ressources ressource = new Ressources();
		String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/CustomComponents.xml");
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			
			
			NodeList nodeList = ressource.getXPathNodeSet("/COMPONENTS/COMPONENT"); 
			Node node = null;
			NamedNodeMap attributes = null;
		
			String idTemp = null;
			for (int i = 0; i < nodeList.getLength(); i++) {
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				
				 idTemp = attributes.getNamedItem("id").getNodeValue();
			
				 if (idTemp.equalsIgnoreCase(customerComponentId)){
					 return attributes.getNamedItem("factoryClass").getNodeValue();
				 }
			}
		}
		return null;
	}
	
	
	
	
	
	
	
	public static GenericServiceMixFactory createFactory(
			final GeneriqueCimeroModel cimeroCompo,
			final NameSpaceClass nms,
			final Boolean mepMode) throws Exception {

		if (cimeroCompo.isCustomComponent()){
			String fullPackageClass = getFactoryClassNameFromConfiguration(cimeroCompo);
			GenericServiceMixFactory myFact = null;
			
			try {
				Constructor  aConstructor = null;
				// Try if this factory has a constructor that take the namespace as input parameter
				// This is the case of ServiceEngines, Lw Service Engines, and particular binding components that
				// are input binding components
				
				CimeroLog.infoLog("FactoryFactory try to find a costructor for class["+fullPackageClass+"] that take a parameter of type NameSpaceClass ");
				Class [] constructorParams = new Class[1];
				constructorParams[0] = NameSpaceClass.class;
				aConstructor = CimeroClassLoader.getConstructorWithParameters(fullPackageClass, constructorParams);
				
				if (aConstructor != null){
					CimeroLog.infoLog("FactoryFactory Found a costructor for class["+fullPackageClass+"] that take a parameter of type NameSpaceClass ");
					myFact = (GenericServiceMixFactory)aConstructor.newInstance(new Object[] {nms});
				}else{
					//Try if this factory has a constructor that take the namespace and  Boolean as input
					CimeroLog.infoLog("FactoryFactory try to find a costructor for class["+fullPackageClass+"] that take two parameter of type NameSpaceClass and Boolean ");
					constructorParams = new Class[2];
					constructorParams[0] = NameSpaceClass.class;
					constructorParams[1] = Boolean.class;
					aConstructor = CimeroClassLoader.getConstructorWithParameters(fullPackageClass, constructorParams);
					myFact = (GenericServiceMixFactory)aConstructor.newInstance(new Object[] {nms, mepMode});
				}
				
				
				return myFact;
			}catch (ClassNotFoundException cnfe) {
				throw new Exception("ClassNotFoundExcetpio "+ cnfe.getMessage());
			} 
			
		}else{
			return originalCreateFactory(cimeroCompo, nms, mepMode);
		}
	}

	/**
	 * This function create the factory associated with the cimero
	 * component in parameter. This factory create the factory with
	 * the name and the type of the cimero component.
	 *
	 * @param cimeroCompo : the cimero component
	 * @param nms : the namespace using by the factory
	 * @param mepMode : the mep mode for BC.
	 * @return the factory associates at the cimero component.
	 * @throws Exception if the factory cannot be created.
	 */
	public static GenericServiceMixFactory originalCreateFactory(
			final GeneriqueCimeroModel cimeroCompo,
			final NameSpaceClass nms,
			final Boolean mepMode) throws Exception {
		
		GenericServiceMixFactory myFact = null;
		//test if the component is not null
		if (cimeroCompo == null) {
			throw new Exception("The cimero component cannot be null");
		}

		//set the current package.
		String currentPackage = CURRENTPACKAGE;

		String fullPackageClass = currentPackage;

		//get the cimero model name.
		String simpleName = cimeroCompo.getClass().getSimpleName();
		//test if the component is standard
		
		if (cimeroCompo.isStandard()) {
			//standard factory
			fullPackageClass += "." + STDFACTORYPACKAGE;
			//test BC or SE
			
			if (cimeroCompo instanceof BindingComponentModel) {
				//BC component.
				fullPackageClass += "." + BC;
			} else {
				//SE Component.
				fullPackageClass += "." + SE;
			}
		} else {
			//lw factory
			fullPackageClass += "." + LWFACTORYPACKAGE;
			if (cimeroCompo instanceof BindingComponentModel) {
				//BC component.
				fullPackageClass += "." + BC;
			} else {
				//SE Component.
				fullPackageClass += "." + SE;
			}
		}

		fullPackageClass += "." + simpleName + SUFFIXEFACTORY;
		//create the factory
		if (cimeroCompo instanceof BindingComponentModel) {
			try {
				myFact = (GenericServiceMixFactory) CimeroClassLoader
					.newInstance(fullPackageClass, new Object[] {nms, mepMode});
			} catch (Exception e) {
				CimeroLog.errorLog("the standard factory"
						+ fullPackageClass + "cannot be create", e);
				return null;
			}
		} else {
			try {
				myFact = (GenericServiceMixFactory) CimeroClassLoader
					.newInstance(fullPackageClass, new Object[] {nms});
			} catch (Exception e) {
				CimeroLog.errorLog("the LW Factory"
						+ fullPackageClass + "cannot be create", e);
				return null;
			}
		}
		return myFact;
	}
}
