package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.lWFactory.BC;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.FTP;
import com.bull.cimero.pluginEditor.generator
			.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator
			.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator
			.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator
			.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator
			.serviceMixGenerator.model.global.PropertyClass;

/**
 * This class is the ftp factory. It is used to generate the servicemix model
 * that represents a ftp component.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class FTPFactory extends BCLWFactory {

	/**XPATH class expression.*/
	private static final String XPATHEXPR =
		"org.apache.servicemix.expression.JaxenStringXPathExpression";
	/**The class using by the ftp poller into servicemix.*/
	private static final String CLASSIN =
		"org.apache.servicemix.components.net.FTPPoller";
	/**The class using by ftp sender into servicemix.*/
	private static final String CLASSOUT =
		"org.apache.servicemix.components.net.FTPSender";
	/**The client poller property name.*/
	private static final String CLIENTPOLL = "clientPool";
	/**The class using by the poller client.*/
	private static final String CLASSCLIENTPOLL =
		"org.apache.servicemix.components.net.FTPClientPool";
	/**the host property name.*/
	private static final String HOST = "host";
	/**the username property name.*/
	private static final String USERNAME = "username";
	/**the password property name.*/
	private static final String PASSWORD = "password";
	/**the workmanager property name.*/
	private static final String WORKMANAGER = "workManager";
	/**The workmanager class.*/
	private static final String WORKMANAGERCLASS =
		"org.jencks.factory.WorkManagerFactoryBean";
	/**The workmanager bean property name.*/
	private static final String THREADPOLLSIZE = "threadPoolSize";
	/**The value of the threadPoolSize property.*/
	private static final String THREADPOLLSIZEVALUE = "30";
	/**the path property name.*/
	private static final String PATH = "path";
	/**the period property name.*/
	private static final String PERIOD = "period";
	/**the name of the marshaler property.*/
	private static final String MARSHALER = "marshaler";
	/**The class using by the servicemix marshaler.*/
	private static final String MARSHALERCLASS =
		"org.apache.servicemix.components.util.DefaultFileMarshaler";
	/**The filename property name of the marshaler.*/
	private static final String FILENAME = "fileName";

	/**constructor with the mep mode in parameter.
	 * @param mepMode : the mep mode of this component. True if INMEP
	 * @param ref : the reference of the current namespace.
	 * */
	public FTPFactory(final NameSpaceClass ref, final Boolean mepMode) {
		super(ref, mepMode);
	}


	/**
	 * This function create the ftp servicemix model.
	 * @param elt : the element model that will be used to generate
	 * 		this activation spec.
	 * @param force : false if the validation must to be do.
	 * @return the activation spec created
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean force) throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of mail model
		if ((elt != null) && (elt instanceof FTP)) {
			if ((elt.isValide(false)) || (force)) {
				FTP monFtp = (FTP) elt;
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				//set the name
				myActi.setComponentName(monFtp.getName());
				//set the service
				myActi.setServiceName(monFtp.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				//we test if this element is in in mep mode or out mep mode.
				if (isInMepMode()) {
					//it is an INMEP => RSS POLLER
					//set the destination service
					setDestinationService(myActi , monFtp);

				}
				//create the components
				ComponentClass compo = createComponent(monFtp);
				myActi.setComponent(compo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");

	}

	/**
	 * This function creates the component of the servicemix ftp element.
	 * @param monFtp : the cimero model element
	 * @return the instance of the component class.
	 */
	private ComponentClass createComponent(final FTP monFtp) {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = null;
		if (monFtp.isInput()) {
			monB = createBeanFTPOUTMEP(monFtp);
		} else {
			monB = createBeanFTPINMEP(monFtp);
		}
		monCompo.setMonBean(monB);
		return monCompo;
	}

	/**
	 * This function creates the ftp bean ftp in OUTMEP mode in the
	 * serviceMix model.
	 * @param monFtp : the cimero element
	 * @return the bean ftp.
	 */
	private LWBeanClass createBeanFTPOUTMEP(final FTP monFtp) {
		LWBeanClass myBean = new LWBeanClass(CLASSOUT);
		//add the client poll property
		myBean.addProperty(createClientPollProperty(monFtp) , "");
//		//add the marshaler property.
		myBean.addProperty(createMarshalerProperty(monFtp) , "");
		return myBean;
	}

	/**
	 * This function create a marshaler property used to the
	 * servicemix ftp sender component.
	 * @param monFtp : the cimero object that present the ftp component.
	 * @return the marshaler property.
	 */
	private PropertyClass createMarshalerProperty(final FTP monFtp) {
		PropertyClass marshalerProp = new PropertyClass(MARSHALER);
		LWBeanClass myBean = new LWBeanClass(MARSHALERCLASS);
		//add the file name
		String xpathRule = "concat(''," + monFtp.getXpathTitle() + ",'.txt')";
		myBean.addProperty(createBeanConstructor(FILENAME , XPATHEXPR
				, xpathRule) , "");
		marshalerProp.setBean(myBean);
		return marshalerProp;
	}

	/**
	 * This function creates the bean using by the ftp in INMEP mode in the
	 * serviceMix model.
	 * @param monFtp : the cimero element
	 * @return the bean ftp.
	 */
	private LWBeanClass createBeanFTPINMEP(final FTP monFtp) {
		LWBeanClass beanRSS = new LWBeanClass(CLASSIN);
		//add the client poll property
		beanRSS.addProperty(createClientPollProperty(monFtp) , "");
		//add the workmanager
		PropertyClass workMan = new PropertyClass(WORKMANAGER);
		LWBeanClass workBean = new LWBeanClass(WORKMANAGERCLASS);
		workBean.addProperty(createProperty(THREADPOLLSIZE ,
				THREADPOLLSIZEVALUE) , "");
		workMan.setBean(workBean);
		beanRSS.addProperty(workMan ,  "");
		//add the path
		if (monFtp.getPath().compareTo("") != 0) {
			beanRSS.addProperty(createProperty(PATH , monFtp.getPath()) ,
					"");
		}
		//add the period
		beanRSS.addProperty(createProperty(PERIOD , "1000"), "");
		return beanRSS;
	}

	/**
	 * this function create the property of the client poller
	 * in a servicemix ftp.
	 * @param monFtp : the cimero object that represent a ftp server
	 * @return the servicemix property.
	 */
	private PropertyClass createClientPollProperty(final FTP monFtp) {
		//add the client poll property
		PropertyClass clienProperty = new PropertyClass(CLIENTPOLL);
		LWBeanClass clientPoll = new LWBeanClass(CLASSCLIENTPOLL);
		clientPoll.addProperty(createProperty(HOST , monFtp.getHostName())
				, "");
		clientPoll.addProperty(createProperty(USERNAME , monFtp.getLogin())
				, "");
		clientPoll.addProperty(createProperty(PASSWORD , monFtp.getPassword()),
				"");
		clienProperty.setBean(clientPoll);
		return clienProperty;
	}

}
