package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.standardFactory.BC;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.FileBC;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.StandardComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardFileClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJBIClass;

/**
 * this class is the factory of standard http element in ServiceMix.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 *  @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class FileBCFactory extends BCStdFactory {

	//-------------------------------------
	//CONSTANTES
	/**The http namespace name.*/
	public static final String NAMESPACENAME = "file";
	/**The http namespace value.*/
	public static final String NAMESPACEVALUE =
		"http://servicemix.apache.org/file/1.0";


	//----------------------------------------------
	//CONSTRUCTOR
	/**
	 * FileBCFactory constructor.
	 * @param ref : the namespace using by this component.
	 * @param mepMode : true if the element has an in mep mode.
	 */
	public FileBCFactory(final NameSpaceClass ref, final Boolean mepMode) {
		super(ref, mepMode);
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory#createJBIElement(
	 * com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel)
	 * @param elt : the cimero http object.
	 * @return the standard http element.
	 * @throws Exception : if the generation return an error
	 */
	public final StandardJBIClass createJBIElement(
			final GeneriqueCimeroModel elt) throws Exception {
		
		StandardFileClass myFile = null;
		if (elt instanceof FileBC) {
			FileBC myFileBC = (FileBC) elt;
			if (isInMepMode()){
				GeneriqueCimeroModel destElt = getDestination(elt);
				if (destElt != null){
					NameSpaceClass destinationNms = getNamespace(destElt);
					myFile = new StandardFileClass(StandardFileClass.MEP_IN
								, destElt.getName()
								, destinationNms
								, new NameSpaceClass(getNamespaceName() ,
													getNamespaceValue())
								, myFileBC.getName()
								, myFileBC);
				}else {
					throw new Exception("The destination component of the "
							+ myFileBC.getName() + " object doesn't exist");
				}
			}else{
				NameSpaceClass nms = getNamespace(myFileBC);
				myFile = new StandardFileClass(StandardFileClass.MEP_OUT
						, myFileBC.getName()
						, nms
						, new NameSpaceClass(getNamespaceName() ,
											getNamespaceValue())
						, myFileBC.getName()
						, myFileBC);
			}
		}
		return myFile;
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory#createXMLJBIElement(
	 * com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel)
	 * @param elt : the cimero http object.
	 * @param isForce : true is the function not do th validation.
	 * @return the ActivationSpecClass element.
	 * @throws Exception : if there is a problem during the model generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt, final Boolean isForce)
			throws Exception {
		if (getNamespaceRef() == null) {
			throw new Exception("namespaceclass cannot be null");
		}
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of mail model
		if ((elt != null) && (elt instanceof FileBC)) {
			if ((elt.isValide(false)) || (isForce)) {
				FileBC monFileBC = (FileBC) elt;
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				myActi.setComponentName(monFileBC.getName());
				myActi.setServiceName(monFileBC.getName(),
					new NameSpaceClass(GeneriqueLWfactory.NAMESPACENAME ,
							GeneriqueLWfactory.NAMESPACEVALUE));
				//we test if this element is in in mep mode or out mep mode.
				if (isInMepMode()) {
					//set the destination service
					setDestinationService(myActi , monFileBC);
				}
				//create the components
				ComponentClass compo = createComponent(monFileBC);
				myActi.setComponent(compo);
				return myActi;
			}
		 } else {
			throw new Exception("The generique cimero object is not valide");
		}
		return null;
	}

	/**
	 * creates the servicemix WS component.
	 * @param monWS : the cimero model element
	 * @return the instance of the component class.
	 * @throws Exception : if the generation return an error
	 */
	private ComponentClass createComponent(final FileBC monFileBC)
		throws Exception {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		StandardComponentClass monS = null;
		monS = createStandardComponentMEP(monFileBC);
		monCompo.setMonComponentStandard(monS);
		return monCompo;
	}

	/**
	 * creates the standard components.
	 * @param monWS : the cimero WS
	 * @return the standard component class.
	 * @throws Exception : if the generation return an error.
	 */
	private StandardComponentClass createStandardComponentMEP(
			final FileBC monFileBC) throws Exception {
		StandardComponentClass monStd = new StandardComponentClass(
				new NameSpaceClass(getNamespaceName() , getNamespaceValue()));
		monStd.setMyJBICompo(createJBIElement(monFileBC));
		return monStd;
	}

	/**
	 * @return the namespace name.
	 */
	public final String getNamespaceName() {
		return NAMESPACENAME;
	}

	/**
	 *@return the namespace value.
	 */
	public final String getNamespaceValue() {
		return NAMESPACEVALUE;
	}

}
