package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.standardFactory;

import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.GenericServiceMixFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.RouteurFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * This class presents a standard JBI object. Il contains the functions
 * necessary to generate a standard jbi object.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class GeneriqueStandardFactory
	implements GenericServiceMixFactory, StandardComponentFactory {

	/**
	 * @return the name of the namespace.
	 */
	public abstract String getNamespaceName();

	/**
	 * @return the value of the namespace.
	 */
	public abstract String getNamespaceValue();

	/**
	 * This function is used to set at the activation spec, the destination
	 * service.
	 * @param myActi : the current activation spec
	 * @param elt : the element of the cimero model.
	 * @throws Exception if the destination service is not find.
	 */
	protected final void setDestinationService(final ActivationSpecClass myActi,
			final GeneriqueCimeroModel elt) throws Exception {
		//we get the destination element
		GeneriqueCimeroModel destElt = getDestination(elt);
		//the destination service has been finded.
		if (destElt != null) {
			//if this is a routeur, the using namespace is not the same
			//as the others
			NameSpaceClass usingNamespace = getNamespace(destElt);

			//we set the destination service of this activation spec
			myActi.setDestinationService(destElt.getName() , usingNamespace);

		} else {
			throw new Exception("the destination service of the mail component"
					+ "cannot be find !");
		}
	}

	/**
	 * return the destination element of the element in parameter. If
	 * there is no destination element, this function return false.
	 * @param elt : the reference object.
	 * @return the destination object of the object in parameter
	 */
	public final GeneriqueCimeroModel getDestination(
			final GeneriqueCimeroModel elt) {
		GeneriqueCimeroModel destElt = null;
		if (elt.getOutgoingConnections().size() != 0) {
			Connection modelTmp =
				(Connection) elt.getOutgoingConnections().get(0);
			if (modelTmp != null) {
				destElt = modelTmp.getTarget();
			}
		}
		return destElt;
	}

	/**
	 * return the namespace using by the object in parameter.
	 * @param destElt : the object reference.
	 * @return the namespace using by this object
	 */
	public final NameSpaceClass getNamespace(
			final GeneriqueCimeroModel destElt) {
		NameSpaceClass usingNamespace = null;
		/*
		if (destElt instanceof Routeur) {
			usingNamespace = new NameSpaceClass(
					RouteurFactory.NAMESPACENAME ,
					RouteurFactory.NAMESPACEVALUE);
		}*/ 
			usingNamespace = new NameSpaceClass(
				GeneriqueLWfactory.NAMESPACENAME,
				GeneriqueLWfactory.NAMESPACEVALUE);

		return usingNamespace;
	}
}
