package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.jarGenerator;

import it.eng.spagosoa.metadb.api.IMetadbApi;
import it.eng.spagosoa.metadb.model.ServiceAssembly;
import it.eng.utils.EndpointModel;
import it.eng.utils.MepConstants;
import it.eng.utils.ProcessModel;
import it.eng.utils.ProcessModelsUtils;
import it.eng.utils.PropertyModel;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.emf.codegen.jet.JETException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IFileEditorInput;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.CimeroRegistry;
import com.bull.cimero.pluginEditor.editors.DiagramEditor;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur.Rule;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.GenericGenerator;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.BC.MailFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.BC.ScreenOutputFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.IModelServiceMix;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.DroolsRouterModel;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.PropertyClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.DroolsRouterModel.ServiceMixRule;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.IBindingComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.IServiceEngineClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardEIPAggregator;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardEIPMessageFilter;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardEIPPipelineClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardEIPSplitter;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardFileClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardHttpClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJBIClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJDBCClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardRouterClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardTCPClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.ServiceAssemblyClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.ServiceUnitClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.StandardJBIJARClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.StandardLWComponentClass;
import com.bull.cimero.pluginEditor.utils.exception.CimeroGenerateJarException;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.zip.ZipUtils;

/**
 * class used to generate JBI package from a cimero file.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class JarGenerator extends GenericGenerator {

	/** constant for yes button in the msgbox. */
	public static final int YESBUTTON = 64;
	/**a boolean to determine if the validation must to be do.*/
	private boolean force = false;

	/**
	 * copy constructor.
	 * @param diagramParam : diagram to use
	 * @param fileParam : IFile associated to the model
	 */
	public JarGenerator(final Diagram diagramParam
			, final IFile fileParam) {
		super(diagramParam, fileParam);
	}
	
	public final ServiceAssembly isSARegisteredInDatabase() throws CimeroGenerateJarException, IOException {
		ServiceAssemblyClass smModel = (ServiceAssemblyClass) generateModel();
		if (smModel == null) {
			throw new CimeroGenerateJarException("the generate model is null");
		}
		
		IMetadbApi metadb = CimeroEditorPlugin.getDefault().getMetadbApi();
		metadb.open();
		ServiceAssembly sa = metadb.getServiceAssemblyByName(smModel.getServiceAssemblyName());
		metadb.close();
		return sa;
	}


	/**
	 * method used to generate the package.
	 * @see com.bull.cimero.pluginEditor.generator
	 * .serviceMixGenerator.model.GenericGenerator
	 * #generateConfiguration()
	 * @return boolean
	 * @throws CimeroGenerateJarException : throw if there is a problem during
	 * the generation configuration
	 * @throws IOException : throw if there is a problem with the generated file
	 */
	
	public final boolean storeProcessModel(ProcessModel aProcessModel) throws CimeroGenerateJarException, IOException {
		
		
		IMetadbApi metadb = CimeroEditorPlugin.getDefault().getMetadbApi();
		try{
			ProcessModelsUtils.storeProcessModel(metadb, aProcessModel);
			return true;
		}catch(Throwable t){
			t.printStackTrace();
			return false;
		}
	}
		
	
	public void inferEndpointListForStandardServiceUnit(ProcessModel aProcessDatabaseModel, ServiceUnitClass su){
		StandardJBIJARClass standardJBI = su.getMonStandard();
		List<StandardJBIClass>  stdJbiComponents = standardJBI.getJbiCompo();
		

		NameSpaceClass nms = null;
		String endpointNameSpace = null;
		
		for (StandardJBIClass stdJbiClass : stdJbiComponents){
		
			String endpoint = null;
			nms = stdJbiClass.getServiceNamespace();
            endpointNameSpace = nms.getNameSpaceValue();
            endpoint = stdJbiClass.getEndpointValue();
            String typeService = null;
            if (stdJbiClass instanceof IServiceEngineClass) {
            	typeService = "SERVICE ENGINES";
				
			}else if (stdJbiClass instanceof IBindingComponentClass) {
				typeService = "BINDING COMPONENTS";
			}
            
            String componentName = su.getSuComponentName();
            
            if (componentName.equals(JarModelFactory.EIPCOMPONENTNAME)){
            	// The EIP Service Units contains different components so we need to distinguish
            	if (stdJbiClass instanceof StandardEIPSplitter) {
            		componentName += " XPathSplitter ";
            	}else if (stdJbiClass instanceof StandardEIPPipelineClass) {
            		componentName += " Pipeline  ";
            	}else if (stdJbiClass instanceof StandardEIPAggregator) {
            		componentName += " Aggregator  ";
            	}if (stdJbiClass instanceof StandardEIPMessageFilter) {
            		componentName += " Message Filter  ";
            	}
            }
            
            List<PropertyModel> propertiesModel = stdJbiClass.getPropertiesToStore();
            aProcessDatabaseModel.addEndpointWithProperties(endpointNameSpace, endpoint, componentName, typeService, propertiesModel);
            if (stdJbiClass  instanceof StandardTCPClass ) {
            	boolean isInOutMep = ((StandardTCPClass)stdJbiClass).getDefaultMep().equalsIgnoreCase(MepConstants.DEFINOUTMEP);
            	boolean isCreatedByPipeline = ((StandardTCPClass)stdJbiClass).isCreatedByPipeline();
            	if (!isCreatedByPipeline && isInOutMep){
            		CimeroLog.infoLog("TCP IP BC Endpoint (BC) ["+endpoint+"] is not created by pipeline and has Inoutmep ");
            		EndpointModel epModel = aProcessDatabaseModel.getEndpointMap().get(endpoint);
            		epModel.setStart(true);
            		epModel.setEnd(true);
            	}else{
            		CimeroLog.infoLog("TCP IP BC Endpoint (BC) ["+endpoint+"] is created by pipeline ");
            	}
            }
            if (stdJbiClass  instanceof StandardHttpClass ) {
            	boolean isInOutMep = ((StandardHttpClass)stdJbiClass).getDefaultMepValue().equalsIgnoreCase(MepConstants.DEFINOUTMEP);
            	boolean isCreatedByPipeline = ((StandardHttpClass)stdJbiClass).isCreatedByPipeline();
            	if (!isCreatedByPipeline && isInOutMep){
            		CimeroLog.infoLog("HTTP BC Endpoint (BC) ["+endpoint+"] is not created by pipeline and has Inoutmep ");
            		EndpointModel epModel = aProcessDatabaseModel.getEndpointMap().get(endpoint);
            		epModel.setStart(true);
            		epModel.setEnd(true);
            	}else{
            		CimeroLog.infoLog("HTTP BC Endpoint (BC) ["+endpoint+"] is created by pipeline ");
            	}
            }
		}
	}
	
	public void inferEndpointListForLightWeightServiceUnit(ProcessModel aProcessDatabaseModel, ServiceUnitClass su){
		String suComponentName = su.getSuComponentName();
		StandardLWComponentClass standardLWComponentClass = su.getLW();
		
		List < ActivationSpecClass > activationSpecLst = standardLWComponentClass.getActivationSpecLst();
		String endpoint = null;
	
		NameSpaceClass nms = null;
		String endpointNameSpace = null;
		String componentName =  null;
		
		
		List<String> lwBindingComponents = new ArrayList<String>();
		lwBindingComponents.add( ScreenOutputFactory.CLASS );
		lwBindingComponents.add( MailFactory.CLASSOUT );
		lwBindingComponents.add( MailFactory.CLASSIN );
		
		String typeService =  "SERVICE ENGINES";
		
		for (ActivationSpecClass actSpec : activationSpecLst){
			endpoint =  actSpec.getServiceName();
			nms = actSpec.getServiceNameSpace();
			
			String componentClassName = actSpec.getComponent().getMonBean().getClassName();
	
			if (componentClassName.contains(componentClassName))
				typeService = "BINDING COMPONENTS";
			
			componentName = suComponentName + " " + componentClassName.substring(componentClassName.lastIndexOf(".") + 1);
			
			List<PropertyClass> propsOfLwComponent = actSpec.getComponent().getMonBean().getlstProp();
			List<PropertyModel> propsModel = new ArrayList<PropertyModel>();
			for (PropertyClass propClass : propsOfLwComponent){
				if (propClass.isValue()){
					List<String> valuesStr = propClass.getLstValue();
					if (valuesStr.size() == 1){
						propsModel.add(createPropertyModel(propClass.getPropertyName(),valuesStr.get(0)));
						
					}else{
						StringBuffer strBufferValues = new StringBuffer();
						strBufferValues.append("[");
						for (int kProp = 0; kProp < valuesStr.size(); kProp++){
							strBufferValues.append(kProp+":"+valuesStr.get(kProp));
							if ((kProp + 1) != valuesStr.size())
								strBufferValues.append(",");
						}
						strBufferValues.append("]");
						propsModel.add(createPropertyModel(propClass.getPropertyName(),strBufferValues.toString()));
						
					}
				}else if (propClass.isRef()){
					PropertyModel prop = createPropertyModel(propClass.getPropertyName(),propClass.getRefValue());
					propsModel.add(prop);
				}
			}
			
			aProcessDatabaseModel.addEndpointWithProperties(nms.getNameSpaceValue(), endpoint, componentName,typeService, propsModel);
		}
	}
	
	public void inferTransitionsForStandardServiceUnit(ProcessModel aProcessDatabaseModel, ServiceUnitClass su, DroolsRouterModel droolsModel){
		String endpointSource = null;
		String endpointDest = null;
		StandardJBIJARClass standardJBI = su.getMonStandard();
		List<StandardJBIClass>  stdJbiComponents = standardJBI.getJbiCompo();
		
		for (StandardJBIClass stdJbiClass : stdJbiComponents){
			
			if (stdJbiClass instanceof StandardHttpClass) {
				if (((StandardHttpClass)stdJbiClass).getProviderRole().equalsIgnoreCase(StandardHttpClass.CONSUMER)){
					endpointSource = ((StandardHttpClass)stdJbiClass).getEndpointValue();
					endpointDest = ((StandardHttpClass)stdJbiClass).getServiceName();
					
					aProcessDatabaseModel.addFlow(endpointSource, endpointDest);
				}
				
			}
			if((stdJbiClass instanceof StandardFileClass) && ((StandardFileClass)stdJbiClass).getDefaultMepValue().equalsIgnoreCase("INPUT")){
                    endpointSource = ((StandardFileClass)stdJbiClass).getEndpointValue();
                    endpointDest = ((StandardFileClass)stdJbiClass).getServiceName();
                   
                    aProcessDatabaseModel.addFlow(endpointSource, endpointDest);
			}

			if (stdJbiClass instanceof StandardEIPSplitter) {
				endpointSource = ((StandardEIPSplitter)stdJbiClass).getEndpointValue();
				endpointDest = ((StandardEIPSplitter)stdJbiClass).getTargetName();
				
				aProcessDatabaseModel.addFlow(endpointSource, endpointDest);
			}
			if (stdJbiClass instanceof StandardEIPPipelineClass) {
				endpointSource = ((StandardEIPPipelineClass)stdJbiClass).getEndpointValue();
				endpointDest = ((StandardEIPPipelineClass)stdJbiClass).getTransformeurServiceName();
				
				
				aProcessDatabaseModel.addFlow(endpointSource, endpointDest);
				
				
				aProcessDatabaseModel.addFlow(endpointDest, endpointSource);
				
				String pipelineEndpointDest = ((StandardEIPPipelineClass)stdJbiClass).getTargetServiceName();
			
				aProcessDatabaseModel.addFlow(endpointSource,pipelineEndpointDest);
				
			}
			if (stdJbiClass instanceof StandardEIPAggregator ) {
				
					endpointSource = ((StandardEIPAggregator)stdJbiClass).getEndpointValue();
					endpointDest = ((StandardEIPAggregator)stdJbiClass).getServiceName();
					
					aProcessDatabaseModel.addFlow(endpointSource, endpointDest);
				
			}
			if (stdJbiClass instanceof StandardEIPMessageFilter ) {
				endpointSource = ((StandardEIPMessageFilter)stdJbiClass).getEndpointValue();
				endpointDest = ((StandardEIPMessageFilter)stdJbiClass).getServiceName();
				
				aProcessDatabaseModel.addFlow(endpointSource, endpointDest);
			}
			if (stdJbiClass instanceof StandardTCPClass ) {
				if (((StandardTCPClass)stdJbiClass).getRole().equalsIgnoreCase(StandardTCPClass.CONSUMER)){
					endpointSource = ((StandardTCPClass)stdJbiClass).getEndpointValue();
					endpointDest = ((StandardTCPClass)stdJbiClass).getTargetServiceName();
				
					aProcessDatabaseModel.addFlow(endpointSource, endpointDest);
				}
			}
			
			if (stdJbiClass instanceof StandardJDBCClass ) {
				endpointSource = ((StandardJDBCClass)stdJbiClass).getEndpointValue();
				endpointDest = ((StandardJDBCClass)stdJbiClass).getServiceName();
				
				aProcessDatabaseModel.addFlow(endpointSource, endpointDest);
			}
			if (stdJbiClass instanceof StandardRouterClass){
				endpointSource = ((StandardRouterClass)stdJbiClass).getEndpointValue();	
				for (int i=0; i < droolsModel.lstRuleSize(); i++){
						ServiceMixRule smxRule = droolsModel.getRuleElt(i);
						
						if (smxRule.getName().equalsIgnoreCase(endpointSource)){
							List<Rule> rules = smxRule.getMyRule();
							for (Rule rl : rules){
								endpointDest = rl.getConn().getTarget().getName();
								
								aProcessDatabaseModel.addFlow(endpointSource, endpointDest);
							}
						}
					}
			 }
		}
	}
	
	public void inferTransitionForLightWeightServiceUnit(ProcessModel aProcessDatabaseModel, ServiceUnitClass su){
		StandardLWComponentClass standardLWComponentClass = su.getLW();
		
		List < ActivationSpecClass > activationSpecLst = standardLWComponentClass.getActivationSpecLst();
	
		String endpointSource = null;
		String endpointDest = null;
		for (ActivationSpecClass actSpec : activationSpecLst){
			endpointSource =  actSpec.getServiceName();
			endpointDest = null;
			
				endpointSource =  actSpec.getServiceName();
				endpointDest = actSpec.getDestinationServiceName();
				
				aProcessDatabaseModel.addFlow(endpointSource, endpointDest);
			
		}
	}
	
	public ProcessModel inferProcessModelFromSpagicStudio(ServiceAssemblyClass smModel){
		
		ProcessModel processModel = new ProcessModel();
		processModel.setProcessName(smModel.getServiceAssemblyName());
		
		CimeroLog.infoLog(" -> Get Process Model From Spagic Studio Diagram...... ");
		DroolsRouterModel droolsModel = generateDroolsModel();
		
		List <ServiceUnitClass> sus = smModel.getLstSU();
		
		CimeroLog.infoLog(" -> Analyzing Service Units to determine Endpoint List...... ");
		for (ServiceUnitClass su : sus){
			CimeroLog.infoLog(" -> Service Unit ["+ su.getSuName() + "].... ");
			if (su.isStandard()){
				inferEndpointListForStandardServiceUnit(processModel, su);		
			}else{
				inferEndpointListForLightWeightServiceUnit(processModel, su);
			}
		}
		
		List <ServiceUnitClass> suList = smModel.getLstSU();
		
		
		CimeroLog.infoLog(" -> Analyzing Service Units to determine Transitions...... ");
		for (ServiceUnitClass su : sus){
			
			if (su.isStandard()){
				inferTransitionsForStandardServiceUnit(processModel, su,droolsModel);
			}else{
				inferTransitionForLightWeightServiceUnit(processModel, su);
			}
		}
		
		processModel.calculateStartEndpoints();
		
		return processModel;

		
	}//---
	
	

	
	
	/**
	 * method used to generate the package.
	 * @see com.bull.cimero.pluginEditor.generator
	 * .serviceMixGenerator.model.GenericGenerator
	 * #generateConfiguration()
	 * @return boolean
	 * @throws CimeroGenerateJarException : throw if there is a problem during
	 * the generation configuration
	 * @throws IOException : throw if there is a problem with the generated file
	 */
	@Override
	public final boolean generateConfiguration()
	throws CimeroGenerateJarException, IOException {
		//test if the file already exists
		if (testFileAlreadyExist()) {
			//already exists
			//--> tell user to replace or not the file
			Shell monShell = new Shell();
			MessageBox msg = new MessageBox(monShell, SWT.YES | SWT.NO);
			msg.setMessage("The JAR file already exists !"
					+ " ! Do you want to replace it ?");
			msg.setText("JAR file already exists");
			//the messagebox is open
			if (msg.open() != YESBUTTON) {
				//if the user had pressed NO
				//throw new CimeroGenerateJarException("the file already exist",
				//		new IOException());
				//if the user had pressed NO
				throw new IOException();
			}
		}
		//
		//generate the serviceMix model
		//
		ServiceAssemblyClass smModel = (ServiceAssemblyClass) generateModel();
		if (smModel == null) {
			//raise an exception if the model is null
			throw new CimeroGenerateJarException("the generate model is null");
		}
		//
		//create the directory that contains the temporary file.
		//
		String dirName = smModel.getServiceAssemblyName() + "Jar";
		IContainer parent = getFile().getParent();
		IFolder jarFolder =
			ZipUtils.getDefault().createDirectory(dirName, parent);
		if (jarFolder == null) {
			ZipUtils.getDefault().deleteDirectory(dirName, parent);
			jarFolder = ZipUtils.getDefault().createDirectory(dirName, parent);
		}
		//create the list to the generation
		List < ContainerEmitArgFile > genLst =
			new ArrayList < ContainerEmitArgFile >();
		//
		//create SA generate configuration
		//
		try {
			genLst.add(createSAFileConfiguration(smModel , jarFolder));
		} catch (CimeroGenerateJarException e) {
			// delete all the generated file
			ZipUtils.getDefault().deleteDirectory(dirName, parent);
			throw e;
		}
		//
		//create SU generate configuration
		//
		try {
			genLst.addAll(createSUComponentConfiguration(smModel, jarFolder));
		} catch (CimeroGenerateJarException e) {
			// delete all the generated files
			ZipUtils.getDefault().deleteDirectory(dirName, parent);
			throw e;
		}
		//
		// launch the generation with the list of configuration
		//
		try {
			generate(genLst);
		} catch (Exception e) {
			ZipUtils.getDefault().deleteDirectory(dirName, parent);
			throw new CimeroGenerateJarException(
					"error during jar generation", e);
		}

		//tell the zip method to build the package
		try {
			boolean res = zipFolderAndSubFolder(jarFolder);
			//refresh to apply modifications
			try {
				parent.refreshLocal(2, null);
			} catch (CoreException e) {
				throw new CimeroGenerateJarException(
						"error during zip generation", e);
			}
			return res;
		} catch (CimeroGenerateJarException e) {
			throw new CimeroGenerateJarException(
					"error during zip generation", e);
		}
	}

	/**
	 * generate the file of all SU components of the servicemix model.
	 * @param smModel : the servicemix model.
	 * @param jarFolder : the folder where this su will be add
	 * @return a list of generate JET configuration
	 * @throws CimeroGenerateJarException : if there is a problem.
	 */
	private List < ContainerEmitArgFile > createSUComponentConfiguration(
			final ServiceAssemblyClass smModel,
			final IFolder jarFolder) throws CimeroGenerateJarException {
		List < ContainerEmitArgFile > maLst =
			new ArrayList<ContainerEmitArgFile>();

		if (smModel != null) {
			for (int i = 0; i < smModel.getLstSU().size(); i++) {
				ServiceUnitClass myUnit = smModel.getLstSU().get(i);
				//
				//create the SU folder
				//
				IFolder suFolder = ZipUtils.getDefault()
				.createDirectory(myUnit.getSuName(), jarFolder);
				if (suFolder == null) {
					throw new CimeroGenerateJarException("impossible to "
							+ "create the directory : '"
							+ myUnit.getSuName() + "'");
				}
				// -----------------------------------------------
				// call function to generate the code.
				// -----------------------------------------------
				if (myUnit.isStandard()) {
					//
					//drools files generation
					//
					StandardJBIJARClass mSTD = myUnit.getMonStandard();
					boolean containDrools = false;
					for (StandardJBIClass j : mSTD.getJbiCompo()){
						if (j instanceof StandardRouterClass) {
							containDrools = true;
							break;
						}
					}
					
					
					DroolsRouterModel drools = (containDrools ? generateDroolsModel() : null);
					
					List<String> includedFiles = new ArrayList<String>();
					
					for (StandardJBIClass j : mSTD.getJbiCompo()){
						if (j instanceof StandardHttpClass) {
							String endpoint = ((StandardHttpClass)j).getEndpointValue();
							
							DiagramEditor diag = CimeroRegistry.getInstance()
							.getDiagramEditor(getDiagram().getCimeroDiagramName());
							IFileEditorInput iFileEditorInput = (IFileEditorInput) diag.getEditorInput();
							
							IFile cimeroFile = iFileEditorInput.getFile();
							
							IProject project = cimeroFile.getProject();
							String selectedFileName = cimeroFile.getName();
							String saName = selectedFileName.substring(0, selectedFileName.lastIndexOf("."));
							IPath wsdlFilePath = new Path(saName+ endpoint +".wsdl");
							IFolder wsdlFolder = project.getFolder("WsdlFiles");
							if (wsdlFolder.exists(wsdlFilePath)){
								IFile wsdlFile = wsdlFolder.getFile(wsdlFilePath);
								((StandardHttpClass)j).setRelativeWSDLResource(saName+ endpoint +".wsdl");
								includedFiles.add("project:" + wsdlFile.getProjectRelativePath());
							}
							
							
						}
					}
					
					maLst.add(createStandardComponentConfiguration(
							myUnit.getMonStandard() , suFolder));
					
					if (drools != null){
						ContainerEmitArgFile[] droolEmitters = createDroolsGenerator(drools, myUnit.getMonStandard() , suFolder);
						if (droolEmitters != null){
							for (int kk = 0; kk < droolEmitters.length; kk++) 
								maLst.add(droolEmitters[kk]);
						}
					}
					
					includedFiles.addAll(myUnit.getMonStandard().getIncludedFiles());
						
					includeFiles(myUnit, includedFiles, suFolder, maLst);
				} else {
					maLst.addAll(createStandardLWComponentConfiguration(
							myUnit.getLW() , suFolder));
					// --- Le risorse statiche 
					for (String s : myUnit.getLW().getIncludedFiles()){
						
						
						//--------------------------
						s = s.replace("\\", "/");
						String relativeFileName = s.substring(s.lastIndexOf("/") + 1);
						IFile file = suFolder.getFile(new Path(relativeFileName));
						if (s.startsWith("file:")){
							maLst.add(new StaticCopyFile(s, file));
						}else if (s.startsWith("project:")){
							DiagramEditor diag = CimeroRegistry.getInstance()
							.getDiagramEditor(getDiagram().getCimeroDiagramName());
							IFileEditorInput iFileEditorInput = (IFileEditorInput) diag.getEditorInput();
							IProject project = iFileEditorInput.getFile().getProject();
							String fileNameRelatedToProject = s.substring(s.indexOf("project:")+ "project:".length());
							
							IFile projectFile = (IFile)project.findMember(fileNameRelatedToProject);
							String absoluteLocation = projectFile.getLocation().toOSString();
							maLst.add(new StaticCopyFile(absoluteLocation, file));
						}
					}
				}
			}
		} else {
			throw new CimeroGenerateJarException(
			"servicemix model cannot be null");
		}
		return maLst;
	}

	public void includeFiles(ServiceUnitClass myUnit, List<String> fileNames, IFolder suFolder,List < ContainerEmitArgFile > maLst){
		for (String s : fileNames){
			s = s.replace("\\", "/");
			String relativeFileName = s.substring(s.lastIndexOf("/") + 1);
			IFile file = suFolder.getFile(new Path(relativeFileName));
			if (s.startsWith("file:")){
				maLst.add(new StaticCopyFile(s, file));
			}else if (s.startsWith("project:")){
				DiagramEditor diag = CimeroRegistry.getInstance()
				.getDiagramEditor(getDiagram().getCimeroDiagramName());
				IFileEditorInput iFileEditorInput = (IFileEditorInput) diag.getEditorInput();
				IProject project = iFileEditorInput.getFile().getProject();
				String fileNameRelatedToProject = s.substring(s.indexOf("project:")+ "project:".length());
				
				IFile projectFile = (IFile)project.findMember(fileNameRelatedToProject);
				String absoluteLocation = projectFile.getLocation().toOSString();
				maLst.add(new StaticCopyFile(absoluteLocation, file));
			}
		}
	}

	/**
	 * generate the code of the standard LW component.
	 * @param monLW : the standard LW component
	 * @param suFolder : the directory where the generate files will be add
	 * @return all the configuration to generate the standard component.
	 * @throws CimeroGenerateJarException : if generation raise an exception.
	 */
	private List <ContainerEmitArgFile> createStandardLWComponentConfiguration(
			final StandardLWComponentClass monLW,
			final IFolder suFolder) throws CimeroGenerateJarException {
		//create the list of configuration
		List < ContainerEmitArgFile > maLst =
			new ArrayList<ContainerEmitArgFile>();
		String base = Platform.getPlugin(CimeroEditorPlugin.getPluginId())
		.getDescriptor().getInstallURL().toString();
		//
		//LW standard file generation
		//
		IFile saFile = suFolder.getFile(new Path("servicemix.xml"));
//		String jetFile = base + "templates"+ File.separator
//		+ "JARGeneration" + File.separator
//		+ "standardLW.jet";
		String jetFile = base + "templates" + "/"
		+ "JARGeneration" + "/"
		+ "standardLW.jet";
		ClassLoader classloader = getClass().getClassLoader();
		JETEmitter emitter = new JETEmitter(jetFile, classloader);
		
		try {
			emitter.addVariable("CIMEROEDITOR" ,
				CimeroEditorPlugin.getPluginId());
		} catch (JETException e1) {
			throw new CimeroGenerateJarException(e1);
		}
		maLst.add(new ContainerEmitArgFile(
				emitter, new Object [] {monLW}, saFile));
		
		return maLst;
		
	}

	/**
	 * This function is used to create the object need to create
	 * the drools files.
	 * @return the DroolsRouterModel if there is router in the
	 * 	cimero model or null in the other case.
	 */
	private DroolsRouterModel generateDroolsModel() {
		//we test if there is router in cimeroModel
		List < Routeur > lst = new ArrayList < Routeur >();
		Diagram diagTmp = getDiagram();
		if (diagTmp != null) {
			for (int i = 0; i < diagTmp.getLstCimeroObject().size(); i++) {
				GeneriqueCimeroModel elt = (GeneriqueCimeroModel) diagTmp
				.getLstCimeroObject().get(i);
				if (elt instanceof Routeur) {
					lst.add((Routeur) elt);
				}
			}
			//we test if there is at least one routeur in the list
			if (lst.size() != 0) {
				DroolsRouterModel droolsModel = new DroolsRouterModel();
				droolsModel.setPrefixeName(diagTmp.getCimeroDiagramName());
				for (int i = 0; i < lst.size(); i++) {
					Routeur routerLstElt = lst.get(i);
					droolsModel.addRule(routerLstElt.getName(),
							routerLstElt.getMaLstRule());
				}
				return droolsModel;
			} else {
				//there is no routeur in model
				return null;
			}
		} else {
			return null;
		}
	}

	private ContainerEmitArgFile[] createDroolsGenerator(
			DroolsRouterModel drools, 
			final StandardJBIJARClass monStandard,
			final IFolder suFolder) throws CimeroGenerateJarException {
		
		if (drools != null) {
			String base = Platform.getPlugin(CimeroEditorPlugin.getPluginId())
			.getDescriptor().getInstallURL().toString();

			IFile saFile = suFolder.getFile(new Path("xbean.xml"));
			ClassLoader classloader = getClass().getClassLoader();
			JETEmitter emitterDrools = null;
		
			String jetFileDrools = base + "templates" + "/"
					+ "DroolsGeneration" + "/" + "Drools31.jet";
			emitterDrools = new JETEmitter(jetFileDrools, classloader);
			
			try{
				emitterDrools.addVariable("CIMEROEDITOR" ,
					CimeroEditorPlugin.getPluginId());
			}catch (JETException e1) {
				throw new CimeroGenerateJarException(e1);
			}
			
		    if (drools.lstRuleSize() > 0){
		    	ContainerEmitArgFile[] emitters = new ContainerEmitArgFile[drools.lstRuleSize()];
		    	for (int i = 0; i < drools.lstRuleSize(); i++) {
					ServiceMixRule tmpRule = drools.getRuleElt(i);
					IFile droolsFile = suFolder.getFile(
							new Path(tmpRule.getName() + ".drl"));
					emitters[i] = new GenericGenerator.ContainerEmitArgFile(emitterDrools,
							new Object[] { tmpRule }, droolsFile);
				}
		    	return emitters;
		    }
			
			
			return null;
		} else {
			throw new CimeroGenerateJarException("service model is not ok");
		}
		
	}
	
		
	
	/**
	 * generate the code of standards components.
	 * @param monStandard : the standrad component.
	 * @param suFolder : the folder where the generat files will be add
	 * @return the configuration to generate this component.
	 * @throws CimeroGenerateJarException : raise if the is an exception.
	 */
	private ContainerEmitArgFile createStandardComponentConfiguration(
			final StandardJBIJARClass monStandard,
			final IFolder suFolder) throws CimeroGenerateJarException {
		String base = Platform.getPlugin(CimeroEditorPlugin.getPluginId())
		.getDescriptor().getInstallURL().toString();

		IFile saFile = suFolder.getFile(new Path("xbean.xml"));
		ClassLoader classloader = getClass().getClassLoader();
		
		
		
		
		
		
		
		
		//select the jet file with the class type
		if (monStandard.getJbiCompo().size() != 0) {
//			String jetFile = base + "templates" + File.separator
//			+ "JARGeneration"
//			+ File.separator + "standard.jet";
			String jetFile = base + "templates" + "/"
			+ "JARGeneration"
			+ "/" + "standard.jet";

			JETEmitter emitter = new JETEmitter(jetFile, classloader);
			try {
				emitter.addVariable(
						"CIMEROEDITOR"
						,CimeroEditorPlugin.getPluginId());
			} catch (JETException e1) {
				throw new CimeroGenerateJarException(e1);
			}
			//return the generate configuration
			return new ContainerEmitArgFile(
					emitter, new Object [] {monStandard}, saFile);
		} else {
			throw new CimeroGenerateJarException("service model is not ok");
		}
		
		
	}

	/**
	 * generate the Service Assembly file of the jar directory.
	 * @param smModel : the SM model.
	 * @param jarFolder : the jar folder.
	 * @return the information to the generation
	 * @throws CimeroGenerateJarException : if the generation is not possible.
	 */
	private ContainerEmitArgFile createSAFileConfiguration(
			final ServiceAssemblyClass smModel,
			final IFolder jarFolder) throws CimeroGenerateJarException {
		//change order of SA to make LW in the last position of this list.
		smModel.setLstSU(makeLWCompoInLastPosition(smModel.getLstSU()));
		//create the META-INF directory
		IFolder metainf =
			ZipUtils.getDefault().createDirectory("META-INF", jarFolder);
		//create the SA File
		IFile saFile = metainf.getFile(new Path("jbi.xml"));
		//initialisation of JET component
		String base = Platform.getPlugin(CimeroEditorPlugin.getPluginId())
		.getDescriptor().getInstallURL().toString();
//		String jetFile = base + "templates" + File.separator + "JARGeneration"
//		+ File.separator + "serviceAssembly.jet";
		String jetFile = base + "templates" + "/" + "JARGeneration"
		+ "/" + "serviceAssembly.jet";
		ClassLoader classloader = getClass().getClassLoader();
		JETEmitter emitter = new JETEmitter(jetFile, classloader);
		try {
			emitter.addVariable("CIMEROEDITOR" ,
					CimeroEditorPlugin.getPluginId());
		} catch (JETException e1) {
			throw new CimeroGenerateJarException(e1);
		}
		return new ContainerEmitArgFile(
				emitter, new Object [] {smModel}, saFile);
	}

	/**
	 * this function is used to set the LW su at the end of the
	 * list passed in parameter.
	 * @param lst : the list to triate
	 * @return the new list with LWSU at the end of the list.
	 */
	private List < ServiceUnitClass > makeLWCompoInLastPosition(
			final List < ServiceUnitClass > lst) {
		List < ServiceUnitClass > newLst = new ArrayList<ServiceUnitClass>(lst);
		ServiceUnitClass lwSU = null;
		//test if a LW exist in the list in parameter
		for (int i = 0; i < newLst.size(); i++) {
			ServiceUnitClass mySU = (ServiceUnitClass) newLst.get(i);
			if (!mySU.isStandard()) {
				//we save and remove this SU
				lwSU = mySU;
				newLst.remove(lwSU);
			}
		}
		if (lwSU != null) {
			//add this su at the end of the list
			newLst.add(lwSU);
		}
		return newLst;
	}
	/**
	 * This function generates the model of ServiceMix using to
	 * generate a Jar configuration file.
	 * @return the servicemix model generate
	 */
	public final IModelServiceMix generateModel() {
		Diagram cimeroDiagram = getDiagram();
		IModelServiceMix smModel  = null;
		if (cimeroDiagram != null) {
			if (cimeroDiagram.getLstCimeroObject().size() != 0) {
				//Create the jarmodelFactory
				JarModelFactory facto = new JarModelFactory(
						cimeroDiagram, force);
				smModel = facto.generateModelServiceMix();
			} else {
				//error
				return null;
			}
		} else {
			//error
			return null;
		}
		return smModel;
	}

	/**
	 * method used to test if the wanted zip file already exists.
	 * @return boolean : true if already exist
	 */
	private boolean testFileAlreadyExist() {
		boolean bExist = false;
		IContainer parent = getFile().getParent();
		//transform the name of the file to create jar path
		//the jar file has the same path and the same name than the cimero file
		//so we just have to delete the extension and to replace it
		String[] tabRegex = getFile().getName().split("\\.");
		String jarPath = "";
		for (int i = 0; i < tabRegex.length - 1; i++) {
			jarPath += tabRegex[i];
		}
		//we concat the extension (Jar.zip)
		jarPath += "Jar.zip";
		//
		IFile target = parent.getFile(new Path(jarPath));
		if (target.exists()) {
			bExist = true;
		} else {
			File systemFile = target.getLocation().toFile();
			if (systemFile.exists()) {
				bExist = true;
			}
		}
		return bExist;
	}

	/**
	 * param to force or not the generation.
	 * @param forceParam the force to set
	 */
	public final void setForce(final boolean forceParam) {
		this.force = forceParam;
	}

	/**
	 * zip method of a folder and all sub-folder.
	 * @param folder : folder to compress
	 * @return true if no errors
	 * @throws CimeroGenerateJarException if error
	 */
	private boolean zipFolderAndSubFolder(final IFolder folder)
	throws CimeroGenerateJarException {
		//call of the zip method in the zip package
		try {
			return ZipUtils.getDefault().zipFolderAndSubFolder(folder);
		} catch (CimeroGenerateJarException e) {
			throw new CimeroGenerateJarException(
					"error during zip generation", e);
		}
	}
	
	
	public static PropertyModel createPropertyModel(String propName, String propValue){
		PropertyModel prop = new PropertyModel();
		prop.setName(propName);
		prop.setValue(propValue);
		return prop;
	}
	
}
