package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global;

/**
 * This class represents an ActivationSpec in the ServiceMix model.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ActivationSpecClass {

	/**The namespace reference using by this activationSpec.*/
	private NameSpaceClass nameSpaceRef = null;
	/**The component name.*/
	private String componentName = "";
	/**The service name.*/
	private String serviceName = "";
	/**The nameSpace service reference.*/
	private NameSpaceClass serviceNameSpace = null;
	/**The destination service name.*/
	private String destinationServiceName = "";
	/**The namespace using by the destination service.*/
	private NameSpaceClass destinationServiceNameSpace = null;
	/**The reference to the componentClass.*/
	private ComponentClass component = null;


	/**
	 * This function is used to set the service name.
	 * @param serviceNameParam : the name of this service
	 * @param serviceNameSpaceParam : the namespace using by this service.
	 */
	public final void setServiceName(final String serviceNameParam,
			final NameSpaceClass serviceNameSpaceParam) {
		this.serviceName = serviceNameParam;
		this.serviceNameSpace = serviceNameSpaceParam;
	}

	/**
	 * This function is used to set the destination service.
	 * @param destinationSer : the name of the destination service.
	 * @param destinationNameSpace : the namespace using by the service.
	 */
	public final void setDestinationService(final String destinationSer ,
			final NameSpaceClass destinationNameSpace) {
		this.destinationServiceName = destinationSer;
		this.destinationServiceNameSpace = destinationNameSpace;
	}

	/**
	 * @return the component class of this activationSpec.
	 */
	public final ComponentClass getComponent() {
		return component;
	}

	/**
	 * @return the name of the component.
	 */
	public final String getComponentName() {
		return componentName;
	}

	/**
	 * @return the name of the destination service.
	 */
	public final String getDestinationServiceName() {
		return destinationServiceName;
	}

	/**
	 * @return the nameSpace of the destinationService.
	 */
	public final NameSpaceClass getDestinationServiceNameSpace() {
		return destinationServiceNameSpace;
	}

	/**
	 * @return the reference of the namespace using by this
	 * activation spec.
	 */
	public final NameSpaceClass getNameSpaceRef() {
		return nameSpaceRef;
	}

	/**
	 * @return the name of the service.
	 */
	public final String getServiceName() {
		return serviceName;
	}

	/**
	 * @return the namespace using by the service.
	 */
	public final NameSpaceClass getServiceNameSpace() {
		return serviceNameSpace;
	}

	/**
	 * @param componentNameP The componentName to set.
	 */
	public final void setComponentName(final String componentNameP) {
		this.componentName = componentNameP;
	}

	/**
	 * @param componentP The component to set.
	 */
	public final void setComponent(final ComponentClass componentP) {
		this.component = componentP;
	}

	/**
	 * @param nameSpaceRefP The nameSpaceRef to set.
	 */
	public final void setNameSpaceRef(final NameSpaceClass nameSpaceRefP) {
		this.nameSpaceRef = nameSpaceRefP;
	}

}
