package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * This class represents a property in ServiceMix model.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class PropertyClass {
	/**The name of this property.*/
	private String propertyName = "";
	/**a boolean to determine if this property has a value
	 * or a reference to another bean.*/
	private boolean isValue = false;
	
	private boolean ref = false;
	private String refValue = null;
	/**a boolean to determine if this a map.*/
	private boolean isMap = false;
	/**The map that contain the properties.*/
	private Map<LWBeanClass, LWBeanClass> lWMap = null;
	/**The value of this property.*/
	private List < String > lstValue = null;
	/**The reference to the bean.*/
	private LWBeanClass myBean = null;


	/**
	 * constructor with the name of the property in parameter.
	 * @param propertyNameP : the name of the property.
	 */
	public PropertyClass(final String propertyNameP) {
		super();
		this.propertyName = propertyNameP;
		this.lstValue = new ArrayList < String >();
		this.lWMap = new HashMap<LWBeanClass, LWBeanClass>();
	}

	//	---------------------------------------------------
	//SETTER FUNCTIONS
	/**
	 * set the value of this property.
	 * @param valueParam : the value of this property.
	 */
	public final void addValue(final String valueParam) {
		this.isValue = true;
		this.lstValue.add(valueParam);
		this.myBean = null;
		//we clear the map
		this.isMap = false;
		this.lWMap.clear();
	}
	
	/**
	 * set the value of this property.
	 * @param valueParam : the value of this property.
	 */
	public final void addRef(final String valueParam) {
		this.isValue = false;
		this.refValue = valueParam;
		this.myBean = null;
		this.ref = true;
		this.isMap = false;
		this.lWMap.clear();
	}
	/**
	 * Set the bean reference of this property.
	 * @param myBeanParam : the reference to the bean.
	 */
	public final void setBean(final LWBeanClass myBeanParam) {
		if (myBeanParam != null) {
			this.isValue = false;
			this.myBean = myBeanParam;
			this.lstValue.clear();
			//clear the map
			this.isMap = false;
			this.lWMap.clear();
		}
	}

	/**
	 * set the name of this property.
	 * @param name : the name of this property.
	 */
	public final void setPropertyName(final String name) {
		this.propertyName = name;
	}

	/**
	 * add in lWMap an new entry.
	 * @param key : the key.
	 * @param bean : the value of this key.
	 */
	public final void addEntryMap(final LWBeanClass key,
			final LWBeanClass bean) {
		this.isValue = false;
		this.lstValue.clear();
		this.myBean = null;
		this.isMap = true;
		this.lWMap.put(key, bean);
	}



	//---------------------------------------------------
	//GETTER FUNCTIONS
	/**
	 * @return Returns the isValue.
	 */
	public final boolean isValue() {
		return isValue;
	}
	/**
	 * @return Returns the myBean.
	 */
	public final LWBeanClass getMyBean() {
		return myBean;
	}
	/**
	 * @return Returns the propertyName.
	 */
	public final String getPropertyName() {
		return propertyName;
	}
	/**
	 * @return Returns the value.
	 */
	public final List < String > getLstValue() {
		return this.lstValue;
	}

	/**
	 * @return the isMap
	 */
	public final boolean isMap() {
		return isMap;
	}

	/**
	 * @return the lWMap
	 */
	public final Map<LWBeanClass, LWBeanClass> getLWMap() {
		return lWMap;
	}

	public boolean isRef() {
		return ref;
	}

	public void setRef(boolean ref) {
		this.ref = ref;
	}

	public String getRefValue() {
		return refValue;
	}

	public void setRefValue(String refValue) {
		this.refValue = refValue;
	}



}
