package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.model.global.standardJBIClassImplementation;

import it.eng.utils.PropertyModel;

import java.util.ArrayList;
import java.util.List;

import com.bull.cimero.pluginEditor.editors.model.BCModel.JdbcBC;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * This class represents the standard JBI JDBC component in the
 * ServiceMix model.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class StandardJDBCClass extends StandardJBIClass implements IBindingComponentClass {

	/**The provider role.*/
	public static final String PROVIDER = "provider";
	/**The consumer role.*/
	public static final String CONSUMER = "consumer";

	
	public JdbcBC jdbcBC = null;
	/**
	 * constructor with parameters.
	 * @param defMepValue : the mep value
	 * @param srcName : the service destination name
	 * @param serviceNamespaceParam : the namespace using by the service.
	 * @param jBINamespace : the namespace reference using by the service
	 * @param endPValue : the endpoint value
	 * @param providerRoleParam :
	 * 				the role of this component : PROVIDER or CONSUMER
	 * @param locURI : the location URI
	 * @param soap : true if it is a soap request
	 *
	 */
	public StandardJDBCClass(
			final String srcName,
			final NameSpaceClass serviceNamespaceParam,
			final NameSpaceClass jBINamespace,
			final String endPValue,
			final JdbcBC jdbcBc 
			) {
		super(jBINamespace);
	
		this.serviceNamespace = serviceNamespaceParam;
		this.serviceName = srcName;
		this.endpointValue = endPValue;
		this.jdbcBC = jdbcBc;
	}

	
	
	
	
	/**
	 * @return Returns the serviceName.
	 */
	public final String getServiceName() {
		return serviceName;
	}

	
	
	public String getXBean(){
		StringBuffer sb = new StringBuffer();
	
		sb.append("<jdbc:poller service=\""
				+ GeneriqueLWfactory.NAMESPACENAME+ ":"
				+ this.endpointValue + "\"\n");
		sb.append("     endpoint=\"" + this.endpointValue + "\"\n");
		sb.append("     targetService=\"" + serviceNamespace.getNameSpaceName()
				+ ":" + this.serviceName + "\"\n");
		sb.append("     databaseType = \""
				+ jdbcBC.getDatabaseType() + "\"\n");
		sb.append("     driver = \""
				+ jdbcBC.getPropertyValue(JdbcBC.CONN_DRIVER) + "\"\n");
		sb.append("     URL = \"" + jdbcBC.getPropertyValue(JdbcBC.CONN_URL)
				+ "\"\n");
		sb.append("     userName = \""
				+ jdbcBC.getPropertyValue(JdbcBC.CONN_USER) + "\"\n");
		sb.append("     password = \""
				+ jdbcBC.getPropertyValue(JdbcBC.CONN_PASSWORD) + "\"\n");
		sb.append("     period = \""
				+ jdbcBC.getPropertyValue(JdbcBC.JDBC_BC_PERIOD) + "\"\n");
		sb.append("     key = \"" + jdbcBC.getPropertyValue(JdbcBC.JDBC_BC_KEY)
				+ "\"\n");
		sb.append("     initialKeyValue = \""
				+ jdbcBC.getPropertyValue(JdbcBC.JDBC_BC_INITIAL_KEY_VALUE)
				+ "\"\n");
		String rowsInMessage = (String)jdbcBC.getPropertyValue(JdbcBC.JDBC_BC_ROWS_IN_MESSAGE);
		if (rowsInMessage != null && rowsInMessage.trim().length() > 0)
			sb.append("     rowsInMessage = \""+rowsInMessage+ "\"\n");
		
		sb.append("     commPointID=\""
				+ jdbcBC.getPropertyValue(JdbcBC.JDBC_BC_COMMPOINTID) + "\"\n");
		
		if (jdbcBC.isRowNamesAsAttribute())
			sb.append("     rowNameAsAttribute=\"true\"\n");
		if (jdbcBC.isColNamesAsAttribute())
			sb.append("     columnNameAsAttribute=\"true\"\n");
		if (jdbcBC.isValueAsAttribute()){
			sb.append("     valueAsAttribute=\"true\"\n");
		}
		sb.append(">");

		String rootQuery = (String)jdbcBC.getPropertyValue(JdbcBC.MAINQUERY_SQL_VALUE);
		rootQuery = rootQuery.replaceAll("<", "&lt;");
		sb.append("<rootStatement>\n");
		sb.append("<bean class=\"it.eng.spagosoa.smx.components.jdbc.StatementElement\">");
		sb.append("    <property name=\"name\" value=\""+jdbcBC.getPropertyValue(JdbcBC.MAINQUERY_NAME)+"\"/>\n");
		sb.append("    <property name=\"sql\" value=\""+rootQuery+"\"/>\n");
	    
		List<JdbcBC.Query> lstQuery = jdbcBC.getMaLstQuery();
		if (lstQuery != null && lstQuery.size() > 0){
			sb.append("    <property name=\"subQueries\">\n");
			sb.append("    <list>\n");
			JdbcBC.Query query = null;
			for (int i = 0; i < jdbcBC.getMaLstQuery().size(); i++ ){
				query = lstQuery.get(i);
				String subQuery = query.getQueryValue();
				subQuery = subQuery.replaceAll("<", "&lt;");
				
				sb.append("  <bean class=\"it.eng.spagosoa.smx.components.jdbc.StatementElement\">\n");
				sb.append("  		<property name=\"name\" value=\""+query.getQueryName()+"\"/>\n");
				sb.append("  		<property name=\"sql\" value=\""+subQuery+"\"/>\n");
				sb.append("	</bean>\n");
			}
			sb.append("    </list>\n");
			sb.append("    </property>\n");
		}

	        
	    sb.append("</bean>\n");
	    sb.append("</rootStatement>\n");
		sb.append("</jdbc:poller>\n");
		return sb.toString();
	}
	
	@Override
	public List<PropertyModel> getPropertiesToStore() {
		
		List<PropertyModel> propertiesToStore = new ArrayList<PropertyModel>();
		propertiesToStore.add(createPropertyModel("databaseType",jdbcBC.getDatabaseType()));
		
		propertiesToStore.add(createPropertyModel("driver", (String)jdbcBC.getPropertyValue(JdbcBC.CONN_DRIVER)));
		propertiesToStore.add(createPropertyModel("URL",(String)jdbcBC.getPropertyValue(JdbcBC.CONN_URL)));
		propertiesToStore.add(createPropertyModel("userName",(String)jdbcBC.getPropertyValue(JdbcBC.CONN_USER)));
		propertiesToStore.add(createPropertyModel("password",(String)jdbcBC.getPropertyValue(JdbcBC.CONN_PASSWORD)));
		propertiesToStore.add(createPropertyModel("period",(String)jdbcBC.getPropertyValue(JdbcBC.JDBC_BC_PERIOD)));
		propertiesToStore.add(createPropertyModel("initialKeyValue",(String)jdbcBC.getPropertyValue(JdbcBC.JDBC_BC_INITIAL_KEY_VALUE)));
		propertiesToStore.add(createPropertyModel("commPointID",(String)jdbcBC.getPropertyValue(JdbcBC.JDBC_BC_COMMPOINTID)));
		
		
		if (jdbcBC.isRowNamesAsAttribute())
			propertiesToStore.add(createPropertyModel("rowNameAsAttribute","true"));
		if (jdbcBC.isColNamesAsAttribute())
			propertiesToStore.add(createPropertyModel("columnNameAsAttribute","true"));
		if (jdbcBC.isValueAsAttribute())
			propertiesToStore.add(createPropertyModel("valueAsAttribute","true"));
		
		propertiesToStore.add(createPropertyModel("mainQueryName",(String)jdbcBC.getPropertyValue(JdbcBC.MAINQUERY_NAME)));
		propertiesToStore.add(createPropertyModel("mainQuerySQL",(String)jdbcBC.getPropertyValue(JdbcBC.MAINQUERY_SQL_VALUE)));
		List<JdbcBC.Query> lstQuery = jdbcBC.getMaLstQuery();
		if (lstQuery != null && lstQuery.size() > 0){
			
			
			JdbcBC.Query query = null;
			for (int i = 0; i < jdbcBC.getMaLstQuery().size(); i++ ){
				query = lstQuery.get(i);
				String subQuery = query.getQueryValue();
				
				
				propertiesToStore.add(createPropertyModel("subquery_"+i+"_Name",query.getQueryName()));
				propertiesToStore.add(createPropertyModel("subquery_"+i+"_SQL",subQuery)); 
				
				
			}
			
		}
		
		return propertiesToStore;
	}
}
