package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.model.standardModel;

import java.util.ArrayList;
import java.util.List;

import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJBIClass;

/**
 * This class represents the StandardJBI component using in
 * a standard configuration package.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class StandardJBIJARClass {
	/**A list of namespace using by this component.*/
	private List < NameSpaceClass > namespaceClass = null;
	/**The standardJBIClass component.*/
	private List < StandardJBIClass > lstJbiCompo = null;
	private List < String > includedFiles  = null;
	
	public final void addIncludeFilesInZip(String absoluteFilePath){
		this.includedFiles.add(absoluteFilePath);
	}

	/**
	 * default constructor.
	 * Initialisation of variables.
	 */
	public StandardJBIJARClass() {
		super();
		namespaceClass = new ArrayList < NameSpaceClass >();
		lstJbiCompo = new ArrayList < StandardJBIClass >();
		this.includedFiles = new ArrayList<String>();
	}

	/**add a namespace to this component.
	 * @param nms : the namespace to add in the list.*/
	public final void addNamespace(final NameSpaceClass nms) {
		namespaceClass.add(nms);
	}
	/**
	 * add in the StandardJBIClass list an element.
	 * @param myElt : the element to add in the StandardJBIClass list.
	 */
	public final void addStandardJBIElement(final StandardJBIClass myElt) {
		lstJbiCompo.add(myElt);
	}

	/**
	 * @return Returns the jbiCompo.
	 */
	public final List < StandardJBIClass > getJbiCompo() {
		return lstJbiCompo;
	}
	/**
	 * @return Returns the namespaceClass.
	 */
	public final List < NameSpaceClass > getNamespaceClass() {
		return namespaceClass;
	}

	/**
	 * to determine if a namespace is in the namespace list.
	 * @param myNms : the namespace to search in the list.
	 * @return true if the namespace already exist.
	 */
	public final boolean nameSpaceExist(final NameSpaceClass myNms) {
		for (int i = 0; i < getNamespaceClass().size(); i++) {
			NameSpaceClass nms = getNamespaceClass().get(i);
			if (nms.getNameSpaceName().compareTo(
					myNms.getNameSpaceName()) == 0) {
				return true;
			}
		}
		return false;
	}

	public List<String> getIncludedFiles() {
		return includedFiles;
	}

	public void setIncludedFiles(List<String> includedFiles) {
		this.includedFiles = includedFiles;
	}


}
