package com.bull.cimero.pluginEditor.popupMenu;

import it.eng.cimero.plugin.dialogs.PublishServiceRegistryDialog;
import it.eng.spagosoa.metadb.model.ServiceAssembly;
import it.eng.utils.DialogUtils;
import it.eng.utils.ProcessModel;

import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchActionConstants;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.CimeroRegistry;
import com.bull.cimero.pluginEditor.editors.DiagramEditor;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.ServiceAssemblyClass;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * This class creates the context popup menu of CIMERO Editor.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ContextPopupMenu extends MenuManager {

	/**The name of cimero Diagram.*/
	private String cimeroDiagramName = null;
	/**The showError action.*/
	private Action showError = null;
	/**The validate file action.*/
	private Action validateFile = null;
	/**The xml generation configuration action.*/
	private Action xmlGeneration = null;
	/**The jar generation configuration action.*/
	private Action jarGeneration = null;
	
	
	/**The xml generation configuration action.*/
	private Action generateWSDLAction = null;
	
	/**The xml generation configuration action.*/
	private Action configureInServiceRegistryAction = null;
	private Action removeFromServiceRegistryAction = null;
	
	/**The db generation configuration action.*/
	private Action registerDBAction = null;
	/**The deploy action.*/
	private Action deploy = null;
	/**The undeploy action.*/
	private Action undeploy = null;
	/**The automatique organisation action.*/
	private Action autoOrg = null;
	
	private Action configureRules = null;

	/**
	 * default constructor.
	 * @param fileName : the name of the diagram.
	 */
	public ContextPopupMenu(final String fileName) {
		super();
		this.cimeroDiagramName = fileName;
		hookContextMenu();
	}

	/**
	 * copy constructor.
	 * @param text : the text to define the menu
	 * @param fileName : the name of the diagram.
	 */
	public ContextPopupMenu(final String text, final String fileName) {
		super(text);
		this.cimeroDiagramName = fileName;
		hookContextMenu();
	}

	/**
	 * initialize of menu.
	 */
	private void hookContextMenu() {
		setRemoveAllWhenShown(true);
		addMenuListener(new IMenuListener() {
			public void menuAboutToShow(final IMenuManager manager) {
				ContextPopupMenu.this.fillContextMenu(manager);
			}
		});
	}

	/**
	 * add action in a context menu.
	 * @param manager : the menu manager.
	 */
	private void fillContextMenu(final IMenuManager manager) {
		//
		//create show error action
		//
		//showError = createShowErrorCallAction();
		//showError.setText("Show errors");
		//manager.add(showError);
		//
		//add new separator
		//
		//manager.add(new Separator());
		//
		//create the validate file action
		//
		//alidateFile = createValidateFileCallAction();
		//validateFile.setText("Validate this file");
		//manager.add(validateFile);
		//
		//add new separator
		//
		//manager.add(new Separator());
		//
		//add deploy and undeploy
		//
		//deploy = createDeployAction();
		//deploy.setText("Deploy on ServiceMix");
		//manager.add(deploy);
		//undeploy = createUndeployAction();
		//undeploy.setText("Undeploy from ServiceMix");
		//manager.add(undeploy);
		//
		//add new separator
		//
		//manager.add(new Separator());
		//
		//add xml generation and jar generation
		//
		jarGeneration = createJarGenerationAction();
		jarGeneration.setText(" Generate JBI package ");
		manager.add(jarGeneration);
		
		registerDBAction = createRegisterDBAction();
		registerDBAction.setText(" Publish In Database ");
		manager.add(registerDBAction);
		
		generateWSDLAction = createGenerateWSDLAction();
		generateWSDLAction.setText(" Generate WSDL ");
		manager.add(generateWSDLAction);
		
		configureInServiceRegistryAction = createConfigureServiceRegistryAction();
		configureInServiceRegistryAction.setText(" Configure In Service Registry ");
		manager.add(configureInServiceRegistryAction);
		
		removeFromServiceRegistryAction = createRemoveFromServiceRegistryAction();
		removeFromServiceRegistryAction.setText(" Remove From Service Registry ");
		manager.add(removeFromServiceRegistryAction);
		
		
		
		
		//
		// add automatique organisation
		//	
		/*
		manager.add(new Separator());
		autoOrg = createAutoOrgAction();
		autoOrg.setText("Align to grid");
		manager.add(autoOrg);
		*/
	
		manager.add(new Separator());
		configureRules = createConfigureRulesForSelectedEndPoint();
		configureRules.setText(" Configure Rules For Endpoint ");
		manager.add(configureRules);
	
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	/**
	 * method to call the automatique organisation action.
	 * @return the automatique organisation action.
	 */
	private Action createAutoOrgAction() {
		return new Action() {
			public void run() {
				DiagramEditor diag = CimeroRegistry.getInstance()
					.getDiagramEditor(cimeroDiagramName);
				if (diag != null) {
					diag.organiseDrawingComponent();
				}
			}
		};
	}
	
	/**
	 * method to call the automatique organisation action.
	 * @return the automatique organisation action.
	 */
	private Action createConfigureRulesForSelectedEndPoint() {
		return new Action() {
			public void run() {
				DiagramEditor diag = CimeroRegistry.getInstance()
					.getDiagramEditor(cimeroDiagramName);
				diag.configureRulesForEndpoint();
				
			}
		};
	}

	/**
	 * method to call the showErrors action.
	 *@return the show error action.
	 
	private Action createShowErrorCallAction() {
		return new Action() {
			public void run() {
				DiagramEditor diag = CimeroRegistry.getInstance()
					.getDiagramEditor(cimeroDiagramName);
				if (diag != null) {
					diag.reverseBShowErrors();
				}
			}
		};
	}
	*/

	/**
	 * method to call the validate action.
	 *@return the validate file action.
	 
	private Action createValidateFileCallAction() {
		return new Action() {
			public void run() {
				Shell shell = new Shell();
				DiagramEditor diag = CimeroRegistry.getInstance()
					.getDiagramEditor(cimeroDiagramName);
				boolean res = false;
				if (diag != null) {
					//no diag associated
					res = diag.validate();
				}
				if (diag != null) {
					if (res) {
						MessageDialog.openInformation(
								shell,
								"CimeroEditor Plug-in",
						"The validation is correct !");
					} else {
						MessageDialog.openInformation(
								shell,
								"CimeroEditor Plug-in",
								"the validation is not correct. "
								+ "Please see error log !");
					}
				}
			}
		};
	}
	*/

	/**
	 * method to call the generate xml action.
	 * @return the create xml generation Action.
	 */
	private Action createRegisterDBAction() {
		return new Action() {
			public void run() {
				Shell shell = new Shell();
				boolean res = false;
				try {
				DiagramEditor diag = CimeroRegistry.getInstance()
				.getDiagramEditor(cimeroDiagramName);
				if (diag != null) {
					
					ProcessModel processModel = diag.getProcessModel();
					//int status = diag.verifyServiceAssemblyStatus();
					
					
					
					res = diag.storeProcessModel(processModel);
					

					
				
					//
					if (!res) {
						DialogUtils.information(shell, "Fail occurs..");

					}else{
						DialogUtils.information(shell, "Process Published..");
					}
				}
				} catch (IOException ioe) {
					return;
				} catch (Exception e) {
					CimeroLog.errorLog(
							"XML file generation exception"
							, e);
				}
			}
		};
	}
	
	
	private Action createRemoveFromServiceRegistryAction() {
		return new Action() {
			public void run() {
				Shell shell = new Shell();
				boolean res = false;
				try {
				DiagramEditor diag = CimeroRegistry.getInstance()
				.getDiagramEditor(cimeroDiagramName);
				if (diag != null) {
					ServiceAssembly serviceAssemblyRegisteredInDatabase = diag.isSARegisteredInDatabase();
					if (serviceAssemblyRegisteredInDatabase == null){
						// The services is not published in Database cannot pubblish in UDDI
						MessageDialog.openInformation(
								shell,
								"CimeroEditor Plug-in",
								"The service does not exist in  Meta Database");
					}else{
						PublishServiceRegistryDialog dialog = null;
						if (serviceAssemblyRegisteredInDatabase.getUddiOrg() != null && 
						   serviceAssemblyRegisteredInDatabase.getUddiName() != null){
						
							CimeroEditorPlugin.getDefault().removeServiceFromServiceRegistry(serviceAssemblyRegisteredInDatabase);
						
						}else{
							 //Service IS NOT published in ServiceRegistry The Dialog must allow specification
							MessageDialog.openInformation(
									shell,
									"CimeroEditor Plug-in",
									"The service is not published in   Meta Database");
						}
						
						
						
						
					}
				}

				} catch (IOException ioe) {
					return;
				} catch (Exception e) {
					CimeroLog.errorLog(
							"XML file generation exception"
							, e);
				}
			}
		};
	}
	
	
	/**
	 * method to call the generate xml action.
	 * @return the create xml generation Action.
	 */
	private Action createConfigureServiceRegistryAction() {
		return new Action() {
			public void run() {
				Shell shell = new Shell();
				boolean res = false;
				try {
				DiagramEditor diag = CimeroRegistry.getInstance()
				.getDiagramEditor(cimeroDiagramName);
				if (diag != null) {
					ServiceAssembly serviceAssemblyRegisteredInDatabase = diag.isSARegisteredInDatabase();
					if (serviceAssemblyRegisteredInDatabase == null){
						// The services is not published in Database cannot pubblish in UDDI
						MessageDialog.openInformation(
								shell,
								"CimeroEditor Plug-in",
								"Publishing in Service Registry Cannot be made: The service must be published first in Meta Database");
					}else{
						PublishServiceRegistryDialog dialog = null;
						if (serviceAssemblyRegisteredInDatabase.getUddiOrg() != null && 
						   serviceAssemblyRegisteredInDatabase.getUddiName() != null){
						   //Service is already published in ServiceRegistry Allow only to modify classification
						   dialog = new PublishServiceRegistryDialog(shell, serviceAssemblyRegisteredInDatabase, PublishServiceRegistryDialog.MODIFY_SERVICE_CLASSIFICATION);
						}else{
							 //Service IS NOT published in ServiceRegistry The Dialog must allow specification
						   dialog = new PublishServiceRegistryDialog(shell, serviceAssemblyRegisteredInDatabase, PublishServiceRegistryDialog.MODE_PUBLISH_NEW_SERVICE);
						}
						
						dialog.open();
						res = dialog.isDialogStatus();
						String dialogStatusMsg = dialog.getDialogStatusMsg();
						
						if (!res) {
							MessageDialog.openInformation(
									shell,
									"CimeroEditor Plug-in",
									dialogStatusMsg);
						}else{
							MessageDialog.openInformation(
								shell,
								"CimeroEditor Plug-in",
								dialogStatusMsg);
						}
					}
					
				}

				} catch (IOException ioe) {
					return;
				} catch (Exception e) {
					CimeroLog.errorLog(
							"XML file generation exception"
							, e);
				}
			}
		};
	}

	/**
	 * method to call the generate jar action.
	 * @return the create jar generation Action.
	 */
	private Action createJarGenerationAction() {
		return new Action() {
			public void run() {
				Shell shell = new Shell();
				//we call the generator
				boolean res = false;
				try {
					DiagramEditor diag = CimeroRegistry.getInstance()
								.getDiagramEditor(cimeroDiagramName);
					if (diag != null) {
						res = diag.generateJar();
						//
						if (!res) {
							MessageDialog.openInformation(
								shell,
								"CimeroEditor Plug-in",
								"Generation failure. Please see log file.");
						}
					}
				} catch (IOException ioe) {
					//user doesn't want to replace the existing file
					//CimeroLog.infoLog(
					//		"Jar file generation stopped");
					return;
				} catch (Exception e) {
					CimeroLog.errorLog(
							"Jar file generation exception"
							, e);
				}
			}
		};
	}
	
	/**
	 * method to call the generate jar action.
	 * @return the create jar generation Action.
	 */
	private Action createGenerateWSDLAction() {
		return new Action() {
			public void run() {
					Shell shell = new Shell();
				
				
					try {
						DiagramEditor diag = CimeroRegistry.getInstance()
						.getDiagramEditor(cimeroDiagramName);
						if (diag != null) {
							ServiceAssemblyClass saClassModel = diag.getServiceAssemblyClassModel();
							Map<String, Map<String, Object>> httpEndpointsMap = diag.getHttpStandardComponentsEndpoints(saClassModel);
							
							Set<String> keyset = httpEndpointsMap.keySet();
							
							for (String ep : keyset){
								diag.generateWSDLForHttpEndpoint(shell, diag.getIFile(), ep, httpEndpointsMap.get(ep));
							}
						}
					
			
				} catch (Exception e) {
					CimeroLog.errorLog(
							"Jar file generation exception"
							, e);
				}
			}
		};
	}

	/**
	 * method used to test if the zip file exists.
	 * @param iFile : the file to test
	 * @return boolean : true if exists
	 */
	private boolean testPackageExist(final IFile iFile) {
		boolean bExist = false;
		//the jar file has the same path and the same name than the cimero file
		//so we just have to delete the extension and to replace it
		String[] tabRegex = iFile.getLocation().toString().split("\\.");
		String jarPath = "";
		for (int i = 0; i < tabRegex.length - 1; i++) {
			jarPath += tabRegex[i];
		}
		//we concat the extension (.jar)
		jarPath += "Jar.zip";
		//
		File systemFile = new File(jarPath);
		if (systemFile.exists()) {
			bExist = true;
		}
		return bExist;
	}

	/**
	 * method used to get the path of the jar
	 * associated with the cimero file choosed.
	 * @param cimeroPath : the relative path of the cimero file
	 * @return String : the absolute path to the jar
	 */
	public final String getJarPath(final String cimeroPath) {
		//the jar file has the same path and the same name than the cimero file
		//so we just have to delete the extension and to replace it
		String[] tabRegex = cimeroPath.split("\\.");
		String jarPath = "";
		for (int i = 0; i < tabRegex.length - 1; i++) {
			jarPath += tabRegex[i];
		}
		//we concat the extension (.jar)
		jarPath += "Jar.zip";
		//return the obtained string
		return jarPath;
	}

	/**
	 * method used to get the common name of all associated file.
	 * @param cimeroPath : the relative path of the cimero file
	 * @return String : the absolute path to the jar
	 */
	public final String getCommonPath(final String cimeroPath) {
		String[] tabRegex = cimeroPath.split("\\.");
		String jarPath = "";
		for (int i = 0; i < tabRegex.length - 1; i++) {
			jarPath += tabRegex[i];
			if (i < tabRegex.length - 2) {
				jarPath += ".";
			}
		}
		//we concat the common string
		jarPath += "droolsFile";
		//we only need the two last part of this string
		//String[] tabStr = jarPath.split(File.separator);
		String[] tabStr = jarPath.split("/");
		jarPath = "";
		for (int i = tabStr.length - 2; i < tabStr.length; i++) {
			if (i >= 0) {
				jarPath += tabStr[i];
			}
			if (i < tabStr.length - 1) {
				//jarPath += File.separator;
				jarPath += "/";
			}
		}
		//return the obtained string
		return jarPath;
	}

	/**
	 * method to call the undeploy action.
	 * @return the undeploy Action.
	 
	private Action createUndeployAction() {
		return new Action() {
			public void run() {
				PluginServiceMix pSM = new PluginServiceMix();
				if (!pSM.testConnection()) {
					//no instance of SMPlugin found
					Shell shell = new Shell();
					MessageDialog.openInformation(shell, "CimeroEditor Plug-in",
						"SMPlugin was not found. "
						+ "Please install it to use this functionnality.");
				} else {
					IFile file = CimeroRegistry.getInstance().getDiagramEditor(
						cimeroDiagramName).getIFile();
					//get the path to the jar
					String pathToJar = getJarPath(
							file.getLocation().toString());
					//get the common path to the jar
					String commonPath = getCommonPath(
							file.getLocation().toString());
					//first, we have to look for drools files
					IResource[] tabRessources = null;
					try {
						tabRessources = file.getParent().members();
					} catch (CoreException e) {
						e.printStackTrace();
					}
					for (int i = 0; i < tabRessources.length; i++) {
						if (tabRessources[i].getType() == IResource.FILE) {
							//it's a file, so test if it's name is
							//the wanted string
							if (tabRessources[i].getFullPath().toString()
											.contains(commonPath)) {
								//it's a godd file, we have to deploy it
								pSM.getDeployer().undeploy(
									tabRessources[i].getLocation().toString());
							}
						}
					}
					//get the path to the jar
					//System.out.println("we undeploy : " + pathToJar);
					//deploy and get the resultat
					int resDeployment = pSM.getDeployer().undeploy(pathToJar);
					switch (resDeployment) {
						case Constante.DONE:
							//System.out.println("undeployed !");
							//add to log
							//CimeroLog.infoLog(
							//		pathToJar + " was undeployed succesfully");
							break;
						case Constante.NOT_STARTED:
							//System.out.println("not undeployed !");
							//add to log
							CimeroLog.errorLog(
									"No ServiceMix server is started !"
									, new Exception());
							break;
						case Constante.ERROR:
							//System.out.println("not undeployed !");
							//add to log
							CimeroLog.errorLog(
									"An unknown error occured !"
									, new Exception());
							break;
						case Constante.PACKAGENOTEXIST:
							//System.out.println("not undeployed !");
							//add to log
							CimeroLog.errorLog(
									"The package to undeploy does not exist !"
									, new Exception());
							break;
						case Constante.UNABLETOUNDEPLOY:
							//System.out.println("not undeployed !");
							//add to log
							CimeroLog.errorLog(
									"Unable to undeploy the package"
									, new Exception());
							break;
						default:
							//add to log
							CimeroLog.errorLog(
									"Bad response received from SMPlugin !"
									, new Exception());
							break;
					} //fin switch
				} //fin else
			}
		};
	}
	*/

	/**
	 * method to call the deploy action.
	 * @return the deploy Action.
	 
	private Action createDeployAction() {
		return new Action() {
			public void run() {
				PluginServiceMix pSM = new PluginServiceMix();
				if (!pSM.testConnection()) {
					//no instance of SMPlugin found
					Shell shell = new Shell();
					MessageDialog.openInformation(shell, "CimeroEditor Plug-in",
						"SMPlugin was not found. "
						+ "Please install it to use this functionnality.");
				} else {
					IFile file = CimeroRegistry.getInstance().getDiagramEditor(
							cimeroDiagramName).getIFile();
					//test if the file already exists
					if (!testPackageExist(file)) {
						//not exist
						Shell monShell = new Shell();
						MessageDialog.openInformation(
								monShell,
								"JBI package doesn't exist",
								"The JBI package to deploy does not exist !"
								+ "You have to generate it before deploy it.");
					} else {
						//package exist
						//get the path to the jar
						String pathToJar = getJarPath(
								file.getLocation().toString());
						//get the common path to the jar
						String commonPath = getCommonPath(
								file.getLocation().toString());
						//first, we have to look for drools files
						IResource[] tabRessources = null;
						try {
							tabRessources = file.getParent().members();
						} catch (CoreException e) {
							e.printStackTrace();
						}
						for (int i = 0; i < tabRessources.length; i++) {
							if (tabRessources[i].getType() == IResource.FILE) {
								//it's a file, so test if it's name is the
								//wanted string
								if (tabRessources[i].getFullPath().toString()
												.contains(commonPath)) {
									//it's a godd file, we have to deploy it
									pSM.getDeployer().deploy(tabRessources[i]
									             .getLocation().toString());
								}
							}
						}
						//
						//System.out.println("we deploy : " + pathToJar);
						//deploy and get the resultat
						int resDeployment = pSM.getDeployer().deploy(pathToJar);
						switch (resDeployment) {
							case Constante.DONE:
								//System.out.println("deployed !");
								//add to log
								//CimeroLog.infoLog(
								//pathToJar + " was deployed succesfully");
								break;
							case Constante.NOT_STARTED:
								//System.out.println("not deployed !");
								//add to log
								CimeroLog.errorLog(
										"No ServiceMix server is started !"
										, new Exception());
								break;
							case Constante.ERROR:
								//System.out.println("not deployed !");
								//add to log
								CimeroLog.errorLog(
										"An unknown error occured !"
										, new Exception());
								break;
							case Constante.PACKAGENOTEXIST:
								//System.out.println("not deployed !");
								//add to log
								CimeroLog.errorLog(
										"The package to deploy does not exist !"
										, new Exception());
								break;
							case Constante.UNABLETOCOPY:
								//System.out.println("not deployed !");
								//add to log
								CimeroLog.errorLog(
										"Error when copying the package "
										+ "into the right directory !"
										, new Exception());
								break;
							case Constante.UNABLETOUNDEPLOY:
								//System.out.println("not deployed !");
								//add to log
								CimeroLog.errorLog(
										"Unable to undeploy previous version"
										+ " of the package !"
										, new Exception());
								break;
							default:
								//add to log
								CimeroLog.errorLog(
										"Bad response received from SMPlugin !"
										, new Exception());
								break;
						} //fin switch
					} //fin else package exist
				} //fin else
			}
		};
	}
	*/
}
