package com.bull.cimero.pluginEditor.popupMenu;

import java.io.File;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

/**
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class DeployAction implements IObjectActionDelegate {

	/** the selected file. */
	private IFile fileSelected = null;

	/**
	 * Constructor for deploy.
	 */
	public DeployAction() {
		super();
	}

	/**
	 * the getter fileselected function.
	 * @return IFile
	 */
	public final IFile getFileSelected() {
		return fileSelected;
	}

	/**
	 * The setter file selected function.
	 * @param filePath : the file
	 */
	public final void setFileSelected(final IFile filePath) {
		this.fileSelected = filePath;
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 * @param targetPart the new part target
	 */
	public final void setActivePart(final IAction action,
			final IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 */
	public final void run(final IAction action) {
		/*
		PluginServiceMix pSM = new PluginServiceMix();
		if (!pSM.testConnection()) {
			//no instance of SMPlugin found
			Shell shell = new Shell();
			MessageDialog.openInformation(shell, "CimeroEditor Plug-in",
				"SMPlugin was not found. "
				+ "Please install it to use this functionnality.");
		} else {
			//test if the file already exists
			if (!testPackageExist()) {
				//not exist
				Shell monShell = new Shell();
				MessageDialog.openInformation(
						monShell,
						"JBI package doesn't exist",
						"The JBI package to deploy does not exist !"
						+ "You have to generate it before deploy it.");
			} else {
				//package exist
				//get the path to the jar
				String pathToJar = getJarPath(
						getFileSelected().getLocation().toString());
				//get the common path to the jar
				String commonPath = getCommonPath(
						getFileSelected().getLocation().toString());
				//first, we have to look for drools files
				IResource[] tabRessources = null;
				try {
					tabRessources = getFileSelected().getParent().members();
				} catch (CoreException e) {
					e.printStackTrace();
				}
				for (int i = 0; i < tabRessources.length; i++) {
					if (tabRessources[i].getType() == IResource.FILE) {
						//it's a file, so test if it's name is the wanted string
						if (tabRessources[i].getFullPath().toString()
										.contains(commonPath)) {
							//it's a godd file, we have to deploy it
							pSM.getDeployer().deploy(
									tabRessources[i].getLocation().toString());
						}
					}
				}
				//
				//System.out.println("we deploy : " + pathToJar);
				//deploy and get the resultat
				int resDeployment = pSM.getDeployer().deploy(pathToJar);
				switch (resDeployment) {
					case Constante.DONE:
						//System.out.println("deployed !");
						//add to log
						//CimeroLog.infoLog(
						//		pathToJar + " was deployed succesfully");
						break;
					case Constante.NOT_STARTED:
						//System.out.println("not deployed !");
						//add to log
						CimeroLog.errorLog(
								"No ServiceMix server is started !"
								, new Exception());
						break;
					case Constante.ERROR:
						//System.out.println("not deployed !");
						//add to log
						CimeroLog.errorLog(
								"An unknown error occured !"
								, new Exception());
						break;
					case Constante.PACKAGENOTEXIST:
						//System.out.println("not deployed !");
						//add to log
						CimeroLog.errorLog(
								"The package to deploy does not exist !"
								, new Exception());
						break;
					case Constante.UNABLETOCOPY:
						//System.out.println("not deployed !");
						//add to log
						CimeroLog.errorLog(
								"Error when copying the package "
								+ "into the right directory !"
								, new Exception());
						break;
					case Constante.UNABLETOUNDEPLOY:
						//System.out.println("not deployed !");
						//add to log
						CimeroLog.errorLog(
								"Unable to undeploy previous version"
								+ " of the package !"
								, new Exception());
						break;
					default:
						//add to log
						CimeroLog.errorLog(
								"Bad response received from SMPlugin !"
								, new Exception());
						break;
				} //fin switch
			} //fin else package exist
		} //fin else
		*/
	}

	/**
	 * method used to get the path of the jar
	 * associated with the cimero file choosed.
	 * @param cimeroPath : the relative path of the cimero file
	 * @return String : the absolute path to the jar
	 */
	public final String getJarPath(final String cimeroPath) {
		//the jar file has the same path and the same name than the cimero file
		//so we just have to delete the extension and to replace it
		String[] tabRegex = cimeroPath.split("\\.");
		String jarPath = "";
		for (int i = 0; i < tabRegex.length - 1; i++) {
			jarPath += tabRegex[i];
		}
		//we concat the extension (.jar)
		jarPath += "Jar.zip";
		//return the obtained string
		return jarPath;
	}

	/**
	 * method used to get the common name of all associated file.
	 * @param cimeroPath : the relative path of the cimero file
	 * @return String : the absolute path to the jar
	 */
	public final String getCommonPath(final String cimeroPath) {
		String[] tabRegex = cimeroPath.split("\\.");
		String jarPath = "";
		for (int i = 0; i < tabRegex.length - 1; i++) {
			jarPath += tabRegex[i];
			if (i < tabRegex.length - 2) {
				jarPath += ".";
			}
		}
		//we concat the common string
		jarPath += "droolsFile";
		//we only need the two last part of this string
		//String[] tabStr = jarPath.split(File.separator);
		String[] tabStr = jarPath.split("/");
		jarPath = "";
		for (int i = tabStr.length - 2; i < tabStr.length; i++) {
			if (i >= 0) {
				jarPath += tabStr[i];
			}
			if (i < tabStr.length - 1) {
				//jarPath += File.separator;
				jarPath += "/";
			}
		}
		//return the obtained string
		return jarPath;
	}

	/**
	 * This function is called when the user selected the file in the package
	 * explorer.
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 * @param action the action proxy that handles the presentation
	 * 			portion of the action
	 * @param selection : the selected file
	 */
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		//we save the select file
		setFileSelected((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}

	/**
	 * method used to test if the zip file exists.
	 * @return boolean : true if exists
	 */
	private boolean testPackageExist() {
		boolean bExist = false;
		//the jar file has the same path and the same name than the cimero file
		//so we just have to delete the extension and to replace it
		String[] tabRegex = getFileSelected().getLocation()
								.toString().split("\\.");
		String jarPath = "";
		for (int i = 0; i < tabRegex.length - 1; i++) {
			jarPath += tabRegex[i];
		}
		//we concat the extension (.jar)
		jarPath += "Jar.zip";
		//
		File systemFile = new File(jarPath);
		if (systemFile.exists()) {
			bExist = true;
		}
		return bExist;
	}

}
