package com.bull.cimero.pluginEditor.popupMenu;

import it.eng.cimero.plugin.dialogs.PublishServiceRegistryDialog;
import it.eng.spagosoa.metadb.model.ServiceAssembly;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import com.bull.cimero.pluginEditor.editors.CimeroRegistry;
import com.bull.cimero.pluginEditor.editors.DiagramEditor;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class PublishServiceAssemblyInServiceRegistryAction implements IObjectActionDelegate {

	/** the selected file. */
	private IFile fileSelected = null;

	/**
	 * Constructor for GenerateJarFileAction.
	 */
	public PublishServiceAssemblyInServiceRegistryAction() {
		super();
	}

	/**
	 * the getter fileselected function.
	 * @return IFile
	 */
	public final IFile getFileSelected() {
		return fileSelected;
	}

	/**
	 * The setter file selected function.
	 * @param filePath : the file
	 */
	public final void setFileSelected(final IFile filePath) {
		this.fileSelected = filePath;
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 * @param targetPart the new part target
	 */
	public final void setActivePart(final IAction action,
			final IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 */
	public final void run(final IAction action) {
		Shell shell = new Shell();
		
		
//		MessageDialog.openInformation(
//				shell,
//				"CimeroEditor Plug-in Engineering Extensione",
//				"Register Service Assembly Action");
		
		if (getFileSelected() == null) {
			//we cannot call the generator without selected file.
			// a message is sent to the user
			MessageDialog.openInformation(
					shell,
					"CimeroEditor Plug-in",
					"There is a problem with the selected file.The registration of service assembly in db"
					+ "cannot be done !");
		} else {
			//we call the generator
			boolean res = false;
			try {
				DiagramEditor diag = CimeroRegistry.getInstance()
							.getDiagramEditor(getFileSelected().getName());
				if (diag != null) {
					ServiceAssembly serviceAssemblyRegisteredInDatabase = diag.isSARegisteredInDatabase();
					if (serviceAssemblyRegisteredInDatabase == null){
						// The services is not published in Database cannot pubblish in UDDI
						MessageDialog.openInformation(
								shell,
								"CimeroEditor Plug-in",
								"Publishing in Service Registry Cannot be made: The service must be published first in Meta Database");
					}else{
						PublishServiceRegistryDialog dialog = null;
						if (serviceAssemblyRegisteredInDatabase.getUddiOrg() != null && 
						   serviceAssemblyRegisteredInDatabase.getUddiName() != null){
						   //Service is already published in ServiceRegistry Allow only to modify classification
						   dialog = new PublishServiceRegistryDialog(shell, serviceAssemblyRegisteredInDatabase, PublishServiceRegistryDialog.MODIFY_SERVICE_CLASSIFICATION);
						}else{
							 //Service IS NOT published in ServiceRegistry The Dialog must allow specification
						   dialog = new PublishServiceRegistryDialog(shell, serviceAssemblyRegisteredInDatabase, PublishServiceRegistryDialog.MODE_PUBLISH_NEW_SERVICE);
						}
						
						dialog.open();
						res = dialog.isDialogStatus();
						String dialogStatusMsg = dialog.getDialogStatusMsg();
						
						if (!res) {
							MessageDialog.openInformation(
									shell,
									"CimeroEditor Plug-in",
									dialogStatusMsg);
						}else{
							MessageDialog.openInformation(
								shell,
								"CimeroEditor Plug-in",
								dialogStatusMsg);
						}
					}
					
					
				} else {
					//we open the diagramEditor
					shell.getDisplay().syncExec(new Runnable() {
						public void run() {
							IWorkbenchPage page2 = null;
							page2 = PlatformUI.getWorkbench()
									.getActiveWorkbenchWindow().getActivePage();
							try {
								IDE.openEditor(page2, getFileSelected(), true);
							} catch (PartInitException e) {
								e.printStackTrace();
							}
						}
					});
					//and we call again this function
					run(action);
				}
			} catch (IOException ioe) {
				return;
			} catch (Exception e) {
				CimeroLog.errorLog(
						"Registration in DB Exception"
						, e);
			}
		}
	}

	/**
	 * This function is called when the user selected the file in the package
	 * explorer.
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 * @param action the action proxy that handles the presentation
	 * 			portion of the action
	 * @param selection : the selected file
	 */
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		//we save the select file
		setFileSelected((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
}
