package com.bull.cimero.pluginEditor.popupMenu;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import com.bull.cimero.pluginEditor.editors.CimeroRegistry;
import com.bull.cimero.pluginEditor.editors.DiagramEditor;

/**
 * Action used to show or to hide errors in the current file.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 */
public class ShowErrorsAction implements IObjectActionDelegate {

	/** the selected file. */
	private IFile fileSelected = null;

	/**
	 * Constructor for ShowsErrorsAction.
	 */
	public ShowErrorsAction() {
		super();
	}
	/**
	 * the getter fileselected function.
	 * @return IFile
	 */
	public final IFile getFileSelected() {
		return fileSelected;
	}

	/**
	 * The setter file selected function.
	 * @param filePath : the file
	 */
	public final void setFileSelected(final IFile filePath) {
		this.fileSelected = filePath;
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 * @param targetPart the new part target
	 */
	public final void setActivePart(final IAction action,
			final IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 */
	public final void run(final IAction action) {
		DiagramEditor diag = CimeroRegistry.getInstance()
						.getDiagramEditor(getFileSelected().getName());
		if (diag != null) {
			diag.reverseBShowErrors();
		} else {
			//we open the diagramEditor
			new Shell().getDisplay().syncExec(new Runnable() {
				public void run() {
					IWorkbenchPage page2 = null;
					page2 = PlatformUI.getWorkbench()
							.getActiveWorkbenchWindow().getActivePage();
					try {
						IDE.openEditor(page2, getFileSelected(), true);
					} catch (PartInitException e) {
						e.printStackTrace();
					}
				}
			});
			//and we call again this function
			run(action);
		}
	}

	/**
	 * This function is called when the user selected the file in the package
	 * explorer.
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 * @param action the action proxy that handles the presentation
	 * 			portion of the action
	 * @param selection : the selected file
	 */
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		//we save the select file
		setFileSelected((IFile) ((StructuredSelection) selection)
					.getFirstElement());
		//
		DiagramEditor diag = null;
		if (!selection.isEmpty()) {
			String fileName = getFileSelected().getName();
			diag = CimeroRegistry.getInstance()
						.getDiagramEditor(fileName);
		}
		if (diag == null) {
			//no diag associated
			//we uncheck
			action.setChecked(false);
		} else {
			//a diag is associated
			action.setChecked(diag.isBShowErrors());
		}
	}
}
