package com.bull.cimero.pluginEditor.popupMenu;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import com.bull.cimero.pluginEditor.editors.CimeroRegistry;
import com.bull.cimero.pluginEditor.editors.DiagramEditor;

/**
 * This class is used to add an popup action. This action calls the validator
 * module in a file. This class must return and say if the selected cimero file
 * is valide.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello ( andrea.zoppelo@eng.it)
 */
public class ValidateAction implements IObjectActionDelegate {

	/** the selected file. */
	private IFile fileSelected = null;

	/**
	 * Constructor for Action1.
	 */
	public ValidateAction() {
		super();
	}

	/**
	 * the getter fileselected function.
	 * @return IFile
	 */
	public final IFile getFileSelected() {
		return fileSelected;
	}

	/**
	 * The setter file selected function.
	 * @param filePath : the file
	 */
	public final void setFileSelected(final IFile filePath) {
		this.fileSelected = filePath;
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 * @param targetPart the new part target
	 */
	public final void setActivePart(final IAction action,
			final IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 */
	public final void run(final IAction action) {
		Shell shell = new Shell();

		if (getFileSelected() == null) {
			//we cannot call the validator without selected file.
			// a message is sent to the user
			MessageDialog.openInformation(
					shell,
					"CimeroEditor Plug-in",
					"There is a problem with the selected file. The validation"
					+ "cannot be realised !");
		} else {
			DiagramEditor diag = CimeroRegistry.getInstance()
							.getDiagramEditor(getFileSelected().getName());
			boolean res = false;
			if (diag != null) {
				//no diag associated
				res = diag.validate();
			}

			if (diag != null) {
				if (res) {
					MessageDialog.openInformation(
							shell,
							"CimeroEditor Plug-in",
					"The validation is correct !");
				} else {
					MessageDialog.openInformation(
							shell,
							"CimeroEditor Plug-in",
							"the validation is not correct. "
							+ "Please see error log !");
				}
			} else {
				//we open the diagramEditor
				shell.getDisplay().syncExec(new Runnable() {
					public void run() {
						IWorkbenchPage page2 = null;
						page2 = PlatformUI.getWorkbench()
								.getActiveWorkbenchWindow().getActivePage();
						try {
							IDE.openEditor(page2, getFileSelected(), true);
						} catch (PartInitException e) {
							e.printStackTrace();
						}
					}
				});
				//and we call again this function
				run(action);
			}
		}
	}

	/**
	 * This function is called when the user selected the file in the package
	 * explorer.
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 * @param action the action proxy that handles the presentation
	 * 			portion of the action
	 * @param selection : the selected file
	 */
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		//we save the select file
		setFileSelected((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}

}
