package it.eng.cimero.plugin.dialogs;

import it.eng.cimero.plugin.providers.AttributeInCatalogContentProvider;
import it.eng.cimero.plugin.providers.AttributeInCatalogLabelProvider;
import it.eng.cimero.plugin.providers.CatalogContentProvider;
import it.eng.cimero.plugin.providers.CatalogLabelProvider;
import it.eng.spagosoa.metadb.api.IMetadbApi;
import it.eng.spagosoa.metadb.model.Attribute;
import it.eng.spagosoa.metadb.model.AttributeCatalog;
import it.eng.spagosoa.metadb.model.AttributeCatalogId;
import it.eng.spagosoa.metadb.model.Catalog;
import it.eng.utils.DialogUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import cimeroEditor.CimeroEditorPlugin;

public class CatalogDialog extends TitleAreaDialog {

//	final SimpleDateFormat formatter = new SimpleDateFormat("dd/MM/yyyy");
	

    private Composite aComposite = null;
	
    private SashForm internalSash  = null; 
    
    private Table tableCatalogs = null;
    private TableViewer tbvCatalogs = null; 
   
    private Table tableAttribute = null;
    private TableViewer tbvAttribute = null;
    
    private List<Catalog> listCatalogs = null;
    private List<AttributeInCatalog> listAttribute = null;
    private Catalog selectedCatalog = null;
    
    public static final int NEW_CATALOG_ID = 99001;

    public static final int LINK_EXISTING_ATTRIBUTE_ID = 99003;
    
    public IMetadbApi metadb = null;
    
    private MenuManager popupMenuTabellaAttributi = null;
    private Menu menuTabellaAttributi = null;
    
	public CatalogDialog(Shell parentShell, IMetadbApi metadb) {
		super(parentShell);
		this.listCatalogs = new ArrayList<Catalog>();
		this.listAttribute = new ArrayList<AttributeInCatalog>();
		this.metadb = metadb;
	}
	
	
				

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
		setTitle("Catalog Configuration");
	    setMessage("Catalog Configuration");
	   
	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       	
    	
    	
    	this.listCatalogs = getCatalogs();
    	
        this.tableCatalogs = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableCatalogs.setLinesVisible(true);
        this.tableCatalogs.setHeaderVisible(true);
        TableColumn column = null;
	
		column = new TableColumn(tableCatalogs, SWT.LEFT);
		column.setText("Name");
		column.setWidth(200);
		column = new TableColumn(tableCatalogs, SWT.LEFT);
		column.setText("Visible");
		column.setWidth(100);
		column = new TableColumn(tableCatalogs, SWT.LEFT);
		column.setText("Mandatory");
		column.setWidth(100);
        this.tbvCatalogs = new TableViewer(tableCatalogs);
        tbvCatalogs.setContentProvider(new CatalogContentProvider());
        tbvCatalogs.setLabelProvider(new CatalogLabelProvider());
        tbvCatalogs.setInput(listCatalogs);    
    	
        
        final CatalogDialog theDialog = this;
        
        this.tableCatalogs.addKeyListener(new KeyListener(){
        	public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
        		System.out.println("KeyCode =" +  e.keyCode);
        		if (e.keyCode == 127){
        			TableItem[] tis = theDialog.tableCatalogs.getSelection();
        			if (tis.length > 0){
        				if (deleteCatalog((Catalog)tis[0].getData())){
        					theDialog.tbvCatalogs.setInput(getCatalogs());
        					theDialog.tbvCatalogs.refresh(true);
        					theDialog.tbvAttribute.setInput(getAttributeForCurrentCatalog());
        					theDialog.tbvAttribute.refresh(true);
        					theDialog.tbvAttribute.refresh();
        				}else{
        					DialogUtils.error(getShell(), "Cannot Delete Catalogs");
        					
        				}
        			}
        		}
        	};
        	public void keyReleased(org.eclipse.swt.events.KeyEvent e) {
        		
        	};
        	
        });
        
        this.tableAttribute = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableAttribute.setLinesVisible(true);
        this.tableAttribute.setHeaderVisible(true);
        column = null;
		column = new TableColumn(tableAttribute, SWT.LEFT);
		column.setText("Name");
		column.setWidth(200);
		column = new TableColumn(tableAttribute, SWT.LEFT);
		column.setText("Type");
		column.setWidth(200);
		column = new TableColumn(tableAttribute, SWT.LEFT);
		column.setText("Vsible");
		column.setWidth(50);
		
		popupMenuTabellaAttributi = new MenuManager();
	    popupMenuTabellaAttributi.add(new MoveUpAction(this));
	    popupMenuTabellaAttributi.add(new MoveDownAction(this));
	    popupMenuTabellaAttributi.add(new ShowAttributeInCatalogAction(this));
	    popupMenuTabellaAttributi.add(new HideAttributeInCatalogAction(this));

	    
	    menuTabellaAttributi = popupMenuTabellaAttributi.createContextMenu(this.tableAttribute);
	    this.tableAttribute.setMenu(menuTabellaAttributi);
		
        this.tbvAttribute = new TableViewer(tableAttribute);
        tbvAttribute.setContentProvider(new AttributeInCatalogContentProvider());
        tbvAttribute.setLabelProvider(new AttributeInCatalogLabelProvider());
        tbvAttribute.setInput(listAttribute);    
    	
        
        this.tableAttribute.addKeyListener(new KeyListener(){
        	public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
        		System.out.println("KeyCode =" +  e.keyCode);
        		if (e.keyCode == 127){
        			TableItem[] tis = theDialog.tableAttribute.getSelection();
        			
        			if (tis.length > 0){
        				if (unlinkAttribute((AttributeInCatalog)tis[0].getData())){
        					theDialog.tbvAttribute.setInput(getAttributeForCurrentCatalog());
        					theDialog.tbvAttribute.refresh(true);
        					theDialog.tbvAttribute.refresh();
        				}else{
        					MessageDialog.openError(
        		        			getShell(),
        		        			"Cimero Editor",
        		        			"Cannot delete attribute");
        				}
        			}

        			
        		}
        	};
        	public void keyReleased(org.eclipse.swt.events.KeyEvent e) {
        		
        	};
        	
        });
        
        this.tbvCatalogs.addSelectionChangedListener(new ISelectionChangedListener(){

			public void selectionChanged(SelectionChangedEvent event) {
				TableItem[] selections = theDialog.tableCatalogs.getSelection();
				if (selections.length > 0){
					theDialog.selectedCatalog = (Catalog)selections[0].getData();
					setMessage("Selected Catalog " + theDialog.selectedCatalog.getName());
					theDialog.tbvAttribute.setInput(getAttributeForCurrentCatalog());
				}
			}
        	
        });
        
        
    	
    	
    	
    	// Weight Settings on Sash
    	internalSash.setWeights(new int[] {50,50});
        return parent; 
	
	}
	
	protected void okPressed() {
		super.okPressed();
		
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		
		if (buttonId == CatalogDialog.NEW_CATALOG_ID){
			NewCatalogDialog newCatalogDialog = new NewCatalogDialog(getShell(), metadb);
			newCatalogDialog.open();
			this.listCatalogs = getCatalogs();
			this.tbvCatalogs.setInput(listCatalogs);
			this.tbvCatalogs.refresh(true);
			this.internalSash.layout();
		} else if (buttonId == CatalogDialog.LINK_EXISTING_ATTRIBUTE_ID){
			if (this.selectedCatalog == null){
				MessageDialog.openWarning(
	        			getShell(),
	        			"No Catalog selected cannot add Attribute",
	        			"No Catalog selected cannot add Attribute");
			}else{
				ListAttributesDialog listAttributesDialog = new ListAttributesDialog(getShell(), metadb, " Add Attributes to Catalog ");
				listAttributesDialog.open();
				Attribute attr = listAttributesDialog.getSelectedAttribute();
				if (attr != null){
					linkAttributeWithCatalog(attr);
					this.tbvAttribute.setInput(getAttributeForCurrentCatalog());
					this.tbvAttribute.refresh(true);
					this.internalSash.layout();
				}
			}
		}else{
			super.buttonPressed(buttonId);
		}
	}


	protected void createButtonsForButtonBar(Composite parent) {
		
	
		createButton(parent, CatalogDialog.NEW_CATALOG_ID, "New Catalog",
				true);
		
		createButton(parent, CatalogDialog.LINK_EXISTING_ATTRIBUTE_ID, "Link an Attribute",
				true);
		
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.CLOSE_LABEL,
				true);
		
	}

	protected void linkAttributeWithCatalog(Attribute attribute) {
		try {
			metadb.openTx();
			Attribute attr = (Attribute) metadb.getObjectById(Attribute.class,
					attribute.getIdAttribute());
			AttributeCatalogId attributeCatalogId = new AttributeCatalogId(attr.getIdAttribute(), selectedCatalog.getIdCatalog());
			if (metadb.getObjectById(AttributeCatalog.class, attributeCatalogId) != null){
				DialogUtils.warning(getShell(), "This Attribute is already link with catalog ");
			}else{
				metadb.linkAttributeToCatalog(attribute, selectedCatalog);
				metadb.storeAttribute(attr);
			}
			metadb.commitAndClose();

		} catch (Throwable e) {
			e.printStackTrace();
			metadb.rollBackAndClose();

		}
	}
	
	protected List<AttributeInCatalog> getAttributeForCurrentCatalog(){
		metadb.open();
		List<AttributeCatalog> result =  metadb.getAttributeCatalogRelationExtentForCatalog(selectedCatalog);
		List<AttributeInCatalog> retList = new ArrayList<AttributeInCatalog>();
		for (AttributeCatalog ac : result){
			retList.add(new AttributeInCatalog(ac.getId(),ac.getAttribute().getName(), ac.getAttribute().getType(), ac.getOrderCriteria(), ac.getVisible()));
		}
		metadb.close();
		return retList;
		
	}



	protected List<Catalog> getCatalogs(){
		metadb.open();
		List<Catalog> result = metadb.getCatalogs();
		metadb.close();
		return result;
	}

	public boolean deleteCatalog(Catalog catalog) {
		try{
			metadb.openTx();
			metadb.deleteCatalog(catalog);
			metadb.commitAndClose();
			return true;
		}catch (Throwable e) {
			e.printStackTrace();
			metadb.rollBackAndClose();
			return false;
		}
	}

	public boolean unlinkAttribute(AttributeInCatalog attribute) {
		try{
			metadb.openTx();
			Attribute attr = (Attribute)metadb.getObjectById(Attribute.class, attribute.getAttributeCatalogId().getIdAttribute());
			metadb.unlinkAttributeToCatalog(attr, selectedCatalog);
			metadb.storeAttribute(attr);
			metadb.commitAndClose();
			return true;
		}catch (Throwable e) {
			e.printStackTrace();
			metadb.rollBackAndClose();
			return false;
		}
	}
	
	public void swapOrderCriteriaInAttributeCatalogTypeRelation(
			AttributeInCatalog ac1, AttributeInCatalog ac2) {
		try {
			metadb.openTx();
			AttributeCatalog attCat1 = (AttributeCatalog) metadb
					.getObjectById(AttributeCatalog.class, ac1
							.getAttributeCatalogId());
			AttributeCatalog attCat2 = (AttributeCatalog) metadb
					.getObjectById(AttributeCatalog.class, ac2
							.getAttributeCatalogId());

			int order1 = attCat1.getOrderCriteria();
			attCat1.setOrderCriteria(attCat2.getOrderCriteria());
			metadb.genericStore(attCat1);
			attCat2.setOrderCriteria(order1);
			metadb.genericStore(attCat2);
			metadb.commitAndClose();

		} catch (Throwable e) {
			e.printStackTrace();
			metadb.rollBackAndClose();

		}
	}
	
	public void hideAttributeInCatalog(AttributeInCatalog ac1){
		try {
			metadb.openTx();
			AttributeCatalog attCat1 = (AttributeCatalog)metadb.getObjectById(AttributeCatalog.class, ac1.getAttributeCatalogId());
		
		
		
			attCat1.setVisible(false);
			metadb.genericStore(attCat1);
			metadb.commitAndClose();

		} catch (Throwable e) {
			e.printStackTrace();
			metadb.rollBackAndClose();

		}
		
	}
	
	public void showAttributeInCatalog(AttributeInCatalog ac1){
		try {
			metadb.openTx();
			AttributeCatalog attCat1 = (AttributeCatalog)metadb.getObjectById(AttributeCatalog.class, ac1.getAttributeCatalogId());
		
		
		
		attCat1.setVisible(true);
		metadb.genericStore(attCat1);
		metadb.commitAndClose();

		} catch (Throwable e) {
			e.printStackTrace();
			metadb.rollBackAndClose();

		}
		
	}
	
	
	
	public class MoveUpAction extends Action{
		public CatalogDialog ctgDialog = null;
		public MoveUpAction(CatalogDialog catalogDialog)
		{
			setToolTipText("Move Up Attribute In Catalog");
			setText("Move Up Attribute In Catalog");
			String imgPath = "icons" + File.separator + "arrow_up.png";
			setImageDescriptor(CimeroEditorPlugin.getImageDescriptor(imgPath));
			this.ctgDialog = catalogDialog;
		}

		public void run()
		{

			int selectionIndex = ctgDialog.tableAttribute.getSelectionIndex();
			
			int itemCount = ctgDialog.tableAttribute.getItemCount();
			//DialogUtils.warning(getShell(), "Selection["+selectionIndex+"] ItemCount ["+itemCount+"]");
			if (selectionIndex > -1 ){
				if (selectionIndex > 0 ){
					TableItem selected = ctgDialog.tableAttribute.getItem(selectionIndex);
					TableItem prev = ctgDialog.tableAttribute.getItem(selectionIndex - 1);
					
					if ((selected != null) && (prev != null)){
						ctgDialog.swapOrderCriteriaInAttributeCatalogTypeRelation((AttributeInCatalog)selected.getData(),(AttributeInCatalog)prev.getData());
						ctgDialog.tbvAttribute.setInput(ctgDialog.getAttributeForCurrentCatalog());
						ctgDialog.tbvAttribute.refresh(true);
					}
				}else{
					DialogUtils.warning(getShell(), " Cannot move up this attribute. It's the first ");
				}
					
			}else{
				DialogUtils.warning(getShell(), " No Attribute Selected ");
			}
		}
	}
	
	public class MoveDownAction extends Action{
		public CatalogDialog ctgDialog = null;
		public MoveDownAction(CatalogDialog catalogDialog)
		{
			setToolTipText("Move Down Attribute In Catalog");
			setText("Move Down Attribute In Catalog");
			String imgPath = "icons" + File.separator + "arrow_down.png";
			setImageDescriptor(CimeroEditorPlugin.getImageDescriptor(imgPath));
			this.ctgDialog = catalogDialog;
		}

		public void run()
		{
			int selectionIndex = ctgDialog.tableAttribute.getSelectionIndex();
			
			int itemCount = ctgDialog.tableAttribute.getItemCount();
			//DialogUtils.warning(getShell(), "Selection["+selectionIndex+"] ItemCount["+itemCount+"]");
			if (selectionIndex > -1){
				if ( (selectionIndex + 1) != itemCount ){
					TableItem selected = ctgDialog.tableAttribute.getItem(selectionIndex);
					TableItem next = ctgDialog.tableAttribute.getItem(selectionIndex + 1);
					
					if ((selected != null) && (next != null)){
						swapOrderCriteriaInAttributeCatalogTypeRelation((AttributeInCatalog)selected.getData(),(AttributeInCatalog)next.getData());
						ctgDialog.tbvAttribute.setInput(ctgDialog.getAttributeForCurrentCatalog());
						ctgDialog.tbvAttribute.refresh(true);
					}
				}else{
					DialogUtils.warning(getShell(), " Cannot move down this Attribute. It's the last one");
				}
					
			}else{
				DialogUtils.warning(getShell(), " No Attribute Selected ");
			}
		}
	}
	
	public class HideAttributeInCatalogAction extends Action{
		public CatalogDialog ctgDialog = null;
		public HideAttributeInCatalogAction(CatalogDialog catalogDialog)
		{
			setToolTipText("Hide Attribute In This Catalog");
			setText("Hide Attribute In This Catalog");
			//String imgPath = "icons" + File.separator + "arrow_down.png";
			//setImageDescriptor(CimeroEditorPlugin.getImageDescriptor(imgPath));
			this.ctgDialog = catalogDialog;
		}

		public void run()
		{
			int selectionIndex = ctgDialog.tableAttribute.getSelectionIndex();
			
			
			//DialogUtils.warning(getShell(), "Selection["+selectionIndex+"] ItemCount["+itemCount+"]");
			if (selectionIndex > -1){
				
					TableItem selected = ctgDialog.tableAttribute.getItem(selectionIndex);
					
					
					if (selected != null){
						ctgDialog.hideAttributeInCatalog((AttributeInCatalog)selected.getData());
						ctgDialog.tbvAttribute.setInput(ctgDialog.getAttributeForCurrentCatalog());
						ctgDialog.tbvAttribute.refresh(true);
					}
				
					
			}else{
				DialogUtils.warning(getShell(), " No Attribute Selected ");
			}
		}
	}
	
	public class ShowAttributeInCatalogAction extends Action{
		public CatalogDialog ctgDialog = null;
		public ShowAttributeInCatalogAction(CatalogDialog catalogDialog)
		{
			setToolTipText("Show this attribute in catalog");
			setText("Show this attribute in catalog");
			//String imgPath = "icons" + File.separator + "arrow_down.png";
			//setImageDescriptor(CimeroEditorPlugin.getImageDescriptor(imgPath));
			this.ctgDialog = catalogDialog;
		}

		public void run()
		{
			int selectionIndex = ctgDialog.tableAttribute.getSelectionIndex();
			
			
			//DialogUtils.warning(getShell(), "Selection["+selectionIndex+"] ItemCount["+itemCount+"]");
			if (selectionIndex > -1){
				
					TableItem selected = ctgDialog.tableAttribute.getItem(selectionIndex);
					
					
					if (selected != null){
						ctgDialog.showAttributeInCatalog((AttributeInCatalog)selected.getData());
						ctgDialog.tbvAttribute.setInput(ctgDialog.getAttributeForCurrentCatalog());
						ctgDialog.tbvAttribute.refresh(true);
					}
				
					
			}else{
				DialogUtils.warning(getShell(), " No Attribute Selected ");
			}
		}
	}
	
	
	
}




