package it.eng.cimero.plugin.preferences;

import org.eclipse.jface.preference.*;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.IWorkbench;
import cimeroEditor.CimeroEditorPlugin;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

public class CimeroPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	public CimeroPreferencePage() {
		super(GRID);
		setPreferenceStore(CimeroEditorPlugin.getDefault().getPreferenceStore());
		setDescription("Preferences For Spagic Studio Plugin");
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {

		addField(
			new StringFieldEditor(PreferenceConstants.P_AUDIT_DB_URL, "Audit DB URL", getFieldEditorParent()));
		addField(
				new StringFieldEditor(PreferenceConstants.P_AUDIT_DB_DRIVER, "Audit DB DRIVER", getFieldEditorParent()));
		addField(
				new StringFieldEditor(PreferenceConstants.P_AUDIT_DB_USER, "Audit DB USER", getFieldEditorParent()));
		addField(
				new StringFieldEditor(PreferenceConstants.P_AUDIT_DB_PASSWORD, "Audit DB PASSWORD", getFieldEditorParent()));
		addField(
				new StringFieldEditor(PreferenceConstants.P_AUDIT_DB_HIB_DIALECT, "Audit DB DIALECT", getFieldEditorParent()));
		addField(
				new BooleanFieldEditor(
					PreferenceConstants.P_AUDIT_DB_SHOW_SQL,
					"Show sql",
					getFieldEditorParent()));
		
		addField(
				new StringFieldEditor(PreferenceConstants.P_SERVICE_REGISTRY_INQUIRY_URL, "Service Registry Inquiry URL", getFieldEditorParent()));
		addField(
				new StringFieldEditor(PreferenceConstants.P_SERVICE_REGISTRY_PUBLISH_URL, "Service Registry Publish URL", getFieldEditorParent()));

	
		addField(
				new StringFieldEditor(PreferenceConstants.P_SERVICE_REGISTRY_USER, "Service Registry User", getFieldEditorParent()));
		
		addField(
				new StringFieldEditor(PreferenceConstants.P_SERVICE_REGISTRY_PASSWORD, "Service Registry Password", getFieldEditorParent()));
		
		addField(
				new StringFieldEditor(PreferenceConstants.P_SERVICE_REGISTRY_DEFAULT_ORG, "Default Organization", getFieldEditorParent()));
		
		
		addField(
				new StringFieldEditor(PreferenceConstants.P_GRAPH_VIZ_DOT_EXE, "Graphviz dot program Location", getFieldEditorParent()));
		
		addField(
				new StringFieldEditor(PreferenceConstants.P_GRAPH_VIZ_TEMP_FOLDER, "Graphviz temp folder", getFieldEditorParent()));
		
		
		addField(
				new StringFieldEditor(PreferenceConstants.P_SMX_GET_RESOURCES, "SMX GetResources Service ", getFieldEditorParent()));
		
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}
	
}