package it.eng.components.cimero;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.SE.SEStdFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.StandardComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.DroolsSemanticValidatorClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJBIClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardRouterClass;

/**
 * The  DroolsSemanticValidatorFactory factory.
 *
 * @author zoppello
 */
public class DroolsSemanticValidatorFactory extends SEStdFactory {
	/**The http namespace name.*/
	public static final String NAMESPACENAME = "drools";
	/**The http namespace value.*/
	public static final String NAMESPACEVALUE =
		"http://servicemix.apache.org/drools/1.0";
	/**
	 * default constructor.
	 * @param nms : the namespace using by the servicemix factory.
	 */
	public DroolsSemanticValidatorFactory(final NameSpaceClass nms) {
		super(nms);
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory
	 * .standardFactory.StandardComponentFactory#createJBIElement(
	 * com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel)
	 * @param elt : the cimero component.
	 * @return the standard JBI class create with the cimero component.
	 * @throws Exception if the generation failed.
	 */
	public final StandardJBIClass createJBIElement(
			final GeneriqueCimeroModel elt)	throws Exception {
		
		DroolsSemanticValidatorClass myClass = null;
		if (elt instanceof DroolsSemanticValidator) {
			DroolsSemanticValidator myDroolsSemanticValidator = (DroolsSemanticValidator) elt;
			//get the destination element.
			GeneriqueCimeroModel destElt = getDestination(elt);
			if (destElt != null) {
				NameSpaceClass destinationNms = getNamespace(destElt);
				String ruleBaseResourcePath = myDroolsSemanticValidator.getRuleBaseResourcePath();
				String relativePath =  null;
				if (ruleBaseResourcePath.startsWith("file:"))
					relativePath = ruleBaseResourcePath.substring("file:".length());
				else
					relativePath = ruleBaseResourcePath.substring("project:".length());
				
				if (relativePath.lastIndexOf("/") > 0){
					relativePath = relativePath.substring(relativePath.lastIndexOf("/") + 1);
				}
				
				myClass = new DroolsSemanticValidatorClass(
						myDroolsSemanticValidator.getName(),
						new NameSpaceClass(GeneriqueLWfactory.NAMESPACENAME,
								GeneriqueLWfactory.NAMESPACEVALUE),
								myDroolsSemanticValidator.getName(),
						new NameSpaceClass(getNamespaceName(),
								  getNamespaceValue()),
						destElt.getName(),
						destinationNms,
						"classpath:"+relativePath);
			} else {
				throw new Exception("The destination component of the "
						+ elt.getName() + " object doesn't exist");
			}
		} else {
			throw new Exception("the object : "
					+ elt.getName() + " hasn't a good type for this factory");
		}
		return myClass;
	}

	

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory#getNamespaceName()
	 * @return the default namespace.
	 */
	public final String getNamespaceName() {
		return NAMESPACENAME;
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory#getNamespaceValue()
	 * @return the default namespace value.
	 */
	public final String getNamespaceValue() {
		return NAMESPACEVALUE;
	}

	/**
	 * @return the namespace using by this servicemix component.
	 */
	public final NameSpaceClass getNamespaceComponent() {
		return new NameSpaceClass(getNamespaceName(), getNamespaceValue());
	}

	
		public final ActivationSpecClass createXMLModel(
				final GeneriqueCimeroModel elt,
				final Boolean isForce) throws Exception {
			if (getNamespaceRef() == null) {
				throw new Exception("namespaceclass cannot be null");
			}
			ActivationSpecClass myActi = null;
			//if not null and valide
			if (elt != null) {
				if ((elt.isValide(false)) || (isForce)) {
					//create the activationSpec
					myActi = new ActivationSpecClass();
					myActi.setNameSpaceRef(getNamespaceRef());
					myActi.setComponentName(elt.getName());
					myActi.setServiceName(elt.getName(),
						new NameSpaceClass(GeneriqueLWfactory.NAMESPACENAME ,
								GeneriqueLWfactory.NAMESPACEVALUE));
					//create the components
					ComponentClass compo = createComponent(elt);
					myActi.setComponent(compo);
					return myActi;
				}
			 } else {
				throw new Exception("The generique cimero object is not valide");
			}
			return null;
		}
		/**
		 * create the component of the xml generation.
		 * @param elt :  the EIP model.
		 * @return the generated component instance class
		 * @throws Exception : if the generation return an error.
		 */
		private ComponentClass createComponent(
				final GeneriqueCimeroModel elt)
				throws Exception {
//			create the componentclass instance
			ComponentClass monCompo = new ComponentClass(getNamespaceRef());
			//add the bean class.
			StandardComponentClass monS = null;
			monS = createStandardComponentMEP(elt);
			monCompo.setMonComponentStandard(monS);
			return monCompo;
		}
		
		/**
		 * creates the standard components.
		 * @param elt : the cimero Wiretap model
		 * @return the standard component class.
		 * @throws Exception
		 * @throws Exception : if the generation return an error.
		 */
		private StandardComponentClass createStandardComponentMEP(
				final GeneriqueCimeroModel elt) throws Exception {
			StandardComponentClass monStd = new StandardComponentClass(
					new NameSpaceClass(getNamespaceName() , getNamespaceValue()));
			monStd.setMyJBICompo(createJBIElement(elt));
			return monStd;
		}
}
