package it.eng.components.cimero;


import java.lang.reflect.Field;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 */
public class GenericJdbcAdvancedQuery extends ServiceEngineModel implements IRequireDataSourceComponent {

	/** Serial UID. */
	private static final long serialVersionUID = 1L;
	/** Default name. */
	private static final String DEFAULTNAME = "myJdbcQuery";
	/**the id of this element.*/
	private static final String ID = "JDBC-QUERY";
	private static final String DATASOURCE = "Datasource";
	
	/**The name of the destination service property.*/
	public static final String NAMEDEST = "destination service";
	
	public static final String[] BOOLEAN_COMBO_VALUES = new String[]{"TRUE".intern(), "FALSE".intern()};
	private static String MS_SQL_SERVER = "MS_SQL";
	private static String MySQL_SERVER = "MySQL";
	private static String Oracle_SERVER = "Oracle";
	private static String Sybase_SERVER = "Sybase";
	private static String DB2_SERVER = "DB2";
	private static String Postgre_SQL_SERVER = "Postgre_SQL";	
	private static String[] supportedDatabases = {MS_SQL_SERVER, MySQL_SERVER, Oracle_SERVER, 
		Sybase_SERVER, DB2_SERVER, Postgre_SQL_SERVER};
	
	public static final String FAULT_MANAGEMENT = "FAULT_MANAGEMENT".intern();
	private static final String FAULT_FLOW = "FAULT_FLOW".intern();
	private static final String FAULT_JBI = "FAULT_JBI".intern();
	private static String[] faultManangementValues = {
		FAULT_FLOW, FAULT_JBI
	};
	private String matchRegexp = "\\$\\w+";
	
	/* Connection settings */
	public static String DRIVER = "DRIVER".intern();
	public static String JDBC_URL = "JDBC_URL".intern();
	public static String USER_NAME = "USER_NAME".intern();
	public static String USER_PASSWORD = "USER_PASSWORD".intern();
	public static String DATABASE_VENDOR = "DATABASE_VENDOR	".intern();
	
	/* Query settings */
	public static String USE_DATASOURCES = "USE_DATASOURCES".intern();
	public static String IS_STOREPROCEDURE = "IS_STOREPROCEDURE".intern();
	public static String QUERY = "QUERY".intern();
	public static String PLACE_HOLDER_SIGN = "$".intern();
	public static String PARAM_XPATH_PREFIX = "PARAM_XPATH_PREFIX".intern();
	public static String PARAM_TYPE_PREFIX = "PARAM_TYPE_PREFIX".intern();
	public static String PARAM_ISOUT_PREFIX = "PARAM_ISOUT_PREFIX".intern();
	public static String ENRICH_MESSAGE = "ENRICH_MESSAGE".intern();
	public static String XML_ENVELOPE = "XML_ENVELOPE".intern();
	public static String ROWS_XML_ENVELOPE = "ROWS_XML_ENVELOPE".intern();
	public static String ROW_XML_ENVELOPE = "ROW_XML_ENVELOPE".intern();
	public static String[] supportedInputParamTypes = {
		java.util.Date.class.getName(),
		java.lang.Double.class.getName(),
		java.lang.Float.class.getName(),
		java.lang.Integer.class.getName(),
        java.lang.Long.class.getName(),
        java.lang.Short.class.getName(),
        java.lang.String.class.getName(),
        "BINARY_BASE64_TYPE"
	};

	private Map<String, Integer> dataSourceIds = null;
	private Map<Integer,String > dataSourcePositions = null;
	
	/**
	 * default constructor.
	 */
	public GenericJdbcAdvancedQuery() {
		super(DEFAULTNAME, 1, DEFAULTOUTIN);
		//set the properties
		setProperty(NAME, DEFAULTNAME);
		
		setProperty(NAMEDEST, "");

		setProperty(USE_DATASOURCES, 0);
		setProperty(DRIVER, "");
		setProperty(JDBC_URL, "");
		setProperty(USER_NAME, "");
		setProperty(USER_PASSWORD, "");
		setProperty(DATABASE_VENDOR, 0);

		setProperty(IS_STOREPROCEDURE, 1);
		setProperty(QUERY, "");
		setProperty(ENRICH_MESSAGE, 1);
		setProperty(XML_ENVELOPE, "");
		setProperty(ROWS_XML_ENVELOPE, "rows");
		setProperty(ROW_XML_ENVELOPE, "row");
		setProperty(FAULT_MANAGEMENT, 1);
	}
	
	public void configureCustomComponent(){
		this.dataSourceIds = new HashMap<String, Integer>();
		this.dataSourcePositions = new HashMap<Integer, String>();
		
		Ressources ressource = new Ressources();
		String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/DataSources.xml");
		String first = "";
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			
			
			NodeList nodeList = ressource.getXPathNodeSet("/DATASOURCES/DATASOURCE");
			Node node = null;
			NamedNodeMap attributes = null;

			for (int i = 0; i < nodeList.getLength(); i++) {
				
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				dataSourceIds.put(attributes.getNamedItem("id").getNodeValue(), i);
				if (i == 0)
					first = attributes.getNamedItem("id").getNodeValue();
				dataSourcePositions.put(i, attributes.getNamedItem("id").getNodeValue());
			}
		}
		setProperty(DATASOURCE, first);
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	@Override
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the sync name cannot be null";
			returnResult = false;
		}
		
		if (getNameFirstDestination().compareTo("") == 0) {
			msg += "\n2 - Destination service => the destination of this "
						+ "element cannot be null!";
			returnResult = false;
		}
		String stringProperty = "";
		// connection configuration checks
		if (isUseDatasources()) {
			stringProperty = (String)getProperty(DATASOURCE); 
			if (stringProperty != null && stringProperty.trim().equals("")) {
				msg += "\n3 - Datasource => the datasource cannot be empty";
				returnResult = false;
			}
		} else {
			stringProperty = (String)getProperty(DRIVER); 
			if (stringProperty.trim().equals("")) {
				msg += "\n3 - Jdbc Driver => the jdbc driver cannot be empty";
				returnResult = false;
			}
	
			stringProperty = (String)getProperty(JDBC_URL);
			// TODO check if you can create a regexp that validate the jdbc URL
			if (stringProperty.trim().equals("")) {
				msg += "\n4 - Jdbc URL => the jdbc URL cannot be empty";
				returnResult = false;
			}
	
			stringProperty = (String)getProperty(USER_NAME); 
			if (stringProperty.trim().equals("")) {
				msg += "\n5 - Database username => the database username is empty";
			}
			
			stringProperty = (String)getProperty(USER_PASSWORD); 
			if (stringProperty.trim().equals("")) {
				msg += "\n6 - Database username => the database user password is empty";
			}
		}
		
		// query configuration checks
		stringProperty = (String)getProperty(QUERY); 
		if (stringProperty.trim().equals("")) {
			msg += "\n7 - Query => the query is empty";
			returnResult = false;
		}

		stringProperty = (String)getProperty(XML_ENVELOPE);
		if (isEnrichMessage() && stringProperty.trim().equals("")) {
			msg += "\n7 - XML envelop => Xml envelop cannot be empty if" +
					" Enrich message is true";
			returnResult = false;
		}
		if (isStoreProcedure() && stringProperty.trim().equals("")) {
			msg += "\n8 - XML envelop => Xml envelop cannot be empty if" +
					" Store procedure is true";
			returnResult = false;
		}

		// check every parameter
		List<String> paramNames = getQueryParameterNames();
		String name = "";
		Iterator<String> iter = paramNames.iterator();
		for (int idx = 9; iter.hasNext();idx++) {
			name = iter.next();
			if (!Boolean.parseBoolean(getParamIsOutput(name))) {
				stringProperty = getParamXpath(name);
				// TODO check if you can match the xpath with a regexp
				if (stringProperty.trim().equals("")) {
					msg += "\n"+idx+" - Parameter ["+name+"] XPath => " +
						"Parameter XPath cannot be empty";
					returnResult = false;
				}
				stringProperty = getParamType(name);
				// TODO check if you can validate the type with java.sql.Types
				if (stringProperty.trim().equals("")) {
					msg += "\n"+idx+" - Parameter ["+name+"] Type => " +
						"Parameter Type cannot be empty";
					returnResult = false;
				}
			}
		}
		
		if (msg.length() != 0) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	@Override
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, getString(indice++) + " - name");
		//addTextDescriptor(LOCATIONURI, getString(indice++)
		//		+ " - select the location URI of the WS");
		//addComboBoxDescriptor(SOAPREQUEST, getString(indice++) + " - soap request ? :",
		//				new String[]{"true", "false"});
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addComboBoxDescriptor(NAMEDEST, getString(indice++)
				+ " - select the destination service", lstStr);

		addComboBoxDescriptor(USE_DATASOURCES, getString(indice++)
				+ " - Use configured datasources",
				BOOLEAN_COMBO_VALUES);
		
		if (isUseDatasources()) {
			Ressources ressource = new Ressources();
			String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/DataSources.xml");
			String[] comboDs = null;
			if (fullPath != null) {
				
				if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
					fullPath = fullPath.substring("file:".length());
				}
			
				ressource.setDocument(fullPath);
				
				
				NodeList nodeList = ressource.getXPathNodeSet("/DATASOURCES/DATASOURCE");
				Node node = null;
				NamedNodeMap attributes = null;
				
				comboDs = new String[nodeList.getLength()];
				for (int i = 0; i < nodeList.getLength(); i++) {
					node = nodeList.item(i);				
					attributes = node.getAttributes();
					comboDs[i] = (String)attributes.getNamedItem("id").getNodeValue();
				}
			}
			addComboBoxDescriptor(DATASOURCE, getString(indice++) +" - DataSource", comboDs);
		} else {
			addTextDescriptor(DRIVER, getString(indice++)
					+ " - Jdbc driver class");
			addTextDescriptor(JDBC_URL, getString(indice++)
					+ " - Jdbc connection url");
			addTextDescriptor(USER_NAME, getString(indice++)
					+ " - Database user name");
			addPasswordTextDescriptor(USER_PASSWORD, getString(indice++)
					+ " - Database user password");
		}

		addComboBoxDescriptor(DATABASE_VENDOR, getString(indice++)
				+ " - Database vendor",
				supportedDatabases);

		addComboBoxDescriptor(IS_STOREPROCEDURE, getString(indice++)
				+ " - Is store procedure call",
				BOOLEAN_COMBO_VALUES);
		
		addTextDescriptor(QUERY, getString(indice++) + " - Query");
		
        List<String> queryParamNames = getQueryParameterNames();
        /*
         * adding properties at runtime we can add many properties that will
         * never be removed.
         */
        for (Iterator<String> iter = queryParamNames.iterator();
        	queryParamNames != null && iter.hasNext();) {
        	String name = iter.next();
			String category = "QueryParam - "+name;
			Integer isOutValue = (Integer)getProperty(PARAM_ISOUT_PREFIX+name);
			
			if (isStoreProcedure()) {
				setProperty(PARAM_ISOUT_PREFIX+name, isOutValue != null? isOutValue.intValue() : 1);
				ComboBoxPropertyDescriptor iDescr = new ComboBoxPropertyDescriptor(PARAM_ISOUT_PREFIX+name,
						"Is output parameter",
						BOOLEAN_COMBO_VALUES);
				iDescr.setCategory(category);
				addDescriptor(iDescr);
				isOutValue = (Integer)getProperty(PARAM_ISOUT_PREFIX+name);
			}
			
			if (isOutValue == null || isOutValue.intValue() == 1) {
				String previousValue = getParamXpath(name);
				setProperty(PARAM_XPATH_PREFIX+name, previousValue != null? previousValue : "");
				TextPropertyDescriptor pDescr = new TextPropertyDescriptor(
						PARAM_XPATH_PREFIX+name, "Parameter xpath expression"
					);
				pDescr.setCategory(category);
				addDescriptor(pDescr);
	
				ComboBoxPropertyDescriptor inParTypeDescr = new ComboBoxPropertyDescriptor(
						PARAM_TYPE_PREFIX+name, "Parameter type",
						supportedInputParamTypes
					);
				inParTypeDescr.setCategory(category);
				addDescriptor(inParTypeDescr);
				previousValue = getParamType(name);
				setProperty(PARAM_TYPE_PREFIX+name, getInputParamTypeIndex(previousValue));
			} else {
				setProperty(PARAM_XPATH_PREFIX+name, null);
				ComboBoxPropertyDescriptor sqltypeDescr =
					new ComboBoxPropertyDescriptor(PARAM_TYPE_PREFIX+name,
						"Parameter type",
						getSqlTypesLabels());
				sqltypeDescr.setCategory(category);
				addDescriptor(sqltypeDescr);
				Object previousValue = getProperty(PARAM_TYPE_PREFIX+name);
				setProperty(PARAM_TYPE_PREFIX+name,
					previousValue != null && previousValue instanceof Integer? previousValue : 0);
			}
		}
		
		IPropertyDescriptor pDescr = new ComboBoxPropertyDescriptor(ENRICH_MESSAGE, getString(indice++)
				+ " - Enrich input message",
				BOOLEAN_COMBO_VALUES);
		addDescriptor(pDescr);

		addTextDescriptor(XML_ENVELOPE, getString(indice++)
				+ " - Xml envelop");

		addTextDescriptor(ROWS_XML_ENVELOPE, getString(indice++)
				+ " - Rows Xml envelop");

		addTextDescriptor(ROW_XML_ENVELOPE, getString(indice++)
				+ " - Row Xml envelop");

		addComboBoxDescriptor(FAULT_MANAGEMENT, getString(indice++)
				+ " - Fault management",
				faultManangementValues);
	}
	
	private int getInputParamTypeIndex(String label) {
		for (int idx = 0; idx < supportedInputParamTypes.length; idx++) {
			if (supportedInputParamTypes[idx].equals(label)) {
				return idx;
			}
		}
		return supportedInputParamTypes.length > 0? 0:-1;
	}

	private String getString(int indice){
		if (indice >= 10)
			return String.valueOf(indice);
		else
			return "0" + String.valueOf(indice);
	}
	
	public boolean isUseDatasources() {
		Object useDatasourcesObj = getProperty(USE_DATASOURCES); 
		if (useDatasourcesObj != null) {
			return ((Integer)useDatasourcesObj) == 0;
		} else {
			setProperty(USE_DATASOURCES, 0);
		}
		return true;
	}	
	public boolean isStoreProcedure() {
		return ((Integer)getProperty(IS_STOREPROCEDURE)) == 0;
	}
	
	public boolean isEnrichMessage() {
		return ((Integer)getProperty(ENRICH_MESSAGE)) == 0;
	}	
	public String getParamIsOutput(String name) {
		Integer isOutput = (Integer)getProperty(PARAM_ISOUT_PREFIX+name);
		int intIsOutput = (isOutput != null)? isOutput.intValue() : 1;
		return BOOLEAN_COMBO_VALUES[intIsOutput];
	}
	
	public String getParamXpath(String name) {
		return (String)getProperty(PARAM_XPATH_PREFIX+name);
	}

	public String getParamType(String name) {
		Object paramType = getProperty(PARAM_TYPE_PREFIX+name);
		if (paramType == null) { return null; }
		if ( Boolean.parseBoolean(getParamIsOutput(name)) ) {
			String fieldName = getSqlTypesLabels()[((Integer)paramType).intValue()];
			Class sqlTypesClass = getSqlTypesClass();
			Integer value = null;
			try {
				value = (Integer)sqlTypesClass.getField(fieldName).get(value);
			} catch (Exception e) {
				e.printStackTrace();
				return null;
			}
			System.out.println("Output parameter type for ["+name+"]: "+value);
			return value.toString();
		} else {
			if (paramType instanceof Integer) {
				return supportedInputParamTypes[((Integer)paramType).intValue()];
			}
		}
		return (paramType instanceof String)? (String)paramType : paramType.toString();
	}

	public List<String> getQueryParameterNames() {
        Pattern pattern = Pattern.compile(matchRegexp);
        Matcher matcher = 
        	pattern.matcher((String)getProperty(QUERY));
        String placeHolder;
        List<String> toReturn = new ArrayList<String>();
		for (int idx = 1; matcher.find(); idx++) {
			placeHolder = matcher.group();
			placeHolder = placeHolder.substring(
				placeHolder.indexOf(PLACE_HOLDER_SIGN)+1);
			toReturn.add(placeHolder);
		}
		return toReturn;
	}
	
	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	@Override
	public final String getComponentId() {
		return ID;
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else if (DATASOURCE.equals(id)) {
			String propertyDs = (String)getProperty(DATASOURCE);
			if (propertyDs == null || propertyDs.trim().length() == 0){
				return 0;
			}else{
				return dataSourceIds.get(propertyDs);
			}
		} else if (ROWS_XML_ENVELOPE.equals(id)) {
			String propertyXmlRows = (String)getProperty(ROWS_XML_ENVELOPE);
			if (propertyXmlRows == null || propertyXmlRows.trim().length() == 0){
				setProperty(ROWS_XML_ENVELOPE, "rows");
				return "rows";
			}
		} else if (ROW_XML_ENVELOPE.equals(id)) {
			String propertyXmlRow = (String)getProperty(ROW_XML_ENVELOPE);
			if (propertyXmlRow == null || propertyXmlRow.trim().length() == 0){
				setProperty(ROW_XML_ENVELOPE, "row");
				return "row";
			}
		} else if (id == FAULT_MANAGEMENT){
			Object faultManagementProp = getProperty(FAULT_MANAGEMENT);
			if (faultManagementProp == null || !(faultManagementProp instanceof Integer)) {
				setProperty(FAULT_MANAGEMENT, 1);
				return 1;
			}
		}

		return super.getPropertyValue(id);
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		}
		if (id == USE_DATASOURCES){
			int intValue = ((Integer)value).intValue();
			if (intValue == 1) {
				setProperty(DATASOURCE, "");
			}
		}
		if (id == DATASOURCE){
			int intValue = ((Integer)value).intValue();
			setProperty(DATASOURCE, dataSourcePositions.get(intValue));
			return;
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public final void resetPropertyValue(final Object id) {
	}
	
	public String getIsEnrich() {
		int intEnrichMessage = ((Integer)getProperty(ENRICH_MESSAGE)).intValue();
		return BOOLEAN_COMBO_VALUES[intEnrichMessage];
	}
	
	public String getFaultManagement() {
		int intFaultManagement = ((Integer)getProperty(FAULT_MANAGEMENT)).intValue();
		return faultManangementValues[intFaultManagement];
	}
	
	public String getDatabaseVendor() {
		int intDatabaseVendor = ((Integer)getProperty(DATABASE_VENDOR)).intValue();
		return supportedDatabases[intDatabaseVendor];
	}
	
	private Class getSqlTypesClass() {
		String databaseVendor = getDatabaseVendor();
		if (Oracle_SERVER.equals(databaseVendor)) {
			try {
				return Class.forName("oracle.jdbc.OracleTypes");
			} catch (ClassNotFoundException e) {
				CimeroLog.warningLog(
					"Unable to load specific JDBC parameter Types default one will be returned", e);
				return Types.class;
			}
		} else {
			return Types.class;
		}
	}
	
	private String[] getSqlTypesLabels() {
		List<String> types = new ArrayList<String>();
		Field[] typesFields = getSqlTypesClass().getFields();
		
		for (int i = 0; i < typesFields.length; i++) {
			types.add(typesFields[i].getName());
		}
		
		return types.toArray(new String[types.size()]);
	}
	
	public int getSqlTypeValue(String sqlType) throws NoSuchFieldException, IllegalArgumentException, SecurityException, IllegalAccessException {
		Object value = Types.class.getField(sqlType).get(null);
		return ((Integer)value).intValue();
	}
	
	public final String getDataSource() {
		return (String) getProperty(DATASOURCE);
	}	
}
