package it.eng.components.cimero;

import org.eclipse.gef.palette.ToolEntry;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.FileUtils;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * The Generic Transformer
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class GenericXSDValidator extends ServiceEngineModel {

	/**serialization number.*/
	private static final long serialVersionUID = 1L;
	/**The xsl transformeur path.*/
	private static final String SCHEMA = "Schema";
	/**The name of the destination service property.*/
	private static final String NAMEDEST = "destination service";
	/**The name of the mep mode property.*/
	private static final String INOUTMEP = "in/out MEP";
	/**The default transformer name.*/
	private static final String DEFAULTNAME = "mySyntaxValidator";
	
	/**the id of this element.*/
	private static final String ID = "SYNTAXVALIDATOR";

	private static final String HANDLING_ERROR_METHOD = "handlingErrorMethod";
	/**
	 * The default constructor.
	 */
	public GenericXSDValidator() {
		super(DEFAULTNAME, 1 , DEFAULTOUTIN);
		setProperty(SCHEMA,"" );
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(INOUTMEP, 0);
		setProperty(HANDLING_ERROR_METHOD, 0);
		
	}
	
	public void configureCustomComponent(){
		
	}
	
	public String getHandlingErrorMethod(){
		Integer o = (Integer)getProperty(HANDLING_ERROR_METHOD);
		if (o == 0){
			return "APP";
		}else{
			return "JBI";
		}
	}
	/**
	 * Path getter function.
	 * @return String
	 */
	public final String getSchema() {
		return (String) getProperty(SCHEMA);
	}

	/**
	 * path setter function.
	 * @param pathParam : the path of the xslt file
	 */
	public final void setSchema(final String pathParam) {
		setProperty(SCHEMA, pathParam);
	}

	/**
	 * inoutMep getter function.
	 * @return true if this component is an inoutmep
	 */
	public final boolean isInOutMep() {
		return ((Integer) getProperty(INOUTMEP) == 0);
	}

	/**
	 * path setter function.
	 * @param inOutMepParam : in or out
	 */
	public final void setInOutMep(final boolean inOutMepParam) {
		if (inOutMepParam) {
			setProperty(INOUTMEP, 0);
		} else {
			setProperty(INOUTMEP, 1);
		}
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the name cannot be null";
			returnResult = false;
		}
		//xsl path verification
		if (getSchema().compareTo("") == 0) {
			msg += "\n2 - Schema => the schema";
			returnResult = false;
		} 
		
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + " \n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAME){
			System.out.println(id + "==> ["+value+"]");
		}
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else {
			return super.getProperty((String) id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public void resetPropertyValue(final Object id) {
		// do nothing
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	protected  void createDescriptor() {
		int indice = 1;
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addTextDescriptor(NAME, indice++ + " - name");
		addTextDescriptor(SCHEMA, indice++ + " - select the schema resource");
		addComboBoxDescriptor(HANDLING_ERROR_METHOD, indice++ + " - error handling ", new String[]{"FLOW", "JBI"});
		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);
	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
}
